<?php
/**
 * ISC License
 *
 * Copyright (c) 2024 idnovate.com
 * idnovate is a Registered Trademark & Property of idnovate.com, innovación y desarrollo SCP
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * @author    idnovate
 * @copyright 2024 idnovate
 * @license   https://www.isc.org/licenses/ https://opensource.org/licenses/ISC ISC License
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class CookiesPlusCookie extends ObjectModel
{
    public $id_cookiesplus_cookie;
    public $id_shop;
    public $active = 1;
    public $id_cookiesplus_finality;
    public $name;
    public $provider;
    public $provider_url;
    public $purpose;
    public $expiry;
    public $module_name = 'cookiesplus';
    public $date_add;
    public $date_upd;

    /**
     * @see ObjectModel::$definition
     */
    public static $definition = [
        'table' => 'cookiesplus_cookie',
        'primary' => 'id_cookiesplus_cookie',
        'multilang' => true,
        'fields' => [
            'id_cookiesplus_cookie' => ['type' => self::TYPE_INT, 'validate' => 'isUnsignedId', 'copy_post' => false],
            'id_shop' => ['type' => self::TYPE_INT, 'validate' => 'isUnsignedId', 'copy_post' => false],
            'active' => ['type' => self::TYPE_BOOL, 'validate' => 'isBool'],
            'id_cookiesplus_finality' => ['type' => self::TYPE_INT, 'validate' => 'isUnsignedId', 'required' => true],
            'name' => ['type' => self::TYPE_STRING, 'required' => true],
            'provider' => ['type' => self::TYPE_STRING],
            'provider_url' => ['type' => self::TYPE_STRING],
            'purpose' => ['type' => self::TYPE_STRING, 'validate' => 'isAnything', 'lang' => true],
            'expiry' => ['type' => self::TYPE_STRING, 'validate' => 'isAnything', 'lang' => true],
            'date_add' => ['type' => self::TYPE_DATE, 'copy_post' => false],
            'date_upd' => ['type' => self::TYPE_DATE, 'copy_post' => false],
        ],
    ];

    public function __construct($id = null, $id_lang = null, $id_shop = null, $translator = null)
    {
        global $_FIELDS;

        $module = Module::getInstanceByName($this->module_name);

        $_FIELDS[get_class($this) . '_' . md5('id_cookiesplus_finality')] = $module->l('Cookie finality');
        $_FIELDS[get_class($this) . '_' . md5('name')] = $module->l('Cookie name');

        parent::__construct($id, $id_lang, $id_shop, $translator);
    }

    public function add($autodate = true, $null_values = false)
    {
        $this->id_shop = ($this->id_shop) ?: Context::getContext()->shop->id;

        return parent::add($autodate, $null_values);
    }

    public function update($null_values = false)
    {
        $result = parent::update($null_values);

        if (Tools::getValue('back') && (int) Tools::getvalue('id_cookiesplus_finality')) {
            Tools::redirectAdmin(Context::getContext()->link->getAdminLink('AdminCookiesPlusFinalities') . '&updatecookiesplus_finality=&conf=4&id_cookiesplus_finality=' . (int) Tools::getvalue('id_cookiesplus_finality'));
        }

        return $result;
    }

    public static function getCookiesPlusCookies($id_cookiesplus_finality, $id_lang = null, $only_active = false, $id_shop = null)
    {
        if ($id_shop === null) {
            $id_shop = (int) Context::getContext()->shop->id;
        }

        $query = '
            SELECT *
            FROM ' . _DB_PREFIX_ . 'cookiesplus_cookie cc '
            . 'LEFT JOIN ' . _DB_PREFIX_ . 'cookiesplus_cookie_lang ccl on cc.`id_cookiesplus_cookie` = ccl.`id_cookiesplus_cookie`
            WHERE
                `id_cookiesplus_finality` = ' . (int) $id_cookiesplus_finality . '
                AND ccl.`id_lang` = ' . ($id_lang ? (int) $id_lang : (int) Context::getContext()->language->id) .
                ($only_active ? ' AND cc.`active` = 1' : '') .
                ' AND cc.`id_shop` =  ' . $id_shop .
            ' ORDER BY cc.`name`;
        ';

        return Db::getInstance()->executeS($query);
    }

    public static function getDefaultValues($cookiesPlusFinality)
    {
        /*
         array(
                'active' => 0,
                'modules' => array(),
                'name' => '',
                'provider' => Tools::getShopDomainSsl(true),
                'provider_url' => '',
                'purpose' => array(
                    'en' => '',
                    'es' => '',
                    'ag' => '',
                    'cb' => '',
                    'mx' => '',
                    'fr' => '',
                    'qc' => '',
                    'pl' => '',
                    'ro' => '',
                    'pt' => '',
                    'br' => '',
                    'sk' => '',
                    'nl' => '',
                    'de' => '',
                    'gr' => '',
                    'it' => '',
                    'si' => '',
                    'da' => '',
                    'no' => '',
                    'cs' => '',
                    'hu' => '',
                ),
                'expiry' => array(
                    'en' => '',
                    'es' => '',
                    'ag' => '',
                    'cb' => '',
                    'mx' => '',
                    'fr' => '',
                    'qc' => '',
                    'pl' => '',
                    'ro' => '',
                    'pt' => '',
                    'br' => '',
                    'sk' => '',
                    'nl' => '',
                    'de' => '',
                    'gr' => '',
                    'it' => '',
                    'si' => '',
                    'da' => '',
                    'no' => '',
                    'cs' => '',
                    'hu' => '',
                    'sv' => '',
                )
            ),
         */
        $cookiesPlusCookieDefaultValues = [
            CookiesPlusFinality::NECESSARY_COOKIE => [
                [
                    'active' => 1,
                    'name' => 'cookiesplus',
                    'provider' => Tools::getShopDomainSsl(true),
                    'provider_url' => '',
                    'purpose' => [
                        'en' => 'Stores your cookie preferences.',
                        'es' => 'Almacena las preferencias sobre cookies.',
                        'ag' => 'Almacena las preferencias sobre cookies.',
                        'cb' => 'Almacena las preferencias sobre cookies.',
                        'mx' => 'Almacena las preferencias sobre cookies.',
                        'fr' => 'Conserver vos préférences en matière de cookies.',
                        'qc' => 'Conserver vos préférences en matière de cookies.',
                        'pl' => 'Zapamiętuje preferencje dotyczące plików cookie.',
                        'ro' => 'Reține preferințele dvs. legate de modulele cookie.',
                        'pt' => 'Guarda as suas preferências quanto aos cookies.',
                        'br' => 'Guarda as suas preferências quanto aos cookies.',
                        'sk' => 'ukladá vaše preferencie týkajúce sa súborov cookie.',
                        'nl' => 'Slaat uw cookie-voorkeuren op.',
                        'de' => 'Speichert Ihre Cookie-Einstellungen.',
                        'gr' => 'Αποθηκεύει τις προτιμήσεις σας για τα cookies.',
                        'it' => 'Ricorda le tue preferenze in fatto di cookie.',
                        'si' => 'shranjuje vaše nastavitve piškotkov..',
                        'da' => 'Gemmer dine cookiepræferencer.',
                        'no' => 'Lagrer informasjonskapselpreferansene dine.',
                        'cs' => 'ukládá vaše preference týkající se cookies.',
                        'hu' => 'A cookie-kkal kapcsolatos beállításokat tárolja.',
                        'sv' => 'Sparar dina inställningar för kakor.',
                    ],
                    'expiry' => [
                        'en' => '1 year',
                        'es' => '1 año',
                        'ag' => '1 año',
                        'cb' => '1 año',
                        'mx' => '1 año',
                        'fr' => '1 année',
                        'qc' => '1 année',
                        'pl' => '1 rok',
                        'ro' => '1 an',
                        'pt' => '1 ano',
                        'br' => '1 ano',
                        'sk' => '1 rok',
                        'nl' => '1 jaar',
                        'de' => '1 Jahr',
                        'gr' => '1 χρόνος',
                        'it' => '1 anno',
                        'si' => '1 leto',
                        'da' => '1 år',
                        'no' => '1 år',
                        'cs' => '1 rok',
                        'hu' => '1 év',
                        'sv' => '1 år',
                    ],
                ],
                [
                    'active' => 0,
                    'name' => 'PHP_SESSID',
                    'provider' => Tools::getShopDomainSsl(true),
                    'provider_url' => '',
                    'purpose' => [
                        'en' => 'This cookie is native to PHP and enables websites to store serialised state data. It is used to establish a user session and to pass state data via a temporary cookie, which is commonly referred to as a session cookie.',
                        'es' => 'Esta cookie es nativa de PHP y permite a los sitios web almacenar datos de estado serializados. Se utiliza para establecer una sesión de usuario y para pasar datos de estado a través de una cookie temporal, que comúnmente se conoce como cookie de sesión.',
                        'ag' => 'Esta cookie es nativa de PHP y permite a los sitios web almacenar datos de estado serializados. Se utiliza para establecer una sesión de usuario y para pasar datos de estado a través de una cookie temporal, que comúnmente se conoce como cookie de sesión.',
                        'cb' => 'Esta cookie es nativa de PHP y permite a los sitios web almacenar datos de estado serializados. Se utiliza para establecer una sesión de usuario y para pasar datos de estado a través de una cookie temporal, que comúnmente se conoce como cookie de sesión.',
                        'mx' => 'Esta cookie es nativa de PHP y permite a los sitios web almacenar datos de estado serializados. Se utiliza para establecer una sesión de usuario y para pasar datos de estado a través de una cookie temporal, que comúnmente se conoce como cookie de sesión.',
                        'fr' => 'Ce cookie est natif de PHP et permet aux sites Web de stocker des données d\'état sérialisées. Il est utilisé pour établir une session utilisateur et pour transmettre des données d\'état via un cookie temporaire, couramment appelé cookie de session.',
                        'qc' => 'Ce cookie est natif de PHP et permet aux sites Web de stocker des données d\'état sérialisées. Il est utilisé pour établir une session utilisateur et pour transmettre des données d\'état via un cookie temporaire, couramment appelé cookie de session.',
                        'pl' => 'Ten plik cookie pochodzi z języka PHP i umożliwia witrynom przechowywanie zserializowanych danych stanu. Służy do ustanowienia sesji użytkownika i przekazywania danych o stanie za pośrednictwem tymczasowego pliku cookie, który jest powszechnie nazywany plikiem cookie sesji.',
                        'ro' => 'Acest cookie este originar din PHP și permite site-urilor web să stocheze date de stare serializate. Este utilizat pentru a stabili o sesiune de utilizator și pentru a transmite date de stare printr-un cookie temporar, care este denumit în mod obișnuit un cookie de sesiune.',
                        'pt' => 'Este cookie é nativo do PHP e permite que sites armazenem dados de estado serializados. Ele é usado para estabelecer uma sessão de usuário e para passar dados de estado por meio de um cookie temporário, comumente referido como um cookie de sessão.',
                        'br' => 'Este cookie é nativo do PHP e permite que sites armazenem dados de estado serializados. Ele é usado para estabelecer uma sessão de usuário e para passar dados de estado por meio de um cookie temporário, comumente referido como um cookie de sessão.',
                        'sk' => 'Tento súbor cookie je pôvodom z jazyka PHP a umožňuje webovým serverom ukladať údaje o sériovom stave. Používa sa na vytvorenie relácie používateľa a na odovzdanie údajov o stave prostredníctvom dočasného súboru cookie, ktorý sa bežne označuje ako súbor cookie relácie.',
                        'nl' => 'Deze cookie is eigen aan PHP en stelt websites in staat om geserialiseerde statusgegevens op te slaan. Het wordt gebruikt om een gebruikerssessie tot stand te brengen en om statusgegevens door te geven via een tijdelijke cookie, die gewoonlijk een sessiecookie wordt genoemd.',
                        'de' => 'Dieses Cookie stammt ursprünglich aus PHP und ermöglicht es Websites, serialisierte Statusdaten zu speichern. Es wird verwendet, um eine Benutzersitzung einzurichten und Statusdaten über ein temporäres Cookie zu übergeben, das üblicherweise als Sitzungscookie bezeichnet wird.',
                        'gr' => 'Αυτό το cookie είναι εγγενές στην PHP και επιτρέπει στους ιστότοπους να αποθηκεύουν σειριακά δεδομένα κατάστασης. Χρησιμοποιείται για τον καθορισμό μιας περιόδου λειτουργίας χρήστη και για τη διαβίβαση δεδομένων κατάστασης μέσω ενός προσωρινού cookie, το οποίο συνήθως αναφέρεται ως cookie περιόδου λειτουργίας.',
                        'it' => 'Questo cookie è nativo di PHP e consente ai siti Web di memorizzare dati di stato serializzati. Viene utilizzato per stabilire una sessione utente e per trasmettere i dati sullo stato tramite un cookie temporaneo, comunemente denominato cookie di sessione.',
                        'si' => 'Ta piškotek je izvorni jezik PHP in spletnim mestom omogoča shranjevanje serializiranih podatkov o državi. Uporablja se za vzpostavitev uporabniške seje in posredovanje podatkov o stanju prek začasnega piškotka, ki ga običajno imenujemo piškotek seje.',
                        'da' => 'Denne cookie er hjemmehørende i PHP og gør det muligt for websteder at gemme serielle statusdata. Det bruges til at etablere en brugersession og til at overføre tilstandsdata via en midlertidig cookie, der almindeligvis kaldes en session-cookie.',
                        'no' => 'Denne informasjonskapselen er innfødt i PHP og gjør det mulig for nettsteder å lagre serielle tilstandsdata. Den brukes til å etablere en brukersession og til å overføre tilstandsdata via en midlertidig informasjonskapsel, som ofte blir referert til som en økt-informasjonskapsel.',
                        'cs' => 'Tento soubor cookie je původem z PHP a umožňuje webovým serverům ukládat údaje o stavu serializace. Používá se k navázání relace uživatele a k předávání údajů o stavu prostřednictvím dočasného souboru cookie, který se běžně označuje jako soubor cookie relace.',
                        'hu' => 'Ez a süti natív a PHP-ben, és lehetővé teszi a webhelyek számára a sorosított állapotadatok tárolását. Felhasználói munkamenet létrehozására és állapotadatok átadására szolgál ideiglenes cookie-n keresztül, amelyet általában munkamenet-süti néven emlegetnek.',
                        'sv' => '',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    'active' => 1,
                    'name' => 'PrestaShop-#',
                    'provider' => Tools::getShopDomainSsl(true),
                    'provider_url' => '',
                    'purpose' => [
                        'en' => 'This cookie helps keep user sessions open while they are visiting a website, and help them make orders and many more operations such as: cookie add date, selected language, used currency, last product category visited, last seen products, client identification, name, first name, encrypted password, email linked to the account, shopping cart identification.',
                        'es' => 'Esta cookie permite conservar abiertas las sesiones del usuario durante su visita y le permite pasar un pedido o toda una serie de funcionamientos como: fecha de adición de la cookie, idioma seleccionado, divisa utilizada, última categoría de producto visitado, productos recientemente vistos, acuerdo de utilización de servicios del sitio, identificador del cliente, identificador de conexión, apellido, nombre, estado conectado, su contraseña cifrada, e-mail relacionado con la cuenta del cliente y el identificador del carrito.',
                        'ag' => 'Esta cookie permite conservar abiertas las sesiones del usuario durante su visita y le permite pasar un pedido o toda una serie de funcionamientos como: fecha de adición de la cookie, idioma seleccionado, divisa utilizada, última categoría de producto visitado, productos recientemente vistos, acuerdo de utilización de servicios del sitio, identificador del cliente, identificador de conexión, apellido, nombre, estado conectado, su contraseña cifrada, e-mail relacionado con la cuenta del cliente y el identificador del carrito.',
                        'cb' => 'Esta cookie permite conservar abiertas las sesiones del usuario durante su visita y le permite pasar un pedido o toda una serie de funcionamientos como: fecha de adición de la cookie, idioma seleccionado, divisa utilizada, última categoría de producto visitado, productos recientemente vistos, acuerdo de utilización de servicios del sitio, identificador del cliente, identificador de conexión, apellido, nombre, estado conectado, su contraseña cifrada, e-mail relacionado con la cuenta del cliente y el identificador del carrito.',
                        'mx' => 'Esta cookie permite conservar abiertas las sesiones del usuario durante su visita y le permite pasar un pedido o toda una serie de funcionamientos como: fecha de adición de la cookie, idioma seleccionado, divisa utilizada, última categoría de producto visitado, productos recientemente vistos, acuerdo de utilización de servicios del sitio, identificador del cliente, identificador de conexión, apellido, nombre, estado conectado, su contraseña cifrada, e-mail relacionado con la cuenta del cliente y el identificador del carrito.',
                        'fr' => 'Ce cookie permet de garder les sessions de l\'utilisateur ouvertes pendant leur visite, et lui permettre de passer commande ou tout un ensemble de fonctionnement tels que : date d\'ajout du cookie, langue sélectionnée, devise utilisée, dernière catégorie de produit visité, produits récemment vus, accord d\'utilisation de services du site, Identifiant client, identifiant de connexion, nom, prénom, état connecté, votre mot de passe chiffré, e-mail lié au compte client, l\'identifiant du panier.',
                        'qc' => 'Ce cookie permet de garder les sessions de l\'utilisateur ouvertes pendant leur visite, et lui permettre de passer commande ou tout un ensemble de fonctionnement tels que : date d\'ajout du cookie, langue sélectionnée, devise utilisée, dernière catégorie de produit visité, produits récemment vus, accord d\'utilisation de services du site, Identifiant client, identifiant de connexion, nom, prénom, état connecté, votre mot de passe chiffré, e-mail lié au compte client, l\'identifiant du panier.',
                        'pl' => 'Ten plik cookie pomaga utrzymać otwarte sesje użytkownika podczas odwiedzania strony internetowej i pomaga im w składaniu zamówień i wielu innych operacjach, takich jak: data dodania pliku cookie, wybrany język, używana waluta, ostatnia odwiedzana kategoria produktów, ostatnio oglądane produkty, identyfikacja klienta, imię, imię, zaszyfrowane hasło, adres e-mail powiązany z kontem, identyfikacja koszyka.',
                        'ro' => 'Acest cookie ajută la menținerea sesiunilor utilizatorului deschise în timp ce vizitează un site web și îi ajută să facă comenzi și multe alte operațiuni, cum ar fi: data adăugării cookie-ului, limba selectată, moneda utilizată, ultima categorie de produse vizitate, ultimele produse văzute, identificarea clientului, numele, prenume, parolă criptată, e-mail conectat la cont, identificare coș de cumpărături.',
                        'pt' => 'Este cookie ajuda a manter as sessões do usuário abertas enquanto eles estão visitando um site, e os ajuda a fazer pedidos e muitas outras operações, como: data de adição do cookie, idioma selecionado, moeda usada, última categoria de produto visitada, produtos vistos pela última vez, identificação do cliente, nome, nome, senha criptografada, e-mail vinculado à conta, identificação do carrinho de compras.',
                        'br' => 'Este cookie ajuda a manter as sessões do usuário abertas enquanto eles estão visitando um site, e os ajuda a fazer pedidos e muitas outras operações, como: data de adição do cookie, idioma selecionado, moeda usada, última categoria de produto visitada, produtos vistos pela última vez, identificação do cliente, nome, nome, senha criptografada, e-mail vinculado à conta, identificação do carrinho de compras.',
                        'sk' => 'Tento súbor cookie pomáha udržiavať relácie používateľa otvorené pri návšteve webových stránok a pomáha im pri objednávaní a mnohých ďalších operáciách, ako napríklad: dátum pridania súboru cookie, zvolený jazyk, použitá mena, posledná navštívená kategória produktu, naposledy zobrazené produkty, identifikácia klienta, meno, krstné meno, šifrované heslo, e-mail prepojený s účtom, identifikácia nákupného košíka.',
                        'nl' => 'Deze cookie helpt gebruikerssessies open te houden terwijl ze een website bezoeken, en helpt hen bij het plaatsen van bestellingen en veel meer bewerkingen zoals: datum toevoegen van cookies, geselecteerde taal, gebruikte valuta, laatst bezochte productcategorie, laatst geziene producten, klantidentificatie, naam, voornaam, gecodeerd wachtwoord, e-mailadres dat aan het account is gekoppeld, identificatie van het winkelwagentje.',
                        'de' => 'Mit diesem Cookie können Benutzersitzungen während des Besuchs einer Website geöffnet bleiben und Bestellungen und viele weitere Vorgänge ausführen, z. B.: Datum des Hinzufügens des Cookies, ausgewählte Sprache, verwendete Währung, zuletzt besuchte Produktkategorie, zuletzt gesehene Produkte, Kundenidentifikation, Name, Vorname, verschlüsseltes Passwort, mit dem Konto verknüpfte E-Mail, Warenkorbidentifikation.',
                        'gr' => 'Αυτό το cookie βοηθά στη διατήρηση ανοικτών περιόδων σύνδεσης χρηστών κατά την επίσκεψή τους σε έναν ιστότοπο και τους βοηθά να κάνουν παραγγελίες και πολλές άλλες λειτουργίες όπως: ημερομηνία προσθήκης cookie, επιλεγμένη γλώσσα, χρησιμοποιημένο νόμισμα, τελευταία κατηγορία προϊόντων που επισκεφτήκατε, προϊόντα που είδατε τελευταία, αναγνώριση πελάτη, όνομα όνομα, κρυπτογραφημένος κωδικός πρόσβασης, διεύθυνση ηλεκτρονικού ταχυδρομείου που συνδέεται με τον λογαριασμό, αναγνώριση καλαθιού αγορών.',
                        'it' => 'Questo cookie aiuta a mantenere aperte le sessioni dell\'utente mentre sta visitando un sito web e lo aiuta a effettuare ordini e molte altre operazioni come: data di aggiunta del cookie, lingua selezionata, valuta utilizzata, ultima categoria di prodotto visitata, prodotti visti per ultimi, identificazione del cliente, nome, nome, password crittografata, e-mail collegata all\'account, identificazione del carrello.',
                        'si' => 'Ta piškotek pomaga, da uporabniške seje ostanejo odprte med obiskom spletnega mesta, in jim pomaga pri naročanju ter številnih drugih operacijah, kot so: datum dodajanja piškotkov, izbrani jezik, uporabljena valuta, zadnja obiskana kategorija izdelkov, zadnji vidni izdelki, identifikacija stranke, ime, ime, šifrirano geslo, e-pošta, povezana z računom, identifikacija košarice.',
                        'da' => 'Denne cookie hjælper med at holde brugersessioner åbne, mens de besøger et websted, og hjælper dem med at foretage ordrer og mange flere handlinger såsom: tilføjelsesdato for cookie, valgt sprog, brugt valuta, sidst besøgte produktkategori, sidst set produkter, klientidentifikation, navn, fornavn, krypteret adgangskode, e-mail knyttet til kontoen, indkøbskurvidentifikation.',
                        'no' => 'Denne informasjonskapselen hjelper med å holde brukersesjonene åpne mens de besøker et nettsted, og hjelper dem med å bestille og mange flere operasjoner, for eksempel: informasjonskapsel legge til dato, valgt språk, brukt valuta, sist besøkte produktkategori, sist sett produkter, klientidentifikasjon, navn, fornavn, kryptert passord, e-post tilknyttet kontoen, handlekurvidentifikasjon.',
                        'cs' => 'Tento soubor cookie pomáhá udržovat relace uživatelů otevřené, když navštěvují web, a pomáhá jim provádět objednávky a mnoho dalších operací, jako jsou: datum přidání souboru cookie, vybraný jazyk, použitá měna, poslední navštívená kategorie produktů, naposledy viděné produkty, identifikace klienta, jméno, křestní jméno, šifrované heslo, e-mail propojený s účtem, identifikace nákupního košíku.',
                        'hu' => 'Ez a süti segít nyitva tartani a felhasználói munkameneteket, amíg ők egy webhelyet látogatnak, és segít nekik megrendeléseket és még sok más műveletet végrehajtani, például: süti hozzáadásának dátuma, kiválasztott nyelv, használt pénznem, utoljára látogatott termékkategória, utoljára látott termékek, kliens azonosító, név, keresztnév, titkosított jelszó, a fiókhoz kapcsolódó e-mail, kosár azonosító.',
                        'sv' => 'Denna cookie hjälper till att hålla användarsessioner öppna medan de besöker en webbplats och hjälper dem att göra beställningar och många fler operationer, såsom: cookie-tilläggsdatum, valt språk, använd valuta, senast besökta produktkategori, senast sett produkter, klientidentifiering, namn, förnamn, krypterat lösenord, e-post länkad till kontot, identifiering av kundvagn.',
                    ],
                    'expiry' => [
                        'en' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' hours',
                        'es' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' horas',
                        'ag' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' horas',
                        'cb' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' horas',
                        'mx' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' horas',
                        'fr' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' heures',
                        'qc' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' heures',
                        'pl' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' godziny',
                        'ro' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' ore',
                        'pt' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' horas',
                        'br' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' horas',
                        'sk' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' hodín',
                        'nl' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' uren',
                        'de' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' Std',
                        'gr' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' ώρες',
                        'it' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' ore',
                        'si' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' ure',
                        'da' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' timer',
                        'no' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' timer',
                        'cs' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' hodin',
                        'hu' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' órák',
                        'hu' => Configuration::get('PS_COOKIE_LIFETIME_FO') . ' timmar',
                    ],
                ],
                [
                    /* Crisp chat */
                    'active' => 0,
                    'modules' => [],
                    'name' => '__cfduid',
                    'provider' => '',
                    'provider_url' => '',
                    'purpose' => [
                        'en' => 'Technical cookies required by CDN provider Cloudflare.',
                        'es' => 'Cookies técnicas requeridas por el proveedor de CDN Cloudflare.',
                        'ag' => 'Cookies técnicas requeridas por el proveedor de CDN Cloudflare.',
                        'cb' => 'Cookies técnicas requeridas por el proveedor de CDN Cloudflare.',
                        'mx' => 'Cookies técnicas requeridas por el proveedor de CDN Cloudflare.',
                        'fr' => 'Cookies techniques requis par le fournisseur de CDN Cloudflare.',
                        'qc' => 'Cookies techniques requis par le fournisseur de CDN Cloudflare.',
                        'pl' => 'Techniczne pliki cookie wymagane przez dostawcę CDN Cloudflare.',
                        'ro' => 'Cookie-uri tehnice solicitate de furnizorul CDN Cloudflare.',
                        'pt' => 'Cookies técnicos exigidos pelo provedor de CDN Cloudflare.',
                        'br' => 'Cookies técnicos exigidos pelo provedor de CDN Cloudflare.',
                        'sk' => 'Technické súbory cookie požadované poskytovateľom CDN Cloudflare.',
                        'nl' => 'Technische cookies vereist door CDN-provider Cloudflare.',
                        'de' => 'Technische Cookies, die vom CDN-Anbieter Cloudflare.',
                        'gr' => 'Απαιτούνται τεχνικά cookie από τον πάροχο CDN Cloudflare.',
                        'it' => 'Cookie tecnici richiesti dal provider di CDN Cloudflare.',
                        'si' => 'Tehnični piškotki, ki jih potrebuje ponudnik CDN Cloudflare.',
                        'da' => 'Tekniske cookies krævet af CDN-udbyder Cloudflare.',
                        'no' => 'Tekniske informasjonskapsler som kreves av CDN-leverandøren Cloudflare.',
                        'cs' => 'Technické soubory cookie vyžadované poskytovatelem CDN Cloudflare.',
                        'hu' => 'A Cloudflare CDN-szolgáltató által megkövetelt technikai cookie-k annak biztosítására.',
                        'sv' => 'Tekniska kakor krävs av CDN-leverantören Cloudflare.',
                    ],
                    'expiry' => [
                        'en' => '30 days',
                        'es' => '30 días',
                        'ag' => '30 días',
                        'cb' => '30 días',
                        'mx' => '30 días',
                        'fr' => '30 jours',
                        'qc' => '30 jours',
                        'pl' => '30 dni',
                        'ro' => '30 de zile',
                        'pt' => '30 dias',
                        'br' => '30 dias',
                        'sk' => '30 dní',
                        'nl' => '30 dagen',
                        'de' => '30 Tage',
                        'gr' => '30 μέρες',
                        'it' => '30 giorni',
                        'si' => '30 dni',
                        'da' => '30 dage',
                        'no' => '30 dager',
                        'cs' => '30 dager',
                        'hu' => '30 nap',
                        'sv' => '30 dagar',
                    ],
                ],
                [
                    /* Crisp chat */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'crisp-client/session',
                    'provider' => 'Crisp Chat',
                    'provider_url' => 'https://help.crisp.chat/en/article/crisp-chatbox-cookie-and-ip-policy-1147xor/',
                    'purpose' => [
                        'en' => 'Used to make the Crisp Chat consistant when going to your website. Without this cookie, the customer would have a new chat session everytime he switchs to a new page.',
                        'es' => 'Se utiliza para hacer que Crisp Chat sea coherente al visitar su sitio web Sin esta cookie, el cliente tendría una nueva sesión de chat cada vez que cambia a una nueva página.',
                        'ag' => 'Se utiliza para hacer que Crisp Chat sea coherente al visitar su sitio web Sin esta cookie, el cliente tendría una nueva sesión de chat cada vez que cambia a una nueva página.',
                        'cb' => 'Se utiliza para hacer que Crisp Chat sea coherente al visitar su sitio web Sin esta cookie, el cliente tendría una nueva sesión de chat cada vez que cambia a una nueva página.',
                        'mx' => 'Se utiliza para hacer que Crisp Chat sea coherente al visitar su sitio web Sin esta cookie, el cliente tendría una nueva sesión de chat cada vez que cambia a una nueva página.',
                        'fr' => 'Utilisé pour rendre le Crisp Chat cohérent lorsque vous accédez à votre site Web. Sans ce cookie, le client aurait une nouvelle session de chat à chaque fois qu\'il passe à une nouvelle page.',
                        'qc' => 'Utilisé pour rendre le Crisp Chat cohérent lorsque vous accédez à votre site Web. Sans ce cookie, le client aurait une nouvelle session de chat à chaque fois qu\'il passe à une nouvelle page.',
                        'pl' => 'Służy do zapewnienia spójności Crisp Chat podczas odwiedzania Twojej witryny. Bez tego pliku cookie klient miałby nową sesję czatu za każdym razem, gdy przełączy się na nową stronę.',
                        'ro' => 'Folosit pentru a face Crisp Chat consecvent atunci când accesați site-ul dvs. web. Fără acest cookie, clientul ar avea o nouă sesiune de chat de fiecare dată când trece la o pagină nouă.',
                        'pt' => 'Usado para tornar o Crisp Chat consistente ao acessar seu site. Sem este cookie, o cliente teria uma nova sessão de chat cada vez que mudasse para uma nova página.',
                        'br' => 'Usado para tornar o Crisp Chat consistente ao acessar seu site. Sem este cookie, o cliente teria uma nova sessão de chat cada vez que mudasse para uma nova página.',
                        'sk' => 'Používa sa na zabezpečenie konzistencie Crisp Chat, keď idete na svoj web. Bez tohto súboru cookie by zákazník mal novú chatovú reláciu zakaždým, keď prejde na novú stránku.',
                        'nl' => 'Wordt gebruikt om de Crisp Chat consistent te maken wanneer u naar uw website gaat. Zonder deze cookie zou de klant een nieuwe chatsessie hebben elke keer dat hij naar een nieuwe pagina overschakelt.',
                        'de' => 'Wird verwendet, um den Crisp Chat beim Aufrufen Ihrer Website konsistent zu machen. Ohne dieses Cookie würde der Kunde jedes Mal eine neue Chat-Sitzung haben, wenn er zu einer neuen Seite wechselt.',
                        'gr' => 'Χρησιμοποιήθηκε για να κάνει το Crisp Chat συνεπές όταν μεταβαίνετε στον ιστότοπό σας. Χωρίς αυτό το cookie, ο πελάτης θα έχει μια νέα συνεδρία συνομιλίας κάθε φορά που αλλάζει σε μια νέα σελίδα.',
                        'it' => 'Utilizzato per rendere coerente Crisp Chat quando si visita il tuo sito web. Senza questo cookie, il cliente avrebbe una nuova sessione di chat ogni volta che passa a una nuova pagina.',
                        'si' => 'Uporablja se za uskladitev + jasnega + klepeta pri obisku vašega spletnega mesta. Brez tega piškotka bi imel kupec novo sejo klepeta vsakič, ko preklopi na novo stran.',
                        'da' => 'Bruges til at gøre Crisp Chat konsekvent, når du går til dit websted. Uden denne cookie ville kunden have en ny chat-session hver gang han skifter til en ny side.',
                        'no' => 'Brukes til å gjøre Crisp Chat konsekvent når du går til nettstedet ditt. Uten denne informasjonskapselen ville kunden ha en ny chat-økt hver gang han bytter til en ny side.',
                        'cs' => 'Slouží k zajištění konzistence Crisp Chat při přechodu na váš web. Bez tohoto cookie by zákazník měl novou relaci chatu pokaždé, když přepne na novou stránku.',
                        'hu' => 'Arra szolgál, hogy a Crisp Chat konzisztens legyen, amikor webhelyére megy. E süti nélkül az ügyfél minden alkalommal új csevegést folytat, amikor új oldalra vált.',
                        'sv' => 'Används för att göra Crisp Chat konsekvent när du går till din webbplats. Utan denna cookie skulle kunden ha en ny chattperiod varje gång han byter till en ny sida.',
                    ],
                    'expiry' => [
                        'en' => '6 months',
                        'es' => '6 meses',
                        'ag' => '6 meses',
                        'cb' => '6 meses',
                        'mx' => '6 meses',
                        'fr' => '6 mois',
                        'qc' => '6 mois',
                        'pl' => '6 miesiące',
                        'ro' => '6 luni',
                        'pt' => '6 meses',
                        'br' => '6 meses',
                        'sk' => '6 mesiace',
                        'nl' => '6 maanden',
                        'de' => '6 Monate',
                        'gr' => '6 μήνες',
                        'it' => '6 mesi',
                        'si' => '6 meseca',
                        'da' => '6 mdr.',
                        'no' => '6 måneder',
                        'cs' => '6 měsíců',
                        'hu' => '6 hónap',
                        'sv' => '6 månader',
                    ],
                ],
                [
                    /* Youtube */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'rc::a',
                    'provider' => 'Google',
                    'provider_url' => 'https://policies.google.com/privacy',
                    'purpose' => [
                        'en' => 'This cookie is used to distinguish between humans and bots. This is beneficial for the website, in order to make valid reports on the use of their website.',
                        'es' => 'Esta cookie se utiliza para distinguir entre humanos y bots. Esto es beneficioso para la web con el objeto de elaborar informes válidos sobre el uso de su web.',
                        'ag' => 'Esta cookie se utiliza para distinguir entre humanos y bots. Esto es beneficioso para la web con el objeto de elaborar informes válidos sobre el uso de su web.',
                        'cb' => 'Esta cookie se utiliza para distinguir entre humanos y bots. Esto es beneficioso para la web con el objeto de elaborar informes válidos sobre el uso de su web.',
                        'mx' => 'Esta cookie se utiliza para distinguir entre humanos y bots. Esto es beneficioso para la web con el objeto de elaborar informes válidos sobre el uso de su web.',
                        'fr' => 'Ce cookie est utilisé pour distinguer les humains des robots. Ceci est bénéfique pour le site web afin de créer des rapports valides sur l\'utilisation du leur site.',
                        'qc' => 'Ce cookie est utilisé pour distinguer les humains des robots. Ceci est bénéfique pour le site web afin de créer des rapports valides sur l\'utilisation du leur site.',
                        'pl' => 'Ten plik cookie służy do odróżniania ludzi od robotów. Jest to korzystne dla witryny internetowej, aby tworzyć prawidłowe raporty na temat korzystania z jej witryny.',
                        'ro' => 'Acest cookie este utilizat pentru a distinge oamenii de roboți. Acest lucru este benefic pentru site-ul web pentru a crea rapoarte valabile cu privire la utilizarea site-ului lor.',
                        'pt' => 'Este cookie é usado para distinguir humanos de robôs. Isso é benéfico para o site criar relatórios válidos sobre o uso de seu site.',
                        'br' => 'Este cookie é usado para distinguir humanos de robôs. Isso é benéfico para o site criar relatórios válidos sobre o uso de seu site.',
                        'sk' => 'Tento súbor cookie sa používa na odlíšenie ľudí od robotov. To je prospešné pre webovú stránku, pretože vytvára platné správy o používaní svojej webovej stránky.',
                        'nl' => 'Deze cookie wordt gebruikt om onderscheid te maken tussen mensen en bots. Dit is gunstig voor de website om juiste rapporten over het gebruik van de website te maken.',
                        'de' => 'Dieser Cookie wird verwendet, um zwischen Menschen und Bots zu unterscheiden. Dies ist vorteilhaft für die webseite, um gültige Berichte über die Nutzung ihrer webseite zu erstellen.',
                        'gr' => 'Αυτό το cookie χρησιμοποιείται για να διακρίνει τους ανθρώπους από τα ρομπότ. Αυτό είναι ωφέλιμο για τον ιστότοπο να δημιουργεί έγκυρες αναφορές σχετικά με τη χρήση του ιστότοπού τους.',
                        'it' => 'Questo cookie è usato per distinguere tra umani e robot. Questo è utile per il sito web, al fine di rendere validi rapporti sull\'uso del sito.',
                        'si' => '',
                        'da' => 'Benyttes til at bestemme, om brugeren er en virkelig person eller en software-robot - Dette muliggør skabelsen af valide rapporter om brugen af hjemmesiden.',
                        'no' => 'Denne informasjonskapselen brukes til å skille mennesker fra roboter. Dette er gunstig for nettstedet å lage gyldige rapporter om bruken av nettstedet deres.',
                        'cs' => 'Tento soubor cookie se používá k odlišení lidí od robotů. To je pro web přínosné pro vytváření platných zpráv o používání jejich stránek.',
                        'hu' => 'Ezt a sütit használják az emberek és a robotok megkülönböztetésére. Ez hasznos a weboldal számára, hogy érvényes jelentéseket készítsen a webhelyük használatáról.',
                        'sv' => 'Denna cookie används för att skilja mellan människor och bots. Detta är fördelaktigt för webbplatsen för att göra giltiga rapporter om användningen av deras webbplats.',
                    ],
                    'expiry' => [
                        'en' => 'Persistent',
                        'es' => 'Persistente',
                        'ag' => 'Persistente',
                        'cb' => 'Persistente',
                        'mx' => 'Persistente',
                        'fr' => 'Persistant',
                        'qc' => 'Persistant',
                        'pl' => 'Trwały',
                        'ro' => 'Persistent',
                        'pt' => 'Persistente',
                        'br' => 'Persistente',
                        'sk' => 'Vytrvalý',
                        'nl' => 'Aanhoudend',
                        'de' => 'Hartnäckig',
                        'gr' => 'Επίμονος',
                        'it' => 'Persistente',
                        'si' => 'Vztrajno',
                        'da' => 'Vedholdende',
                        'no' => 'Vedvarende',
                        'cs' => 'Trvalý',
                        'hu' => 'Kitartó',
                        'sv' => 'Beständig',
                    ],
                ],
                [
                    /* Youtube */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'rc::c',
                    'provider' => 'Google',
                    'provider_url' => 'https://policies.google.com/privacy',
                    'purpose' => [
                        'en' => 'This cookie is used to distinguish between humans and bots.',
                        'es' => 'Esta cookie se utiliza para distinguir entre humanos y bots.',
                        'ag' => 'Esta cookie se utiliza para distinguir entre humanos y bots.',
                        'cb' => 'Esta cookie se utiliza para distinguir entre humanos y bots.',
                        'mx' => 'Esta cookie se utiliza para distinguir entre humanos y bots.',
                        'fr' => 'Ce cookie est utilisé pour distinguer les humains des robots.',
                        'qc' => 'Ce cookie est utilisé pour distinguer les humains des robots.',
                        'pl' => 'Ten plik cookie służy do rozróżniania ludzi i botów.',
                        'ro' => 'Acest cookie este utilizat pentru a distinge între oameni și roboți..',
                        'pt' => 'Este cookie é usado para distinguir entre humanos e bots.',
                        'br' => 'Este cookie é usado para distinguir entre humanos e bots.',
                        'sk' => 'Tento súbor cookie sa používa na rozlíšenie ľudí a robotov.',
                        'nl' => 'Deze cookie wordt gebruikt om onderscheid te maken tussen mensen en bots.',
                        'de' => 'Dieser Cookie wird verwendet, um zwischen Menschen und Bots zu unterscheiden.',
                        'gr' => 'Αυτό το cookie χρησιμοποιείται για τη διάκριση μεταξύ ανθρώπων και ρομπότ.',
                        'it' => 'Questo cookie è usato per distinguere tra umani e robot.',
                        'si' => 'Tento súbor cookie sa používa na rozlíšenie ľudí a robotov.',
                        'da' => 'Benyttes til at bestemme, om brugeren er en virkelig person eller en robot.',
                        'no' => 'Denne informasjonskapselen brukes til å skille mellom mennesker og roboter.',
                        'cs' => 'Tento soubor cookie se používá k rozlišení mezi lidmi a roboty.',
                        'hu' => 'Ezt a sütit használják az emberek és a botok megkülönböztetésére.',
                        'sv' => 'Denna cookie används för att skilja mellan människor och bots.',
                    ],
                    'expiry' => [
                        'en' => 'Persistent',
                        'es' => 'Persistente',
                        'ag' => 'Persistente',
                        'cb' => 'Persistente',
                        'mx' => 'Persistente',
                        'fr' => 'Persistant',
                        'qc' => 'Persistant',
                        'pl' => 'Trwały',
                        'ro' => 'Persistent',
                        'pt' => 'Persistente',
                        'br' => 'Persistente',
                        'sk' => 'Vytrvalý',
                        'nl' => 'Aanhoudend',
                        'de' => 'Hartnäckig',
                        'gr' => 'Επίμονος',
                        'it' => 'Persistente',
                        'si' => 'Vztrajno',
                        'da' => 'Vedholdende',
                        'no' => 'Vedvarende',
                        'cs' => 'Trvalý',
                        'hu' => 'Kitartó',
                        'sv' => 'Beständig',
                    ],
                ],
                [
                    /* Crisp chat */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'crisp-client/domain-detect',
                    'provider' => 'Crisp Chat',
                    'provider_url' => 'https://help.crisp.chat/en/article/crisp-chatbox-cookie-and-ip-policy-1147xor/',
                    'purpose' => [
                        'en' => 'Disables features for Crisp Chat that are having too many visitors at the same time',
                        'es' => 'Desactiva las funciones de Crisp Chat que tienen demasiados visitantes al mismo tiempo',
                        'ag' => 'Desactiva las funciones de Crisp Chat que tienen demasiados visitantes al mismo tiempo',
                        'cb' => 'Desactiva las funciones de Crisp Chat que tienen demasiados visitantes al mismo tiempo',
                        'mx' => 'Desactiva las funciones de Crisp Chat que tienen demasiados visitantes al mismo tiempo',
                        'fr' => 'Désactive les fonctionnalités pour Crisp Chat qui ont trop de visiteurs en même temps',
                        'qc' => 'Désactive les fonctionnalités pour Crisp Chat qui ont trop de visiteurs en même temps',
                        'pl' => 'Wyłącza funkcje Crisp Chat, które mają zbyt wielu odwiedzających w tym samym czasie',
                        'ro' => 'Dezactivează funcțiile pentru Crisp Chat care au prea mulți vizitatori în același timp',
                        'pt' => 'Desativa recursos do Crisp Chat que estão recebendo muitos visitantes ao mesmo tempo',
                        'br' => 'Desativa recursos do Crisp Chat que estão recebendo muitos visitantes ao mesmo tempo',
                        'sk' => 'Zakáže funkcie pre Crisp Chat, ktoré majú príliš veľa návštevníkov súčasne',
                        'nl' => 'Schakelt functies uit voor Crisp Chat die te veel bezoekers tegelijkertijd hebben',
                        'de' => 'Deaktiviert Funktionen für Crisp Chat, die zu viele Besucher gleichzeitig haben',
                        'gr' => 'Απενεργοποιεί τις λειτουργίες για Crisp Chat που έχουν πάρα πολλούς επισκέπτες ταυτόχρονα',
                        'it' => 'Disabilita le funzionalità per Crisp Chat che hanno troppi visitatori contemporaneamente',
                        'si' => 'Onemogoči funkcije za Crisp Chat, ki imajo hkrati preveč obiskovalcev',
                        'da' => 'Deaktiverer funktioner til Crisp Chat, der har for mange besøgende på samme tid',
                        'no' => 'Deaktiver funksjoner for Crisp Chat som har for mange besøkende samtidig',
                        'cs' => 'Zakáže funkce pro Crisp Chat, které mají příliš mnoho návštěvníků současně',
                        'hu' => 'Letiltja azokat a Crisp Chat funkciókat, amelyek túl sok látogatóval rendelkeznek egyszerre',
                        'sv' => 'Inaktiverar funktioner för Crisp Chat som har för många besökare samtidigt',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [],
                    'name' => 'laravel_session',
                    'provider' => Tools::getShopDomainSsl(true),
                    'provider_url' => '',
                    'purpose' => [
                        'en' => 'Cookie used to identify a session instance for the user.',
                        'es' => 'Cookie utilizada para identificar una instancia de sesión para el usuario.',
                        'ag' => 'Cookie utilizada para identificar una instancia de sesión para el usuario.',
                        'cb' => 'Cookie utilizada para identificar una instancia de sesión para el usuario.',
                        'mx' => 'Cookie utilizada para identificar una instancia de sesión para el usuario.',
                        'fr' => 'Cookie utilisé pour identifier une instance de session pour l\'utilisateur.',
                        'qc' => 'Cookie utilisé pour identifier une instance de session pour l\'utilisateur.',
                        'pl' => 'Plik cookie używany do identyfikacji instancji sesji dla użytkownika.',
                        'ro' => 'Cookie folosit pentru a identifica o instanță de sesiune pentru utilizator.',
                        'pt' => 'Cookie usado para identificar uma instância de sessão para o usuário.',
                        'br' => 'Cookie usado para identificar uma instância de sessão para o usuário.',
                        'sk' => 'Cookie používaný na identifikáciu inštancie relácie pre používateľa.',
                        'nl' => 'Cookie dat wordt gebruikt om een sessie-instantie voor de gebruiker te identificeren.',
                        'de' => 'Cookie zur Identifizierung einer Sitzungsinstanz für den Benutzer.',
                        'gr' => 'Το cookie χρησιμοποιείται για την αναγνώριση μιας παρουσίας περιόδου σύνδεσης για τον χρήστη.',
                        'it' => 'Cookie utilizzato per identificare un\'istanza di sessione per l\'utente.',
                        'si' => 'Cookie používaný na identifikáciu inštancie relácie pre používateľa.',
                        'da' => 'Cookie bruges til at identificere en sessionsforekomst for brugeren.',
                        'no' => 'Informasjonskapsel brukes til å identifisere en øktforekomst for brukeren.',
                        'cs' => 'Cookie slouží k identifikaci instance relace pro uživatele.',
                        'hu' => 'A cookie azonosítja a munkamenet példányát a felhasználó számára.',
                        'sv' => 'Cookie används för att identifiera en sessionsinstans för användaren.',
                    ],
                    'expiry' => [
                        'en' => '1 day',
                        'es' => '1 día',
                        'ag' => '1 día',
                        'cb' => '1 día',
                        'mx' => '1 día',
                        'fr' => '1 jour',
                        'qc' => '1 jour',
                        'pl' => '1 dzień',
                        'ro' => '1 zi',
                        'pt' => '1 dia',
                        'br' => '1 dia',
                        'sk' => '1 deň',
                        'nl' => '1 dag',
                        'de' => '1 Tag',
                        'gr' => '1 μέρα',
                        'it' => '1 giorno',
                        'si' => '1 dan',
                        'da' => '1 dag',
                        'no' => '1 dag',
                        'cs' => '1 den',
                        'hu' => '1 nap',
                        'sv' => '1 dag',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [],
                    'name' => 'CONSENT',
                    'provider' => 'youtube.com',
                    'provider_url' => '',
                    'purpose' => [
                        'en' => 'Used to detect if the visitor has accepted the marketing category in the cookie banner. This cookie is necessary for GDPR-compliance of the website.',
                        'es' => 'Utilizada para detectar si el visitante ha aceptado la categoría de marketing en el banner de cookies. Esta cookie es necesaria para que la web cumpla con el RGPD.',
                        'ag' => 'Utilizada para detectar si el visitante ha aceptado la categoría de marketing en el banner de cookies. Esta cookie es necesaria para que la web cumpla con el RGPD.',
                        'cb' => 'Utilizada para detectar si el visitante ha aceptado la categoría de marketing en el banner de cookies. Esta cookie es necesaria para que la web cumpla con el RGPD.',
                        'mx' => 'Utilizada para detectar si el visitante ha aceptado la categoría de marketing en el banner de cookies. Esta cookie es necesaria para que la web cumpla con el RGPD.',
                        'fr' => 'Utilisé pour détecter si le visiteur a accepté la catégorie marketing dans la bannière de cookie. Ce cookie est nécessaire pour la conformité du site Web avec le RGPD.',
                        'qc' => 'Utilisé pour détecter si le visiteur a accepté la catégorie marketing dans la bannière de cookie. Ce cookie est nécessaire pour la conformité du site Web avec le RGPD.',
                        'pl' => '',
                        'ro' => '',
                        'pt' => '',
                        'br' => '',
                        'sk' => '',
                        'nl' => '',
                        'de' => 'Wird verwendet, um festzu stellen, ob der Besucher die Marketingkategorie im Cookie-Banner akzeptierthat. Dieser Cookie ist notwendig für die Einhaltung der DSGVO der Webseite.',
                        'gr' => '',
                        'it' => 'Utilizzato per rilevare se il visitatore ha accettato la categoria di marketing nel banner dei cookie. Questo cookie è necessario per la conformità GDPR del sito web.',
                        'si' => '',
                        'da' => 'Viser, hvorvidt den besøgende hartilvalgt cookies i marketin gkategorien. Cookien er nødvendig for overholdelsen af GDPR-regu lativet.',
                        'no' => '',
                        'cs' => '',
                        'hu' => '',
                        'sv' => '',
                    ],
                    'expiry' => [
                        'en' => '2 years',
                        'es' => '2 años',
                        'ag' => '2 años',
                        'cb' => '2 años',
                        'mx' => '2 años',
                        'fr' => '2 années',
                        'qc' => '2 années',
                        'pl' => '2 lata',
                        'ro' => '2 ani',
                        'pt' => '2 anos',
                        'br' => '2 anos',
                        'sk' => '2 roky',
                        'nl' => '2 jaar',
                        'de' => '2 Jahre',
                        'gr' => '2 χρόνια',
                        'it' => '2 anni',
                        'si' => '2 leti',
                        'da' => '2 år',
                        'no' => '2 år',
                        'cs' => '2 roky',
                        'hu' => '2 év',
                        'sv' => '2 år',
                    ],
                ],
                [
                    /* Advanced popup creator */
                    'active' => 0,
                    'modules' => ['advancedpopupcreator'],
                    'name' => 'apc_popup_session',
                    'provider' => Tools::getShopDomainSsl(true),
                    'provider_url' => '',
                    'purpose' => [
                        'en' => 'Used to know if it is a new browser session.',
                        'es' => 'Utilizada para saber si se trata de una nueva sesión de navegador.',
                        'ag' => 'Utilizada para saber si se trata de una nueva sesión de navegador.',
                        'cb' => 'Utilizada para saber si se trata de una nueva sesión de navegador.',
                        'mx' => 'Utilizada para saber si se trata de una nueva sesión de navegador.',
                        'fr' => 'Utilisé pour savoir s\'il s\'agit d\'une nouvelle session de navigateur.',
                        'qc' => 'Utilisé pour savoir s\'il s\'agit d\'une nouvelle session de navigateur.',
                        'pl' => 'Kiedyś wiedzieć, czy jest to nowa sesja przeglądarki.',
                        'ro' => '',
                        'pt' => '',
                        'br' => '',
                        'sk' => '',
                        'nl' => '',
                        'de' => 'Wird verwendet, um zu wissen, ob es sich um eine neue Browsersitzung handelt.',
                        'gr' => '',
                        'it' => 'Utilizzato per sapere se si tratta di una nuova sessione del browser.',
                        'si' => '',
                        'da' => '',
                        'no' => '',
                        'cs' => '',
                        'hu' => '',
                        'sv' => '',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
            ],
            CookiesPlusFinality::STATISTIC_COOKIE => [
                [
                    'active' => 0,
                    'modules' => [
                        'gapi',
                        'ganalytics',
                        'ps_googleanalytics',
                    ],
                    'name' => '_ga',
                    'provider' => 'Google',
                    'provider_url' => 'https://policies.google.com/privacy',
                    'purpose' => [
                        'en' => 'Registers a unique ID that is used to generate statistical data on how the visitor uses the website.',
                        'es' => 'Registra una identificación única que se utiliza para generar datos estadísticos acerca de cómo utiliza el visitante el sitio web.',
                        'ag' => 'Registra una identificación única que se utiliza para generar datos estadísticos acerca de cómo utiliza el visitante el sitio web.',
                        'cb' => 'Registra una identificación única que se utiliza para generar datos estadísticos acerca de cómo utiliza el visitante el sitio web.',
                        'mx' => 'Registra una identificación única que se utiliza para generar datos estadísticos acerca de cómo utiliza el visitante el sitio web.',
                        'fr' => 'Enregistre un identifiant unique utilisé pour générer des données statistiques sur la façon dont le visiteur utilise le site.',
                        'qc' => 'Enregistre un identifiant unique utilisé pour générer des données statistiques sur la façon dont le visiteur utilise le site.',
                        'pl' => 'Rejestruje unikalny identyfikator, który służy do generowania danych statystycznych dotyczących sposobu, w jaki odwiedzający korzysta ze strony internetowej.',
                        'ro' => 'Înregistrează un ID unic care este utilizat pentru a genera date statistice despre modul în care vizitatorul folosește site-ul web.',
                        'pt' => 'Registra um ID exclusivo que é usado para gerar dados estatísticos sobre como o visitante usa o site.',
                        'br' => 'Registra um ID exclusivo que é usado para gerar dados estatísticos sobre como o visitante usa o site.',
                        'sk' => 'Registruje jedinečné ID, ktoré sa používa na generovanie štatistických údajov o tom, ako návštevník používa webovú stránku.',
                        'nl' => 'Registreert een uniek ID die wordt gebruikt om statistische gegevens te genereren over hoe de bezoeker de website gebruikt.',
                        'de' => 'Registriert eine eindeutige ID, die verwendet wird, um statistische Daten dazu, wie der Besucher die Website nutzt, zu generieren.',
                        'gr' => 'Καταγράφει ένα μοναδικό αναγνωριστικό που χρησιμοποιείται για τη δημιουργία στατιστικών δεδομένων σχετικά με τον τρόπο με τον οποίο ο επισκέπτης χρησιμοποιεί τον ιστότοπο.',
                        'it' => 'Registra un ID univoco utilizzato per generare dati statistici su come il visitatore utilizza il sito internet.',
                        'si' => 'Registrira enolični ID, ki se uporablja za ustvarjanje statističnih podatkov o tem, kako obiskovalec uporablja spletno mesto.',
                        'da' => 'Registrerer et unikt ID, der anvendes til at føre statistik over hvordan den besøgende bruger hjemmesiden.',
                        'no' => 'Registrerer en unik ID som brukes til å generere statistiske data om hvordan den besøkende bruker nettstedet.',
                        'cs' => 'Zaregistruje jedinečné ID, které se používá ke generování statistických údajů o tom, jak návštěvník používá web.',
                        'hu' => 'Nyilvántartásba vesz egy egyedi azonosítót, amely statisztikai adatok előállítására szolgál arra vonatkozóan, hogy a látogató hogyan használja a weboldalt.',
                        'sv' => 'Registrerar ett unikt ID som används för att generera statistisk information om hur besökaren använder webbplatsen.',
                    ],
                    'expiry' => [
                        'en' => '2 years',
                        'es' => '2 años',
                        'ag' => '2 años',
                        'cb' => '2 años',
                        'mx' => '2 años',
                        'fr' => '2 années',
                        'qc' => '2 années',
                        'pl' => '2 lata',
                        'ro' => '2 ani',
                        'pt' => '2 anos',
                        'br' => '2 anos',
                        'sk' => '2 roky',
                        'nl' => '2 jaar',
                        'de' => '2 Jahre',
                        'gr' => '2 χρόνια',
                        'it' => '2 anni',
                        'si' => '2 leti',
                        'da' => '2 år',
                        'no' => '2 år',
                        'cs' => '2 roky',
                        'hu' => '2 év',
                        'sv' => '2 år',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [
                        'gapi',
                        'ganalytics',
                        'ps_googleanalytics',
                    ],
                    'name' => '_gat',
                    'provider' => 'Google',
                    'provider_url' => 'https://policies.google.com/privacy',
                    'purpose' => [
                        'en' => 'Used by Google Analytics to throttle request rate',
                        'es' => 'Utilizado por Google Analytics para controlar la tasa de peticiones',
                        'ag' => 'Utilizado por Google Analytics para controlar la tasa de peticiones',
                        'cb' => 'Utilizado por Google Analytics para controlar la tasa de peticiones',
                        'mx' => 'Utilizado por Google Analytics para controlar la tasa de peticiones',
                        'fr' => 'Utilisé par Google Analytics pour diminuer radicalement le taux de requêtes',
                        'qc' => 'Utilisé par Google Analytics pour diminuer radicalement le taux de requêtes',
                        'pl' => 'Używany przez Google Analytics do ograniczania liczby żądań',
                        'ro' => 'Folosit de Google Analytics pentru a restrânge rata solicitării',
                        'pt' => 'Usado pelo Google Analytics para controlar a taxa de solicitação',
                        'br' => 'Usado pelo Google Analytics para controlar a taxa de solicitação',
                        'sk' => 'Používa program Google Analytics na obmedzenie rýchlosti požiadaviek',
                        'nl' => 'Gebruikt door Google Analytics om verzoeksnelheid te vertragen',
                        'de' => 'Wird von Google Analytics verwendet, um die Anforderungsrate einzuschränken',
                        'gr' => 'Χρησιμοποιείται από το Google Analytics για να ρυθμίσει το ρυθμό αιτημάτων',
                        'it' => 'Utilizzato da Google Analytics per limitare la frequenza delle richieste',
                        'si' => 'Uporablja Google Analytics za zmanjšanje stopnje zahtev',
                        'da' => 'Anvendes af Google Analytics til at drosle hastigheden på antallet af forespørgsler til serveren',
                        'no' => 'Brukes av Google Analytics for å begrense forespørselsfrekvensen',
                        'cs' => 'Používá Google Analytics k omezení rychlosti požadavků',
                        'hu' => 'A Google Analytics használja a kérelmek arányának csökkentésére',
                        'sv' => 'Används av Google Analytics för att strypa begäran',
                    ],
                    'expiry' => [
                        'en' => '1 day',
                        'es' => '1 día',
                        'ag' => '1 día',
                        'cb' => '1 día',
                        'mx' => '1 día',
                        'fr' => '1 jour',
                        'qc' => '1 jour',
                        'pl' => '1 dzień',
                        'ro' => '1 zi',
                        'pt' => '1 dia',
                        'br' => '1 dia',
                        'sk' => '1 deň',
                        'nl' => '1 dag',
                        'de' => '1 Tag',
                        'gr' => '1 μέρα',
                        'it' => '1 giorno',
                        'si' => '1 dan',
                        'da' => '1 dag',
                        'no' => '1 dag',
                        'cs' => '1 den',
                        'hu' => '1 nap',
                        'sv' => '1 dag',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [
                        'gapi',
                        'ganalytics',
                        'ps_googleanalytics',
                    ],
                    'name' => '_gid',
                    'provider' => 'Google',
                    'provider_url' => 'https://policies.google.com/privacy',
                    'purpose' => [
                        'en' => 'Registers a unique ID that is used to generate statistical data on how the visitor uses the website.',
                        'es' => 'Registra una identificación única que se utiliza para generar datos estadísticos acerca de cómo utiliza el visitante el sitio web.',
                        'ag' => 'Registra una identificación única que se utiliza para generar datos estadísticos acerca de cómo utiliza el visitante el sitio web.',
                        'cb' => 'Registra una identificación única que se utiliza para generar datos estadísticos acerca de cómo utiliza el visitante el sitio web.',
                        'mx' => 'Registra una identificación única que se utiliza para generar datos estadísticos acerca de cómo utiliza el visitante el sitio web.',
                        'fr' => 'Enregistre un identifiant unique utilisé pour générer des données statistiques sur la façon dont le visiteur utilise le site.',
                        'qc' => 'Enregistre un identifiant unique utilisé pour générer des données statistiques sur la façon dont le visiteur utilise le site.',
                        'pl' => 'Rejestruje unikalny identyfikator, który służy do generowania danych statystycznych dotyczących sposobu, w jaki odwiedzający korzysta ze strony internetowej.',
                        'ro' => 'Înregistrează un ID unic care este utilizat pentru a genera date statistice despre modul în care vizitatorul folosește site-ul web.',
                        'pt' => 'Registra um ID exclusivo que é usado para gerar dados estatísticos sobre como o visitante usa o site.',
                        'br' => 'Registra um ID exclusivo que é usado para gerar dados estatísticos sobre como o visitante usa o site.',
                        'sk' => 'Registruje jedinečné ID, ktoré sa používa na generovanie štatistických údajov o tom, ako návštevník používa webovú stránku.',
                        'nl' => 'Registreert een uniek ID die wordt gebruikt om statistische gegevens te genereren over hoe de bezoeker de website gebruikt.',
                        'de' => 'Registriert eine eindeutige ID, die verwendet wird, um statistische Daten dazu, wie der Besucher die Website nutzt, zu generieren.',
                        'gr' => 'Καταγράφει ένα μοναδικό αναγνωριστικό που χρησιμοποιείται για τη δημιουργία στατιστικών δεδομένων σχετικά με τον τρόπο με τον οποίο ο επισκέπτης χρησιμοποιεί τον ιστότοπο.',
                        'it' => 'Registra un ID univoco utilizzato per generare dati statistici su come il visitatore utilizza il sito internet.',
                        'si' => 'Registrira enolični ID, ki se uporablja za ustvarjanje statističnih podatkov o tem, kako obiskovalec uporablja spletno mesto.',
                        'da' => 'Registrerer et unikt ID, der anvendes til at føre statistik over hvordan den besøgende bruger hjemmesiden.',
                        'no' => 'Registrerer en unik ID som brukes til å generere statistiske data om hvordan den besøkende bruker nettstedet.',
                        'cs' => 'Registrerer en unik ID som brukes til å generere statistiske data om hvordan den besøkende bruker nettstedet.',
                        'hu' => 'Nyilvántartásba vesz egy egyedi azonosítót, amely statisztikai adatok előállítására szolgál arra vonatkozóan, hogy a látogató hogyan használja a weboldalt.',
                        'sv' => 'Registrerar ett unikt ID som används för att generera statistisk information om hur besökaren använder webbplatsen.',
                    ],
                    'expiry' => [
                        'en' => '1 day',
                        'es' => '1 día',
                        'ag' => '1 día',
                        'cb' => '1 día',
                        'mx' => '1 día',
                        'fr' => '1 jour',
                        'qc' => '1 jour',
                        'pl' => '1 dzień',
                        'ro' => '1 zi',
                        'pt' => '1 dia',
                        'br' => '1 dia',
                        'sk' => '1 deň',
                        'nl' => '1 dag',
                        'de' => '1 Tag',
                        'gr' => '1 μέρα',
                        'it' => '1 giorno',
                        'si' => '1 dan',
                        'da' => '1 dag',
                        'no' => '1 dag',
                        'cs' => '1 den',
                        'hu' => '1 nap',
                        'sv' => '1 dag',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [
                        'gapi',
                        'ganalytics',
                        'ps_googleanalytics',
                    ],
                    'name' => '_gd#',
                    'provider' => 'Google',
                    'provider_url' => 'https://policies.google.com/privacy',
                    'purpose' => [
                        'en' => 'This is a Google Analytics Session cookie used to generate statistical data on how you use the website which is removed when you quit your browser.',
                        'es' => 'Se trata de una cookie de sesión de Google Analytics que se utiliza para generar datos estadísticos sobre cómo utiliza el sitio web que se elimina cuando sale de su navegador.',
                        'ag' => 'Se trata de una cookie de sesión de Google Analytics que se utiliza para generar datos estadísticos sobre cómo utiliza el sitio web que se elimina cuando sale de su navegador.',
                        'cb' => 'Se trata de una cookie de sesión de Google Analytics que se utiliza para generar datos estadísticos sobre cómo utiliza el sitio web que se elimina cuando sale de su navegador.',
                        'mx' => 'Se trata de una cookie de sesión de Google Analytics que se utiliza para generar datos estadísticos sobre cómo utiliza el sitio web que se elimina cuando sale de su navegador.',
                        'fr' => 'Il s\'agit d\'un cookie de session Google Analytics utilisé pour générer des données statistiques sur la façon dont vous utilisez le site Web, qui est supprimé lorsque vous quittez votre navigateur.',
                        'qc' => 'Il s\'agit d\'un cookie de session Google Analytics utilisé pour générer des données statistiques sur la façon dont vous utilisez le site Web, qui est supprimé lorsque vous quittez votre navigateur.',
                        'pl' => 'To jest sesyjny plik cookie Google Analytics służący do generowania danych statystycznych o sposobie korzystania ze strony internetowej, który jest usuwany po zamknięciu przeglądarki.',
                        'ro' => 'Acesta este un cookie de sesiune Google Analytics utilizat pentru a genera date statistice despre modul în care utilizați site-ul web, care este eliminat atunci când părăsiți browserul.',
                        'pt' => 'Este é um cookie de sessão do Google Analytics usado para gerar dados estatísticos sobre como você usa o site, que são removidos quando você fecha o navegador.',
                        'br' => 'Este é um cookie de sessão do Google Analytics usado para gerar dados estatísticos sobre como você usa o site, que são removidos quando você fecha o navegador.',
                        'sk' => 'Toto je súbor cookie relácie Google Analytics, ktorý sa používa na generovanie štatistických údajov o tom, ako používate webovú stránku, ktorá sa odstráni po ukončení prehliadača.',
                        'nl' => 'Dit is een Google Analytics-sessiecookie die wordt gebruikt om statistische gegevens te genereren over hoe u de website gebruikt en die wordt verwijderd wanneer u uw browser afsluit.',
                        'de' => 'Dies ist ein Google Analytics-Sitzungscookie, mit dem statistische Daten zur Nutzung der Website generiert werden, die beim Beenden Ihres Browsers entfernt werden.',
                        'gr' => 'Αυτό είναι ένα cookie περιόδου σύνδεσης του Google Analytics που χρησιμοποιείται για τη δημιουργία στατιστικών δεδομένων σχετικά με τον τρόπο χρήσης του ιστότοπου που καταργείται όταν κλείσετε το πρόγραμμα περιήγησής σας.',
                        'it' => 'Si tratta di un cookie di sessione di Google Analytics utilizzato per generare dati statistici su come utilizzi il sito web che vengono rimossi quando chiudi il browser.',
                        'si' => 'Toto je súbor cookie relácie Google Analytics, ktorý sa používa na generovanie štatistických údajov o tom, ako používate webovú stránku, ktorá sa odstráni po ukončení prehliadača.',
                        'da' => 'Dette er en Google Analytics Session-cookie, der bruges til at generere statistiske data om, hvordan du bruger det websted, der fjernes, når du lukker din browser.',
                        'no' => 'Dette er en Google Analytics Session-informasjonskapsel som brukes til å generere statistiske data om hvordan du bruker nettstedet som fjernes når du avslutter nettleseren.',
                        'cs' => 'Toto je soubor cookie relace Google Analytics, který se používá ke generování statistických údajů o tom, jak používáte web, který se odstraní, když opustíte prohlížeč.',
                        'hu' => 'Ez egy Google Analytics Munkamenet cookie, amelyet statisztikai adatok előállítására használnak a webhely használatáról, amelyet eltávolítunk, amikor kilép a böngészőből.',
                        'sv' => 'Detta är en Google Analytics Session-cookie som används för att generera statistisk information om hur du använder webbplatsen som tas bort när du avslutar din webbläsare.',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [
                        'gapi',
                        'ganalytics',
                        'ps_googleanalytics',
                    ],
                    'name' => 'collect',
                    'provider' => 'Google',
                    'provider_url' => 'https://policies.google.com/privacy',
                    'purpose' => [
                        'en' => 'It is used to send data to Google Analytics about the visitor\'s device and its behavior. Track the visitor across devices and marketing channels.',
                        'es' => 'Se utiliza para enviar datos a Google Analytics sobre el dispositivo del visitante y su comportamiento. Rastrea al visitante a través de dispositivos y canales de marketing.',
                        'ag' => 'Se utiliza para enviar datos a Google Analytics sobre el dispositivo del visitante y su comportamiento. Rastrea al visitante a través de dispositivos y canales de marketing.',
                        'cb' => 'Se utiliza para enviar datos a Google Analytics sobre el dispositivo del visitante y su comportamiento. Rastrea al visitante a través de dispositivos y canales de marketing.',
                        'mx' => 'Se utiliza para enviar datos a Google Analytics sobre el dispositivo del visitante y su comportamiento. Rastrea al visitante a través de dispositivos y canales de marketing.',
                        'fr' => 'Il est utilisé pour envoyer des données à Google Analytics sur l\'appareil du visiteur et son comportement. Suivez le visiteur à travers les appareils et les canaux marketing.',
                        'qc' => 'Il est utilisé pour envoyer des données à Google Analytics sur l\'appareil du visiteur et son comportement. Suivez le visiteur à travers les appareils et les canaux marketing.',
                        'pl' => 'Służy do wysyłania danych do Google Analytics o urządzeniu odwiedzającego i jego zachowaniu. Śledź odwiedzającego na różnych urządzeniach i kanałach marketingowych.',
                        'ro' => 'Este folosit pentru a trimite date către Google Analytics despre dispozitivul vizitatorului și comportamentul acestuia. Urmăriți vizitatorul pe dispozitive și canale de marketing.',
                        'pt' => 'Ele é usado para enviar dados ao Google Analytics sobre o dispositivo do visitante e seu comportamento. Rastreie o visitante em dispositivos e canais de marketing.',
                        'br' => 'Ele é usado para enviar dados ao Google Analytics sobre o dispositivo do visitante e seu comportamento. Rastreie o visitante em dispositivos e canais de marketing.',
                        'sk' => 'Používa sa na zasielanie údajov do služby Google Analytics o zariadení návštevníka a jeho správaní. Sledujte návštevníka naprieč zariadeniami a marketingovými kanálmi.',
                        'nl' => 'Het wordt gebruikt om gegevens naar Google Analytics te sturen over het apparaat van de bezoeker en zijn gedrag. Volg de bezoeker op verschillende apparaten en marketingkanalen.',
                        'de' => 'Es wird verwendet, um Daten über das Gerät des Besuchers und sein Verhalten an Google Analytics zu senden. Verfolgen Sie den Besucher über Geräte und Marketingkanäle hinweg.',
                        'gr' => 'Χρησιμοποιείται για την αποστολή δεδομένων στο Google Analytics σχετικά με τη συσκευή του επισκέπτη και τη συμπεριφορά του. Παρακολουθήστε τον επισκέπτη σε διάφορες συσκευές και κανάλια μάρκετινγκ.',
                        'it' => 'Viene utilizzato per inviare dati a Google Analytics sul dispositivo del visitatore e sul suo comportamento. Tieni traccia del visitatore su dispositivi e canali di marketing.',
                        'si' => 'Uporablja se za pošiljanje podatkov v Google Analytics o obiskovalčevi napravi in njenem vedenju. Sledite obiskovalcu po napravah in tržnih kanalih.',
                        'da' => 'Det bruges til at sende data til Google Analytics om den besøgendes enhed og dens adfærd. Spor den besøgende på tværs af enheder og marketingkanaler.',
                        'no' => 'Den brukes til å sende data til Google Analytics om den besøkendes enhet og dens oppførsel. Spor den besøkende på tvers av enheter og markedsføringskanaler.',
                        'cs' => 'Používá se k odesílání údajů do Google Analytics o zařízení návštěvníka a jeho chování. Sledujte návštěvníka napříč zařízeními a marketingovými kanály.',
                        'hu' => 'Arra szolgál, hogy adatokat küldjön a Google Analytics számára a látogató eszközéről és viselkedéséről. Kövesse nyomon a látogatót eszközök és marketing csatornák között.',
                        'sv' => 'Den används för att skicka data till Google Analytics om besökarens enhet och dess beteende. Spåra besökaren över enheter och marknadsföringskanaler.',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [
                        'gapi',
                        'ganalytics',
                        'ps_googleanalytics',
                    ],
                    'name' => 'r/collect',
                    'provider' => 'Google',
                    'provider_url' => 'https://policies.google.com/privacy',
                    'purpose' => [
                        'en' => 'It is used to send data to Google Analytics about the visitor\'s device and its behavior. Track the visitor across devices and marketing channels.',
                        'es' => 'Se utiliza para enviar datos a Google Analytics sobre el dispositivo del visitante y su comportamiento. Rastrea al visitante a través de dispositivos y canales de marketing.',
                        'ag' => 'Se utiliza para enviar datos a Google Analytics sobre el dispositivo del visitante y su comportamiento. Rastrea al visitante a través de dispositivos y canales de marketing.',
                        'cb' => 'Se utiliza para enviar datos a Google Analytics sobre el dispositivo del visitante y su comportamiento. Rastrea al visitante a través de dispositivos y canales de marketing.',
                        'mx' => 'Se utiliza para enviar datos a Google Analytics sobre el dispositivo del visitante y su comportamiento. Rastrea al visitante a través de dispositivos y canales de marketing.',
                        'fr' => 'Il est utilisé pour envoyer des données à Google Analytics sur l\'appareil du visiteur et son comportement. Suivez le visiteur à travers les appareils et les canaux marketing.',
                        'qc' => 'Il est utilisé pour envoyer des données à Google Analytics sur l\'appareil du visiteur et son comportement. Suivez le visiteur à travers les appareils et les canaux marketing.',
                        'pl' => 'Służy do wysyłania danych do Google Analytics o urządzeniu odwiedzającego i jego zachowaniu. Śledź odwiedzającego na różnych urządzeniach i kanałach marketingowych.',
                        'ro' => 'Este folosit pentru a trimite date către Google Analytics despre dispozitivul vizitatorului și comportamentul acestuia. Urmăriți vizitatorul pe dispozitive și canale de marketing.',
                        'pt' => 'Ele é usado para enviar dados ao Google Analytics sobre o dispositivo do visitante e seu comportamento. Rastreie o visitante em dispositivos e canais de marketing.',
                        'br' => 'Ele é usado para enviar dados ao Google Analytics sobre o dispositivo do visitante e seu comportamento. Rastreie o visitante em dispositivos e canais de marketing.',
                        'sk' => 'Používa sa na zasielanie údajov do služby Google Analytics o zariadení návštevníka a jeho správaní. Sledujte návštevníka naprieč zariadeniami a marketingovými kanálmi.',
                        'nl' => 'Het wordt gebruikt om gegevens naar Google Analytics te sturen over het apparaat van de bezoeker en zijn gedrag. Volg de bezoeker op verschillende apparaten en marketingkanalen.',
                        'de' => 'Es wird verwendet, um Daten über das Gerät des Besuchers und sein Verhalten an Google Analytics zu senden. Verfolgen Sie den Besucher über Geräte und Marketingkanäle hinweg.',
                        'gr' => 'Χρησιμοποιείται για την αποστολή δεδομένων στο Google Analytics σχετικά με τη συσκευή του επισκέπτη και τη συμπεριφορά του. Παρακολουθήστε τον επισκέπτη σε διάφορες συσκευές και κανάλια μάρκετινγκ.',
                        'it' => 'Viene utilizzato per inviare dati a Google Analytics sul dispositivo del visitatore e sul suo comportamento. Tieni traccia del visitatore su dispositivi e canali di marketing.',
                        'si' => 'Uporablja se za pošiljanje podatkov v Google Analytics o obiskovalčevi napravi in njenem vedenju. Sledite obiskovalcu po napravah in tržnih kanalih.',
                        'da' => 'Det bruges til at sende data til Google Analytics om den besøgendes enhed og dens adfærd. Spor den besøgende på tværs af enheder og marketingkanaler.',
                        'no' => 'Den brukes til å sende data til Google Analytics om den besøkendes enhet og dens oppførsel. Spor den besøkende på tvers av enheter og markedsføringskanaler.',
                        'cs' => 'Používá se k odesílání údajů do Google Analytics o zařízení návštěvníka a jeho chování. Sledujte návštěvníka napříč zařízeními a marketingovými kanály.',
                        'hu' => 'Arra szolgál, hogy adatokat küldjön a Google Analytics számára a látogató eszközéről és viselkedéséről. Kövesse nyomon a látogatót eszközök és marketing csatornák között.',
                        'sv' => 'Den används för att skicka data till Google Analytics om besökarens enhet och dess beteende. Spåra besökaren över enheter och marknadsföringskanaler.',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [],
                    'name' => '_gat_gtag_UA_*',
                    'provider' => 'Google',
                    'provider_url' => 'https://policies.google.com/privacy',
                    'purpose' => [
                        'en' => 'Used to throttle request rate.',
                        'es' => 'Se utiliza para acelerar la tasa de solicitudes.',
                        'ag' => 'Se utiliza para acelerar la tasa de solicitudes.',
                        'cb' => 'Se utiliza para acelerar la tasa de solicitudes.',
                        'mx' => 'Se utiliza para acelerar la tasa de solicitudes.',
                        'fr' => 'Utilisé pour limiter le taux de demande.',
                        'qc' => 'Utilisé pour limiter le taux de demande.',
                        'pl' => 'Służy do ograniczania szybkości żądań.',
                        'ro' => 'Folosit pentru a limita rata de solicitare.',
                        'pt' => 'Usado para controlar a taxa de solicitação.',
                        'br' => 'Usado para controlar a taxa de solicitação.',
                        'sk' => 'Používa sa na obmedzenie rýchlosti požiadaviek.',
                        'nl' => 'Wordt gebruikt om de verzoeksnelheid te vertragen.',
                        'de' => 'Wird verwendet, um die Anforderungsrate zu drosseln.',
                        'gr' => 'Χρησιμοποιείται για να ρυθμίσει το ρυθμό αιτήματος.',
                        'it' => 'Utilizzato per limitare la frequenza delle richieste.',
                        'si' => 'Uporablja se za stopnjo plina.',
                        'da' => 'Bruges til at reducere anmodningshastigheden.',
                        'no' => 'Brukes til å begrense forespørselsfrekvensen.',
                        'cs' => 'Slouží k omezení rychlosti požadavku.',
                        'hu' => 'A fojtószelep kérési arányához használják.',
                        'sv' => 'Används för att strypa begäran.',
                    ],
                    'expiry' => [
                        'en' => '1 minute',
                        'es' => '1 minuto',
                        'ag' => '1 minuto',
                        'cb' => '1 minuto',
                        'mx' => '1 minuto',
                        'fr' => '1 minute',
                        'qc' => '1 minute',
                        'pl' => '1 minuta',
                        'ro' => '1 minut',
                        'pt' => '1 minuto',
                        'br' => '1 minuto',
                        'sk' => '1 minútu',
                        'nl' => '1 minuut',
                        'de' => '1 Minute',
                        'gr' => '1 λεπτό',
                        'it' => '1 minuto',
                        'si' => '1 minuto',
                        'da' => '1 minut',
                        'no' => '1 minutt',
                        'cs' => '1 minuta',
                        'hu' => '1 perc',
                        'sv' => '1 minut',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [
                        'gapi',
                        'ganalytics',
                        'ps_googleanalytics',
                    ],
                    'name' => '_ga_#',
                    'provider' => 'Google',
                    'provider_url' => 'https://policies.google.com/privacy',
                    'purpose' => [
                        'en' => 'Used by Google Analytics to collect data on the number of times a user has visited the website as well as
dates for the first and most recent visit.',
                        'es' => 'Recopila datos sobre el número de veces que un usuario ha visitado el sitio web además de las fechas de la primera visita y de la más reciente. Utilizada por Google Analytics.',
                        'ag' => 'Recopila datos sobre el número de veces que un usuario ha visitado el sitio web además de las fechas de la primera visita y de la más reciente. Utilizada por Google Analytics.',
                        'cb' => 'Recopila datos sobre el número de veces que un usuario ha visitado el sitio web además de las fechas de la primera visita y de la más reciente. Utilizada por Google Analytics.',
                        'mx' => 'Recopila datos sobre el número de veces que un usuario ha visitado el sitio web además de las fechas de la primera visita y de la más reciente. Utilizada por Google Analytics.',
                        'fr' => 'Utilisé par Google Analytics our recueillir des données sur le nombre de fois qu\'un utilisateur a visité le site web ainsi que les dates de la première et de la plus récente visite.',
                        'qc' => 'Utilisé par Google Analytics our recueillir des données sur le nombre de fois qu\'un utilisateur a visité le site web ainsi que les dates de la première et de la plus récente visite.',
                        'pl' => 'Używany przez Google Analytics do zbierania danych o tym, ile razy użytkownik odwiedził witrynę, a także
daty pierwszej i ostatniej wizyty.',
                        'ro' => '',
                        'pt' => '',
                        'br' => '',
                        'sk' => '',
                        'nl' => '',
                        'de' => 'Sammelt Daten dazu, wie oft ein Benutzer eine Website besucht hat, sowie Daten für den ersten und letzten Besuch. Von Google Analytics verwendet.',
                        'gr' => '',
                        'it' => 'Utilizzato da Google Analytics per raccogliere dati sul numero di volte che un utente ha visitato il sito internet, oltre che le dati per la prima visita e la visita più recente.',
                        'si' => '',
                        'da' => '',
                        'no' => '',
                        'cs' => '',
                        'hu' => '',
                        'sv' => '',
                    ],
                    'expiry' => [
                        'en' => '2 years',
                        'es' => '2 años',
                        'ag' => '2 años',
                        'cb' => '2 años',
                        'mx' => '2 años',
                        'fr' => '2 années',
                        'qc' => '2 années',
                        'pl' => '2 lata',
                        'ro' => '2 ani',
                        'pt' => '2 anos',
                        'br' => '2 anos',
                        'sk' => '2 roky',
                        'nl' => '2 jaar',
                        'de' => '2 Jahre',
                        'gr' => '2 χρόνια',
                        'it' => '2 anni',
                        'si' => '2 leti',
                        'da' => '2 år',
                        'no' => '2 år',
                        'cs' => '2 roky',
                        'hu' => '2 év',
                        'sv' => '2 år',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => ['smartsupp'],
                    'name' => 'ssupp.vid',
                    'provider' => 'Smartsupp',
                    'provider_url' => 'https://www.smartsupp.com/help/privacy/',
                    'purpose' => [
                        'en' => 'Necessary for the functionality of the website\'s chat-box function.',
                        'es' => 'Necesaria para la funcionalidad del chat en la web.',
                        'ag' => 'Necesaria para la funcionalidad del chat en la web.',
                        'cb' => 'Necesaria para la funcionalidad del chat en la web.',
                        'mx' => 'Necesaria para la funcionalidad del chat en la web.',
                        'fr' => 'Nécessaire pour la fonctionnalité de la boîte de dialogue du site Web.',
                        'qc' => 'Nécessaire pour la fonctionnalité de la boîte de dialogue du site Web.',
                        'pl' => 'Niezbędne do funkcjonowania czatu w sieci.',
                        'ro' => 'Necesar pentru funcționalitatea chat-ului pe web.',
                        'pt' => 'Necessário para a funcionalidade do chat na web.',
                        'br' => 'Necessário para a funcionalidade do chat na web.',
                        'sk' => 'Nevyhnutné pre funkčnosť četu na webe.',
                        'nl' => 'Noodzakelijk voor de functionaliteit van de chatboxfunctie van de website.',
                        'de' => 'Notwendig für die Funktionalität der Chat-Box-Funktion der Webseite.',
                        'gr' => 'Απαραίτητο για τη λειτουργικότητα της συνομιλίας στον Ιστό.',
                        'it' => 'Necessario per la corretta funzionalità della chat-box del sito web.',
                        'si' => 'Potrebno za funkcionalnost klepeta v spletu.',
                        'da' => 'Nødvendig for chatboks-funktionen på hjemmesiden.',
                        'no' => 'Nødvendig for funksjonaliteten til chatten på nettet.',
                        'cs' => 'Nezbytné pro funkčnost chatu na webu.',
                        'hu' => 'Szükséges a webes csevegés funkcionalitásához.',
                        'sv' => 'Nödvändigt för funktionaliteten i webbplatsens chattboxfunktion.',
                    ],
                    'expiry' => [
                        'en' => '6 months',
                        'es' => '6 meses',
                        'ag' => '6 meses',
                        'cb' => '6 meses',
                        'mx' => '6 meses',
                        'fr' => '6 mois',
                        'qc' => '6 mois',
                        'pl' => '6 miesiące',
                        'ro' => '6 luni',
                        'pt' => '6 meses',
                        'br' => '6 meses',
                        'sk' => '6 mesiace',
                        'nl' => '6 maanden',
                        'de' => '6 Monate',
                        'gr' => '6 μήνες',
                        'it' => '6 mesi',
                        'si' => '6 meseca',
                        'da' => '6 mdr.',
                        'no' => '6 måneder',
                        'cs' => '6 měsíců',
                        'hu' => '6 hónap',
                        'sv' => '6 månader',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => ['smartsupp'],
                    'name' => 'ssupp.chatid',
                    'provider' => 'Smartsupp',
                    'provider_url' => 'https://www.smartsupp.com/help/privacy/',
                    'purpose' => [
                        'en' => 'Conversation ID from live chat service provided by Smartsupp',
                        'es' => 'ID de conversación del servicio de chat en vivo proporcionado por Smartsupp',
                        'ag' => 'ID de conversación del servicio de chat en vivo proporcionado por Smartsupp',
                        'cb' => 'ID de conversación del servicio de chat en vivo proporcionado por Smartsupp',
                        'mx' => 'ID de conversación del servicio de chat en vivo proporcionado por Smartsupp',
                        'fr' => 'ID de conversation du service de chat en direct fourni par Smartsupp',
                        'qc' => 'ID de conversation du service de chat en direct fourni par Smartsupp',
                        'pl' => 'Identyfikator rozmowy z usługi czatu na żywo dostarczanej przez Smartsupp',
                        'ro' => 'ID-ul conversației din serviciul de chat live furnizat de Smartsupp',
                        'pt' => 'ID da conversa do serviço de chat ao vivo fornecido pela Smartsupp',
                        'br' => 'ID da conversa do serviço de chat ao vivo fornecido pela Smartsupp',
                        'sk' => 'ID konverzácie zo služby živého chatu poskytované spoločnosťou Smartsupp',
                        'nl' => 'Gesprek-ID van live chatservice aangeboden door Smartsupp',
                        'de' => 'Gesprächs-ID vom Live-Chat-Dienst von Smartsupp',
                        'gr' => 'Αναγνωριστικό συνομιλίας από την υπηρεσία ζωντανής συνομιλίας που παρέχεται από το Smartsupp',
                        'it' => 'ID conversazione dal servizio di chat dal vivo fornito da Smartsupp',
                        'si' => 'ID pogovora iz storitve klepeta v živo, ki jo nudi Smartsupp',
                        'da' => 'Samtale-id fra live chat-tjeneste leveret af Smartsupp',
                        'no' => 'Samtale-ID fra live chat-tjeneste levert av Smartsupp',
                        'cs' => 'ID konverzace ze služby živého chatu poskytované společností Smartsupp',
                        'hu' => 'Beszélgetési azonosító a Smartsupp által biztosított élő chat szolgáltatásból',
                        'sv' => 'Konversations-ID från livechatttjänst från Smartsupp',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => ['smartsupp'],
                    'name' => 'ssupp.group',
                    'provider' => 'Smartsupp',
                    'provider_url' => 'https://www.smartsupp.com/help/privacy/',
                    'purpose' => [
                        'en' => 'Last group of visitor from live chat service provided by Smartsupp.',
                        'es' => 'Último grupo de visitantes del servicio de chat en vivo proporcionado por Smartsupp.',
                        'ag' => 'Último grupo de visitantes del servicio de chat en vivo proporcionado por Smartsupp.',
                        'cb' => 'Último grupo de visitantes del servicio de chat en vivo proporcionado por Smartsupp.',
                        'mx' => 'Último grupo de visitantes del servicio de chat en vivo proporcionado por Smartsupp.',
                        'fr' => 'Dernier groupe de visiteurs du service de chat en direct fourni par Smartsupp.',
                        'qc' => 'Dernier groupe de visiteurs du service de chat en direct fourni par Smartsupp.',
                        'pl' => 'Ostatnia grupa gości z usługi czatu na żywo dostarczonej przez Smartsupp.',
                        'ro' => 'Ultimul grup de vizitatori din serviciul de chat live furnizat de Smartsupp.',
                        'pt' => 'Último grupo de visitantes do serviço de chat ao vivo fornecido pela Smartsupp.',
                        'br' => 'Último grupo de visitantes do serviço de chat ao vivo fornecido pela Smartsupp.',
                        'sk' => 'Posledná skupina návštevníkov zo služby živého chatu poskytovanej spoločnosťou Smartsupp.',
                        'nl' => 'Laatste groep bezoekers van de live chatservice van Smartsupp.',
                        'de' => 'Letzte Besuchergruppe des Live-Chat-Dienstes von Smartsupp.',
                        'gr' => 'Τελευταία ομάδα επισκεπτών από την υπηρεσία ζωντανής συνομιλίας που παρέχεται από το Smartsupp.',
                        'it' => 'Ultimo gruppo di visitatori dal servizio di live chat fornito da Smartsupp.',
                        'si' => 'Zadnja skupina obiskovalcev storitve klepeta v živo, ki jo ponuja Smartsupp.',
                        'da' => 'Sidste gruppe af besøgende fra live chat service leveret af Smartsupp.',
                        'no' => 'Siste gruppe besøkende fra live chat-tjenesten levert av Smartsupp.',
                        'cs' => 'Poslední skupina návštěvníků ze služby živého chatu poskytovaná společností Smartsupp.',
                        'hu' => 'Utolsó látogatócsoport a Smartsupp által biztosított élő chat szolgáltatásból.',
                        'sv' => 'Den senaste gruppen besökare från livechatttjänsten från Smartsupp.',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => ['smartsupp'],
                    'name' => 'ssupp.visits',
                    'provider' => 'Smartsupp',
                    'provider_url' => 'https://www.smartsupp.com/help/privacy/',
                    'purpose' => [
                        'en' => 'Number of previous visits, necessary to track for automatic messages from live chat service provided by Smartsupp.',
                        'es' => 'Número de visitas previas, necesario para rastrear mensajes automáticos del servicio de chat en vivo proporcionado por Smartsupp.',
                        'ag' => 'Número de visitas previas, necesario para rastrear mensajes automáticos del servicio de chat en vivo proporcionado por Smartsupp.',
                        'cb' => 'Número de visitas previas, necesario para rastrear mensajes automáticos del servicio de chat en vivo proporcionado por Smartsupp.',
                        'mx' => 'Número de visitas previas, necesario para rastrear mensajes automáticos del servicio de chat en vivo proporcionado por Smartsupp.',
                        'fr' => 'Nombre de visites précédentes, nécessaires pour suivre les messages automatiques du service de chat en direct fourni par Smartsupp.',
                        'qc' => 'Nombre de visites précédentes, nécessaires pour suivre les messages automatiques du service de chat en direct fourni par Smartsupp.',
                        'pl' => 'Liczba poprzednich wizyt, niezbędna do śledzenia automatycznych wiadomości z usługi czatu na żywo dostarczanej przez Smartsupp.',
                        'ro' => 'Numărul de vizite anterioare, necesar pentru a urmări mesajele automate din serviciul de chat live furnizat de Smartsupp.',
                        'pt' => 'Número de visitas anteriores, necessário para rastrear mensagens automáticas do serviço de chat ao vivo fornecido pela Smartsupp.',
                        'br' => 'Número de visitas anteriores, necessário para rastrear mensagens automáticas do serviço de chat ao vivo fornecido pela Smartsupp.',
                        'sk' => 'Počet predchádzajúcich návštev, ktoré je potrebné sledovať pre automatické správy zo služby živého chatu poskytovanej spoločnosťou Smartsupp.',
                        'nl' => 'Aantal eerdere bezoeken, nodig om bij te houden voor automatische berichten van de live chatservice van Smartsupp.',
                        'de' => 'Anzahl der vorherigen Besuche, die erforderlich sind, um automatische Nachrichten vom Live-Chat-Dienst von Smartsupp zu verfolgen.',
                        'gr' => 'Αριθμός προηγούμενων επισκέψεων, απαραίτητος για την παρακολούθηση αυτόματων μηνυμάτων από την υπηρεσία ζωντανής συνομιλίας που παρέχεται από το Smartsupp.',
                        'it' => 'Numero di visite precedenti, necessario per tenere traccia dei messaggi automatici dal servizio di live chat fornito da Smartsupp.',
                        'si' => 'Število prejšnjih obiskov, potrebnih za sledenje samodejnim sporočilom iz storitve klepeta v živo, ki jo ponuja Smartsupp.',
                        'da' => 'Antal tidligere besøg, der er nødvendige for at spore for automatiske meddelelser fra live chat-tjeneste leveret af Smartsupp.',
                        'no' => 'Antall tidligere besøk, nødvendig for å spore for automatiske meldinger fra live chat-tjenesten levert av Smartsupp.',
                        'cs' => 'Počet předchozích návštěv, které je nutné sledovat pro automatické zprávy ze služby živého chatu poskytované společností Smartsupp.',
                        'hu' => 'Az előző látogatások száma, amely szükséges a Smartsupp által biztosított élő chat szolgáltatás automatikus üzeneteinek nyomon követéséhez.',
                        'sv' => 'Antal tidigare besök, nödvändigt för att spåra efter automatiska meddelanden från livechatttjänsten från Smartsupp.',
                    ],
                    'expiry' => [
                        'en' => '6 months',
                        'es' => '6 meses',
                        'ag' => '6 meses',
                        'cb' => '6 meses',
                        'mx' => '6 meses',
                        'fr' => '6 mois',
                        'qc' => '6 mois',
                        'pl' => '6 miesiące',
                        'ro' => '6 luni',
                        'pt' => '6 meses',
                        'br' => '6 meses',
                        'sk' => '6 mesiace',
                        'nl' => '6 maanden',
                        'de' => '6 Monate',
                        'gr' => '6 μήνες',
                        'it' => '6 mesi',
                        'si' => '6 meseca',
                        'da' => '6 mdr.',
                        'no' => '6 måneder',
                        'cs' => '6 měsíců',
                        'hu' => '6 hónap',
                        'sv' => '6 månader',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [],
                    'name' => 'yt-player-headers-readable',
                    'provider' => 'youtube.com',
                    'provider_url' => '',
                    'purpose' => [
                        'en' => 'Used to determine the optimal video quality based on the visitor\'s device and network settings.',
                        'es' => 'Utilizada para determinar la calidad óptima del video basada en el dispositivo del visitante y en los ajustes de la red.',
                        'ag' => 'Utilizada para determinar la calidad óptima del video basada en el dispositivo del visitante y en los ajustes de la red.',
                        'cb' => 'Utilizada para determinar la calidad óptima del video basada en el dispositivo del visitante y en los ajustes de la red.',
                        'mx' => 'Utilizada para determinar la calidad óptima del video basada en el dispositivo del visitante y en los ajustes de la red.',
                        'fr' => 'Utilisé pour déterminer la qualité vidéo optimale en fonction du périphériqu e du visiteur et des paramètres réseau.',
                        'qc' => 'Utilisé pour déterminer la qualité vidéo optimale en fonction du périphériqu e du visiteur et des paramètres réseau.',
                        'pl' => '',
                        'ro' => '',
                        'pt' => '',
                        'br' => '',
                        'sk' => '',
                        'nl' => '',
                        'de' => 'Wird verwendet, um basierend auf den Geräte-und Netzwerkeinstellungen des Besuchers die optimale Videoqualität zuermitteln.',
                        'gr' => '',
                        'it' => 'Utilizzato per determinare la qualità video ottimale in base al dispositivo del visitatore e alle impostazioni di rete.',
                        'si' => '',
                        'da' => 'Bestemmer den optimale video kvalitet ud fraden besøgendes platform og netværkshastighed.',
                        'no' => '',
                        'cs' => '',
                        'hu' => '',
                        'sv' => '',
                    ],
                    'expiry' => [
                        'en' => 'Persistent',
                        'es' => 'Persistente',
                        'ag' => 'Persistente',
                        'cb' => 'Persistente',
                        'mx' => 'Persistente',
                        'fr' => 'Persistant',
                        'qc' => 'Persistant',
                        'pl' => 'Trwały',
                        'ro' => 'Persistent',
                        'pt' => 'Persistente',
                        'br' => 'Persistente',
                        'sk' => 'Vytrvalý',
                        'nl' => 'Aanhoudend',
                        'de' => 'Hartnäckig',
                        'gr' => 'Επίμονος',
                        'it' => 'Persistente',
                        'si' => 'Vztrajno',
                        'da' => 'Vedholdende',
                        'no' => 'Vedvarende',
                        'cs' => 'Trvalý',
                        'hu' => 'Kitartó',
                        'sv' => 'Beständig',
                    ],
                ],
                [
                    /* Tawk.to */
                    'active' => 0,
                    'modules' => ['tawkto', 'tawktoconfig'],
                    'name' => 'TawkConnectionTime',
                    'provider' => Tools::getShopDomainSsl(true),
                    'provider_url' => '',
                    'purpose' => [
                        'en' => 'Allows the website to recognise the visitor, in order to optimize the chat-box functionality.',
                        'es' => 'Permite a la web reconocer al visitante con el objeto de optimizar la función de chat.',
                        'ag' => 'Permite a la web reconocer al visitante con el objeto de optimizar la función de chat.',
                        'cb' => 'Permite a la web reconocer al visitante con el objeto de optimizar la función de chat.',
                        'mx' => 'Permite a la web reconocer al visitante con el objeto de optimizar la función de chat.',
                        'fr' => 'Permet au site Web de reconnaître le visiteur afin d’optimiser la fonctionnalité de la boîte de dialogue.',
                        'qc' => 'Permet au site Web de reconnaître le visiteur afin d’optimiser la fonctionnalité de la boîte de dialogue.',
                        'pl' => 'Umożliwia witrynie ponowne rozpoznanie osoby odwiedzającej, aby zoptymalizować funkcjonalność pola czatu.',
                        'ro' => '',
                        'pt' => '',
                        'br' => '',
                        'sk' => '',
                        'nl' => '',
                        'de' => 'Ermöglicht der Webseite, den Besucher zu erkennen, um die Chat-Box-Funktionalität zu optimieren.',
                        'gr' => '',
                        'it' => 'Consente al sito web di riconoscere il visitatore al fine di ottimizzare la funzionalità della chat-box.',
                        'si' => '',
                        'da' => '',
                        'no' => '',
                        'cs' => '',
                        'hu' => '',
                        'sv' => '',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
            ],
            CookiesPlusFinality::MARKETING_COOKIE => [
                [
                    'active' => 0,
                    'modules' => [
                        'pspixel',
                        'facebookconversionpixel',
                        'ps_facebook',
                    ],
                    'name' => 'fr',
                    'provider' => 'Facebook',
                    'provider_url' => 'https://www.facebook.com/policies/cookies/',
                    'purpose' => [
                        'en' => 'Used by Facebook to deliver a series of advertisement products such as real time bidding from third party advertisers.',
                        'es' => 'Utilizada por Facebook para proporcionar una serie de productos publicitarios como pujas en tiempo real de terceros anunciantes.',
                        'ag' => 'Utilizada por Facebook para proporcionar una serie de productos publicitarios como pujas en tiempo real de terceros anunciantes.',
                        'cb' => 'Utilizada por Facebook para proporcionar una serie de productos publicitarios como pujas en tiempo real de terceros anunciantes.',
                        'mx' => 'Utilizada por Facebook para proporcionar una serie de productos publicitarios como pujas en tiempo real de terceros anunciantes.',
                        'fr' => 'Utilisé par Facebook pour fournir une série de produits publicitaires tels que les offres en temps réel d\'annonceurs tiers.',
                        'qc' => 'Utilisé par Facebook pour fournir une série de produits publicitaires tels que les offres en temps réel d\'annonceurs tiers.',
                        'pl' => 'Używany przez Facebooka do dostarczania serii produktów reklamowych, takich jak licytowanie w czasie rzeczywistym od reklamodawców zewnętrznych.',
                        'ro' => 'Folosit de Facebook pentru a livra o serie de produse publicitare, cum ar fi licitarea în timp real de la agenții de publicitate terți.',
                        'pt' => 'Usado pelo Facebook para entregar uma série de produtos de publicidade, como lances em tempo real de anunciantes terceiros.',
                        'br' => 'Usado pelo Facebook para entregar uma série de produtos de publicidade, como lances em tempo real de anunciantes terceiros.',
                        'sk' => 'Používa Facebook na dodanie radu reklamných produktov, ako napríklad ponúkanie cien v reálnom čase od inzerentov tretích strán.',
                        'nl' => 'Gebruikt door Facebook om een reeks advertentieproducten te leveren, zoals realtime bieden van externe adverteerders.',
                        'de' => 'Wird von Facebook genutzt, um eine Reihe von Werbeprodukten anzuzeigen, zum Beispiel Echtzeitgebote dritter Werbetreibender.',
                        'gr' => 'Χρησιμοποιείται από το Facebook για την παράδοση μιας σειράς διαφημιστικών προϊόντων, όπως η υποβολή προσφορών σε πραγματικό χρόνο από διαφημιστές τρίτων.',
                        'it' => 'Utilizzato da Facebook per fornire una serie di prodotti pubblicitari come offerte in tempo reale da inserzionisti terzi.',
                        'si' => 'Facebook uporablja Facebook za zagotavljanje vrste oglasnih izdelkov, kot so ponudbe v realnem času od neodvisnih oglaševalcev.',
                        'da' => 'Anvendes af Facebook til at levere forskellige reklame-tjenester, herunder realtids-bud fra tredjeparts-annoncører.',
                        'no' => 'Brukt av Facebook for å levere en serie reklameprodukter som for eksempel sanntidsbud fra tredjepartsannonsører.',
                        'cs' => 'Používá Facebook k poskytování řady reklamních produktů, jako je nabízení cen v reálném čase od inzerentů třetích stran.',
                        'hu' => 'A Facebook egy sor olyan reklámtermék szállítására használja, mint például valós idejű ajánlattétel harmadik fél hirdetőitől.',
                        'sv' => 'Används av Facebook för att leverera en serie reklamprodukter såsom budgivning i realtid från tredjepartsannonsörer.',
                    ],
                    'expiry' => [
                        'en' => '3 months',
                        'es' => '3 meses',
                        'ag' => '3 meses',
                        'cb' => '3 meses',
                        'mx' => '3 meses',
                        'fr' => '3 mois',
                        'qc' => '3 mois',
                        'pl' => '3 miesiące',
                        'ro' => '3 luni',
                        'pt' => '3 meses',
                        'br' => '3 meses',
                        'sk' => '3 mesiace',
                        'nl' => '3 maanden',
                        'de' => '3 Monate',
                        'gr' => '3 μήνες',
                        'it' => '3 mesi',
                        'si' => '3 meseca',
                        'da' => '3 mdr.',
                        'no' => '3 måneder',
                        'cs' => '3 měsíců',
                        'hu' => '3 hónap',
                        'sv' => '3 månader',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [
                        'pspixel',
                        'facebookconversionpixel',
                        'cartsguru',
                        'ps_facebook',
                    ],
                    'name' => '_fbp',
                    'provider' => 'Facebook',
                    'provider_url' => 'https://www.facebook.com/policies/cookies/',
                    'purpose' => [
                        'en' => 'Used by Facebook to deliver a series of advertisement products such as real time bidding from third party advertisers.',
                        'es' => 'Utilizada por Facebook para proporcionar una serie de productos publicitarios como pujas en tiempo real de terceros anunciantes.',
                        'ag' => 'Utilizada por Facebook para proporcionar una serie de productos publicitarios como pujas en tiempo real de terceros anunciantes.',
                        'cb' => 'Utilizada por Facebook para proporcionar una serie de productos publicitarios como pujas en tiempo real de terceros anunciantes.',
                        'mx' => 'Utilizada por Facebook para proporcionar una serie de productos publicitarios como pujas en tiempo real de terceros anunciantes.',
                        'fr' => 'Utilisé par Facebook pour fournir une série de produits publicitaires tels que les offres en temps réel d\'annonceurs tiers.',
                        'qc' => 'Utilisé par Facebook pour fournir une série de produits publicitaires tels que les offres en temps réel d\'annonceurs tiers.',
                        'pl' => 'Używany przez Facebooka do dostarczania serii produktów reklamowych, takich jak licytowanie w czasie rzeczywistym od reklamodawców zewnętrznych.',
                        'ro' => 'Folosit de Facebook pentru a livra o serie de produse publicitare, cum ar fi licitarea în timp real de la agenții de publicitate terți.',
                        'pt' => 'Usado pelo Facebook para entregar uma série de produtos de publicidade, como lances em tempo real de anunciantes terceiros.',
                        'br' => 'Usado pelo Facebook para entregar uma série de produtos de publicidade, como lances em tempo real de anunciantes terceiros.',
                        'sk' => 'Používa Facebook na dodanie radu reklamných produktov, ako napríklad ponúkanie cien v reálnom čase od inzerentov tretích strán.',
                        'nl' => 'Gebruikt door Facebook om een reeks advertentieproducten te leveren, zoals realtime bieden van externe adverteerders.',
                        'de' => 'Wird von Facebook genutzt, um eine Reihe von Werbeprodukten anzuzeigen, zum Beispiel Echtzeitgebote dritter Werbetreibender.',
                        'gr' => 'Χρησιμοποιείται από το Facebook για την παράδοση μιας σειράς διαφημιστικών προϊόντων, όπως η υποβολή προσφορών σε πραγματικό χρόνο από διαφημιστές τρίτων.',
                        'it' => 'Utilizzato da Facebook per fornire una serie di prodotti pubblicitari come offerte in tempo reale da inserzionisti terzi.',
                        'si' => 'Facebook uporablja Facebook za zagotavljanje vrste oglasnih izdelkov, kot so ponudbe v realnem času od neodvisnih oglaševalcev.',
                        'da' => 'Anvendes af Facebook til at levere forskellige reklame-tjenester, herunder realtids-bud fra tredjeparts-annoncører.',
                        'no' => 'Brukt av Facebook for å levere en serie reklameprodukter som for eksempel sanntidsbud fra tredjepartsannonsører.',
                        'cs' => 'Používá Facebook k poskytování řady reklamních produktů, jako je nabízení cen v reálném čase od inzerentů třetích stran.',
                        'hu' => 'A Facebook egy sor olyan reklámtermék szállítására használja, mint például valós idejű ajánlattétel harmadik fél hirdetőitől.',
                        'sv' => 'Används av Facebook för att leverera en serie reklamprodukter såsom budgivning i realtid från tredjepartsannonsörer.',
                    ],
                    'expiry' => [
                        'en' => '3 months',
                        'es' => '3 meses',
                        'ag' => '3 meses',
                        'cb' => '3 meses',
                        'mx' => '3 meses',
                        'fr' => '3 mois',
                        'qc' => '3 mois',
                        'pl' => '3 miesiące',
                        'ro' => '3 luni',
                        'pt' => '3 meses',
                        'br' => '3 meses',
                        'sk' => '3 mesiace',
                        'nl' => '3 maanden',
                        'de' => '3 Monate',
                        'gr' => '3 μήνες',
                        'it' => '3 mesi',
                        'si' => '3 meseca',
                        'da' => '3 mdr.',
                        'no' => '3 måneder',
                        'cs' => '3 měsíců',
                        'hu' => '3 hónap',
                        'sv' => '3 månader',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [
                        'pspixel',
                        'facebookconversionpixel',
                        'ps_facebook',
                    ],
                    'name' => 'tr',
                    'provider' => 'Facebook',
                    'provider_url' => 'https://www.facebook.com/policies/cookies/',
                    'purpose' => [
                        'en' => 'Used by Facebook to deliver a series of advertisement products such as real time bidding from third party advertisers.',
                        'es' => 'Utilizada por Facebook para proporcionar una serie de productos publicitarios como pujas en tiempo real de terceros anunciantes.',
                        'ag' => 'Utilizada por Facebook para proporcionar una serie de productos publicitarios como pujas en tiempo real de terceros anunciantes.',
                        'cb' => 'Utilizada por Facebook para proporcionar una serie de productos publicitarios como pujas en tiempo real de terceros anunciantes.',
                        'mx' => 'Utilizada por Facebook para proporcionar una serie de productos publicitarios como pujas en tiempo real de terceros anunciantes.',
                        'fr' => 'Utilisé par Facebook pour fournir une série de produits publicitaires tels que les offres en temps réel d\'annonceurs tiers.',
                        'qc' => 'Utilisé par Facebook pour fournir une série de produits publicitaires tels que les offres en temps réel d\'annonceurs tiers.',
                        'pl' => 'Używany przez Facebooka do dostarczania serii produktów reklamowych, takich jak licytowanie w czasie rzeczywistym od reklamodawców zewnętrznych.',
                        'ro' => 'Folosit de Facebook pentru a livra o serie de produse publicitare, cum ar fi licitarea în timp real de la agenții de publicitate terți.',
                        'pt' => 'Usado pelo Facebook para entregar uma série de produtos de publicidade, como lances em tempo real de anunciantes terceiros.',
                        'br' => 'Usado pelo Facebook para entregar uma série de produtos de publicidade, como lances em tempo real de anunciantes terceiros.',
                        'sk' => 'Používa Facebook na dodanie radu reklamných produktov, ako napríklad ponúkanie cien v reálnom čase od inzerentov tretích strán.',
                        'nl' => 'Gebruikt door Facebook om een reeks advertentieproducten te leveren, zoals realtime bieden van externe adverteerders.',
                        'de' => 'Wird von Facebook genutzt, um eine Reihe von Werbeprodukten anzuzeigen, zum Beispiel Echtzeitgebote dritter Werbetreibender.',
                        'gr' => 'Χρησιμοποιείται από το Facebook για την παράδοση μιας σειράς διαφημιστικών προϊόντων, όπως η υποβολή προσφορών σε πραγματικό χρόνο από διαφημιστές τρίτων.',
                        'it' => 'Utilizzato da Facebook per fornire una serie di prodotti pubblicitari come offerte in tempo reale da inserzionisti terzi.',
                        'si' => 'Facebook uporablja Facebook za zagotavljanje vrste oglasnih izdelkov, kot so ponudbe v realnem času od neodvisnih oglaševalcev.',
                        'da' => 'Anvendes af Facebook til at levere forskellige reklame-tjenester, herunder realtids-bud fra tredjeparts-annoncører.',
                        'no' => 'Brukt av Facebook for å levere en serie reklameprodukter som for eksempel sanntidsbud fra tredjepartsannonsører.',
                        'cs' => 'Používá Facebook k poskytování řady reklamních produktů, jako je nabízení cen v reálném čase od inzerentů třetích stran.',
                        'hu' => 'A Facebook egy sor olyan reklámtermék szállítására használja, mint például valós idejű ajánlattétel harmadik fél hirdetőitől.',
                        'sv' => 'Används av Facebook för att leverera en serie reklamprodukter såsom budgivning i realtid från tredjepartsannonsörer.',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    /* Kelkoo */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'kk_leadtag',
                    'provider' => 'Kelkoo',
                    'provider_url' => 'https://www.kelkoo.co.uk/company-pages/cookie-policy/',
                    'purpose' => [
                        'en' => 'Product price comparison.',
                        'es' => 'Comparación de precios de productos.',
                        'ag' => 'Comparación de precios de productos.',
                        'cb' => 'Comparación de precios de productos.',
                        'mx' => 'Comparación de precios de productos.',
                        'fr' => 'Comparaison des prix des produits.',
                        'qc' => 'Comparaison des prix des produits.',
                        'pl' => 'Porównanie cen produktów.',
                        'ro' => 'Compararea prețului produsului.',
                        'pt' => 'Comparação de preços do produto.',
                        'br' => 'Comparação de preços do produto.',
                        'sk' => 'Porovnanie cien výrobkov.',
                        'nl' => 'Productprijsvergelijking.',
                        'de' => 'Produktpreisvergleich.',
                        'gr' => 'Σύγκριση τιμών προϊόντος.',
                        'it' => 'Confronto dei prezzi dei prodotti.',
                        'si' => 'Primerjava cen izdelkov.',
                        'da' => 'Produkt prissammenligning.',
                        'no' => 'Produktprissammenligning.',
                        'cs' => 'Porovnání cen produktů.',
                        'hu' => 'Termékár-összehasonlítás.',
                        'sv' => 'Produktjämförelse.',
                    ],
                    'expiry' => [
                        'en' => '1 year',
                        'es' => '1 año',
                        'ag' => '1 año',
                        'cb' => '1 año',
                        'mx' => '1 año',
                        'fr' => '1 année',
                        'qc' => '1 année',
                        'pl' => '1 rok',
                        'ro' => '1 an',
                        'pt' => '1 ano',
                        'br' => '1 ano',
                        'sk' => '1 rok',
                        'nl' => '1 jaar',
                        'de' => '1 Jahr',
                        'gr' => '1 χρόνος',
                        'it' => '1 anno',
                        'si' => '1 leto',
                        'da' => '1 år',
                        'no' => '1 år',
                        'cs' => '1 rok',
                        'hu' => '1 év',
                        'sv' => '1 år',
                    ],
                ],
                [
                    /* Adform */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'uid',
                    'provider' => 'adform.net',
                    'provider_url' => 'https://site.adform.com/privacy-center/adform-cookies/',
                    'purpose' => [
                        'en' => 'Registers a unique user ID that recognises the user\'s browser when visiting websites that use the same ad network. The purpose is to optimise display of ads based on the user\'s movements and various ad providers\' bids for displaying user ads.',
                        'es' => 'Registra una identificación de usuario única que reconoce el navegador del usuario al visitar sitios web que usan la misma red publicitaria. El propósito es optimizar la visualización de anuncios según los movimientos del usuario y diversas campañas publicitarias de los proveedores para mostrar anuncios al usuario.',
                        'ag' => 'Registra una identificación de usuario única que reconoce el navegador del usuario al visitar sitios web que usan la misma red publicitaria. El propósito es optimizar la visualización de anuncios según los movimientos del usuario y diversas campañas publicitarias de los proveedores para mostrar anuncios al usuario.',
                        'cb' => 'Registra una identificación de usuario única que reconoce el navegador del usuario al visitar sitios web que usan la misma red publicitaria. El propósito es optimizar la visualización de anuncios según los movimientos del usuario y diversas campañas publicitarias de los proveedores para mostrar anuncios al usuario.',
                        'mx' => 'Registra una identificación de usuario única que reconoce el navegador del usuario al visitar sitios web que usan la misma red publicitaria. El propósito es optimizar la visualización de anuncios según los movimientos del usuario y diversas campañas publicitarias de los proveedores para mostrar anuncios al usuario.',
                        'fr' => 'Enregistre un identifiant utilisateur unique qui reconnaît le navigateur de l\'utilisateur lors de la visite de sites utilisant le même réseau de publicité. L\'objectif étant d\'optimiser l\'affichage des annonces publicitaires en fonction des mouvements de l\'utilisateur et des offres des différents fournisseurs de pubs.',
                        'qc' => 'Enregistre un identifiant utilisateur unique qui reconnaît le navigateur de l\'utilisateur lors de la visite de sites utilisant le même réseau de publicité. L\'objectif étant d\'optimiser l\'affichage des annonces publicitaires en fonction des mouvements de l\'utilisateur et des offres des différents fournisseurs de pubs.',
                        'pl' => 'Rejestruje unikalny identyfikator użytkownika, który rozpoznaje przeglądarkę użytkownika podczas odwiedzania witryn internetowych korzystających z tej samej sieci reklamowej. Celem jest optymalizacja wyświetlania reklam na podstawie ruchów użytkownika i ofert różnych dostawców reklam za wyświetlanie reklam użytkownika.',
                        'ro' => 'Înregistrează un ID de utilizator unic care recunoaște browserul utilizatorului atunci când vizitează site-uri web care utilizează aceeași rețea publicitară. Scopul este de a optimiza afișarea anunțurilor pe baza mișcărilor utilizatorului și a diverselor oferte de furnizori de anunțuri pentru afișarea anunțurilor utilizatorilor.',
                        'pt' => 'Registra uma ID de usuário exclusiva que reconhece o navegador do usuário ao visitar sites que usam a mesma rede de anunciantes. O objetivo é otimizar a exibição de anúncios com base nos movimentos do usuário e vários lances de fornecedores de anúncios para exibir os anúncios do usuário.',
                        'br' => 'Registra uma ID de usuário exclusiva que reconhece o navegador do usuário ao visitar sites que usam a mesma rede de anunciantes. O objetivo é otimizar a exibição de anúncios com base nos movimentos do usuário e vários lances de fornecedores de anúncios para exibir os anúncios do usuário.',
                        'sk' => 'Zaregistruje jedinečné ID používateľa, ktoré rozpozná prehliadač používateľa pri návšteve webových stránok, ktoré používajú rovnakú reklamnú sieť. Účelom je optimalizovať zobrazovanie reklám na základe pohybov používateľov a ponúk rôznych poskytovateľov reklám na zobrazovanie reklám používateľov.',
                        'nl' => 'Registreert een uniek gebruikers-ID dat de browser van de gebruiker herkent wanneer websites worden bezocht die hetzelfde advertentienetwerk gebruiken. Het doel is om de weergave van advertenties te optimaliseren op basis van de bewegingen van de gebruiker en de biedingen van verschillende advertentieproviders voor het weergeven van gebruikersadvertenties.',
                        'de' => 'Registriert eine eindeutige Benutzer-ID, die der Browser des Benutzers bei einem Besuch von Websites erkennt, die das gleiche Werbenetzwerk verwenden. Der Zweck ist die Optimierung der Anzeige von Werbung auf Basis von Bewegungen des Benutzers und verschiedener Angebote von Werbeträgern für die Anzeige von Benutzerwerbung.',
                        'gr' => 'Καταγράφει ένα μοναδικό αναγνωριστικό χρήστη που αναγνωρίζει το πρόγραμμα περιήγησης του χρήστη όταν επισκέπτεται ιστότοπους που χρησιμοποιούν το ίδιο δίκτυο διαφημίσεων. Ο σκοπός είναι η βελτιστοποίηση της προβολής των διαφημίσεων με βάση τις κινήσεις του χρήστη και διάφορες προσφορές παρόχων διαφημίσεων για την προβολή διαφημίσεων χρήστη.',
                        'it' => 'Registra un ID utente univoco che riconosce il browser dell\'utente durante la visita di siti internet che utilizzano la stessa rete pubblicitaria. Lo scopo è di ottimizzare la visualizzazione di annunci pubblicitari sulla base dei movimenti dell\'utente e dei parametri di visualizzazione degli annunci agli utenti da parte dei fornitori di pubblicità.',
                        'si' => 'Registrira edinstven ID uporabnika, ki prepozna brskalnik uporabnika med obiskom spletnih mest, ki uporabljajo isto oglaševalsko omrežje. Namen je optimizirati prikaz oglasov na podlagi uporabnikovih gibanj in ponudb različnih ponudnikov oglasov za prikaz uporabniških oglasov.',
                        'da' => 'Registerer et unikt bruger-ID, som genkender brugerens browser ved besøg på hjemmesider, der anvender samme annoncenetværk. Formålet er at optimere visning af annoncer ud fra brugerens adfærd kombineret med forskellige annoncørers bud på at vise annoncer for brugeren.',
                        'no' => 'Registrerer en unik bruker-ID som gjenkjenner brukerens nettleser når du besøker nettsteder som bruker samme annonsenettverk. Hensikten er å optimalisere visning av annonser basert på brukerens bevegelser og ulike annonseleverandører for å vise brukerannonser.',
                        'cs' => 'Zaregistruje jedinečné ID uživatele, které rozpozná prohlížeč uživatele při návštěvě webových stránek, které používají stejnou reklamní síť. Účelem je optimalizovat zobrazování reklam na základě pohybu uživatele a nabídek různých poskytovatelů reklam pro zobrazování reklam uživatele.',
                        'hu' => 'Egyedi felhasználói azonosítót regisztrál, amely felismeri a felhasználó böngészőjét, amikor ugyanazon hirdetési hálózatot használó webhelyeket látogat. A cél a hirdetések megjelenítésének optimalizálása a felhasználó mozgása és a különböző hirdetési szolgáltatók felhasználói hirdetések megjelenítésére vonatkozó ajánlatai alapján.',
                        'sv' => 'Registrerar ett unikt användar-ID som känner igen användarens webbläsare när man besöker webbplatser som använder samma annonsnätverk. Syftet är att optimera visningen av annonser baserat på användarens rörelser och olika annonsleverantörers bud för att visa användarannonser.',
                    ],
                    'expiry' => [
                        'en' => '2 months',
                        'es' => '2 meses',
                        'ag' => '2 meses',
                        'cb' => '2 meses',
                        'mx' => '2 meses',
                        'fr' => '2 mois',
                        'qc' => '2 mois',
                        'pl' => '2 miesiące',
                        'ro' => '2 luni',
                        'pt' => '2 meses',
                        'br' => '2 meses',
                        'sk' => '2 mesiace',
                        'nl' => '2 maanden',
                        'de' => '2 Monate',
                        'gr' => '2 μήνες',
                        'it' => '2 mesi',
                        'si' => '2 meseca',
                        'da' => '2 mdr.',
                        'no' => '2 måneder',
                        'cs' => '2 měsíců',
                        'hu' => '2 hónap',
                        'sv' => '2 månader',
                    ],
                ],
                [
                    /* Adform */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'C',
                    'provider' => 'adform.net',
                    'provider_url' => 'https://site.adform.com/privacy-center/adform-cookies/',
                    'purpose' => [
                        'en' => 'Used to check if the user\'s browser supports cookies.',
                        'es' => 'Utilizada para comprobar si el navegador del usuario admite cookies.',
                        'ag' => 'Utilizada para comprobar si el navegador del usuario admite cookies.',
                        'cb' => 'Utilizada para comprobar si el navegador del usuario admite cookies.',
                        'mx' => 'Utilizada para comprobar si el navegador del usuario admite cookies.',
                        'fr' => 'Utilisé pour vérifier si le navigateur de l\'utilisateur accepte les cookies.',
                        'qc' => 'Utilisé pour vérifier si le navigateur de l\'utilisateur accepte les cookies.',
                        'pl' => 'Służy do sprawdzenia, czy przeglądarka użytkownika obsługuje pliki cookies.',
                        'ro' => 'Folosit pentru a verifica dacă browserul utilizatorului acceptă cookie-uri.',
                        'pt' => 'Usado para verificar se o navegador do usuário oferece suporte a cookies.',
                        'br' => 'Usado para verificar se o navegador do usuário oferece suporte a cookies.',
                        'sk' => 'Slúži na kontrolu, či prehliadač používateľa podporuje súbory cookie.',
                        'nl' => 'Gebruikt om te controleren of de browser van de gebruiker cookies ondersteunt.',
                        'de' => 'Verwendet, um zu überprüfen, ob der Browser des Benutzers Cookies unterstützt.',
                        'gr' => 'Χρησιμοποιείται για να ελέγξει αν το πρόγραμμα περιήγησης του χρήστη υποστηρίζει cookie.',
                        'it' => 'Utilizzato per verificare se il browser dell\'utente supporta i cookie.',
                        'si' => 'Uporablja se za preverjanje, ali uporabniški brskalnik podpira piškotke.',
                        'da' => 'Anvendes til at tjekke om brugerens browser understøtter cookies.',
                        'no' => 'Brukes til å sjekke om brukerens nettleser støtter informasjonskapsler.',
                        'cs' => 'Slouží ke kontrole, zda prohlížeč uživatele podporuje soubory cookie.',
                        'hu' => 'Annak ellenőrzésére szolgál, hogy a felhasználó böngészője támogatja-e a sütiket.',
                        'sv' => 'Används för att kontrollera om användarens webbläsare stöder cookies.',
                    ],
                    'expiry' => [
                        'en' => '30 days',
                        'es' => '30 días',
                        'ag' => '30 días',
                        'cb' => '30 días',
                        'mx' => '30 días',
                        'fr' => '30 jours',
                        'qc' => '30 jours',
                        'pl' => '30 dni',
                        'ro' => '30 de zile',
                        'pt' => '30 dias',
                        'br' => '30 dias',
                        'sk' => '30 dní',
                        'nl' => '30 dagen',
                        'de' => '30 Tage',
                        'gr' => '30 μέρες',
                        'it' => '30 giorni',
                        'si' => '30 dni',
                        'da' => '30 dage',
                        'no' => '30 dager',
                        'cs' => '30 dager',
                        'hu' => '30 nap',
                        'sv' => '30 dagar',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => ['criteoonetag'],
                    'name' => 'uid',
                    'provider' => 'Criteo',
                    'provider_url' => 'https://www.criteo.com/privacy/corporate-privacy-policy/',
                    'purpose' => [
                        'en' => 'Collects visitor data related to the user\'s visits to the website, such as the number of visits, average time spent on the website and what pages have been loaded, with the purpose of displaying targeted ads.',
                        'es' => 'Recopila datos de visitantes relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo promedio que pasa en el sitio web y las páginas que se han cargado, con el propósito de mostrar anuncios dirigidos.',
                        'ag' => 'Recopila datos de visitantes relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo promedio que pasa en el sitio web y las páginas que se han cargado, con el propósito de mostrar anuncios dirigidos.',
                        'cb' => 'Recopila datos de visitantes relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo promedio que pasa en el sitio web y las páginas que se han cargado, con el propósito de mostrar anuncios dirigidos.',
                        'mx' => 'Recopila datos de visitantes relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo promedio que pasa en el sitio web y las páginas que se han cargado, con el propósito de mostrar anuncios dirigidos.',
                        'fr' => 'Collecte des données des visiteurs liées aux visites de l\'utilisateur sur le site Web, telles que le nombre de visites, le temps moyen passé sur le site Web et les pages chargées, dans le but d\'afficher des publicités ciblées.',
                        'qc' => 'Collecte des données des visiteurs liées aux visites de l\'utilisateur sur le site Web, telles que le nombre de visites, le temps moyen passé sur le site Web et les pages chargées, dans le but d\'afficher des publicités ciblées.',
                        'pl' => 'Gromadzi dane o odwiedzających związane z wizytami użytkownika w serwisie, takie jak liczba odwiedzin, średni czas spędzony w serwisie i jakie strony zostały wczytane, w celu wyświetlania ukierunkowanych reklam.',
                        'ro' => 'Colectează date despre vizitatori legate de vizitele utilizatorului pe site, cum ar fi numărul de vizite, timpul mediu petrecut pe site și ce pagini au fost încărcate, cu scopul de a afișa reclame direcționate.',
                        'pt' => 'Coleta dados do visitante relacionados às visitas do usuário ao site, como número de visitas, tempo médio de permanência no site e quais páginas foram carregadas, com o objetivo de exibir anúncios direcionados.',
                        'br' => 'Coleta dados do visitante relacionados às visitas do usuário ao site, como número de visitas, tempo médio de permanência no site e quais páginas foram carregadas, com o objetivo de exibir anúncios direcionados.',
                        'sk' => 'Zhromažďuje údaje o návštevníkoch súvisiace s návštevami webových stránok používateľom, napríklad počet návštev, priemerný čas strávený na webových stránkach a načítané stránky, aby sa zobrazili cielené reklamy.',
                        'nl' => 'Verzamelt bezoekersgegevens met betrekking tot de bezoeken van de gebruiker aan de website, zoals het aantal bezoeken, de gemiddelde tijd die op de website is doorgebracht en welke pagina\'s zijn geladen, met als doel gerichte advertenties weer te geven.',
                        'de' => 'Sammelt Besucherdaten im Zusammenhang mit den Besuchen des Benutzers auf der Website, z. B. die Anzahl der Besuche, die durchschnittliche Zeit, die auf der Website verbracht wurde, und die Seiten, die geladen wurden, um zielgerichtete Anzeigen anzuzeigen.',
                        'gr' => 'Συλλέγει δεδομένα επισκεπτών που σχετίζονται με τις επισκέψεις του χρήστη στον ιστότοπο, όπως ο αριθμός των επισκέψεων, ο μέσος χρόνος που αφιερώνεται στον ιστότοπο και ποιες σελίδες έχουν φορτωθεί, με σκοπό την προβολή στοχευμένων διαφημίσεων.',
                        'it' => 'Raccoglie i dati dei visitatori relativi alle visite dell\'utente al sito web, come il numero di visite, il tempo medio trascorso sul sito web e quali pagine sono state caricate, con lo scopo di visualizzare annunci mirati.',
                        'si' => 'Zbira podatke o obiskovalcih, povezane z obiski uporabnika na spletnem mestu, kot so število obiskov, povprečni čas, ki ga je preživel na spletnem mestu in katere strani so bile naložene, z namenom prikazovanja ciljanih oglasov.',
                        'da' => 'Indsamler besøgsdata relateret til brugerens besøg på webstedet, såsom antallet af besøg, den gennemsnitlige tid brugt på webstedet og hvilke sider der er blevet indlæst med det formål at vise målrettede annoncer.',
                        'no' => 'Samler inn besøksdata relatert til brukerens besøk på nettstedet, for eksempel antall besøk, gjennomsnittlig tid brukt på nettstedet og hvilke sider som er lastet inn, med det formål å vise målrettede annonser.',
                        'cs' => 'Shromažďuje údaje o návštěvnících související s návštěvami uživatele na webu, jako je počet návštěv, průměrný čas strávený na webu a jaké stránky byly načteny, za účelem zobrazení cílených reklam.',
                        'hu' => 'Célzott hirdetések megjelenítésének céljából gyűjti a látogatóknak a felhasználó webhelyre tett látogatásával kapcsolatos adatokat, például a látogatások számát, a webhelyen eltöltött átlagos időt és az oldalakat.',
                        'sv' => 'Samlar in besökardata relaterade till användarens besök på webbplatsen, till exempel antalet besök, genomsnittlig tid på webbplatsen och vilka sidor som har laddats, i syfte att visa riktade annonser.',
                    ],
                    'expiry' => [
                        'en' => '30 days',
                        'es' => '30 días',
                        'ag' => '30 días',
                        'cb' => '30 días',
                        'mx' => '30 días',
                        'fr' => '30 jours',
                        'qc' => '30 jours',
                        'pl' => '30 dni',
                        'ro' => '30 de zile',
                        'pt' => '30 dias',
                        'br' => '30 dias',
                        'sk' => '30 dní',
                        'nl' => '30 dagen',
                        'de' => '30 Tage',
                        'gr' => '30 μέρες',
                        'it' => '30 giorni',
                        'si' => '30 dni',
                        'da' => '30 dage',
                        'no' => '30 dager',
                        'cs' => '30 dager',
                        'hu' => '30 nap',
                        'sv' => '30 dagar',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => ['cartsguru'],
                    'name' => 'trkcg_fid',
                    'provider' => 'Carts Guru',
                    'provider_url' => 'https://www.carts.guru/privacy-policy/website-privacy-policy',
                    'purpose' => [
                        'en' => 'Identify the user ID for remarketing services',
                        'es' => 'Identifica el ID de usuario para servicios de remarketing',
                        'ag' => 'Identifica el ID de usuario para servicios de remarketing',
                        'cb' => 'Identifica el ID de usuario para servicios de remarketing',
                        'mx' => 'Identifica el ID de usuario para servicios de remarketing',
                        'fr' => 'Identifiez l\'ID utilisateur des services de remarketing',
                        'qc' => 'Identifiez l\'ID utilisateur des services de remarketing',
                        'pl' => 'Zidentyfikuj identyfikator użytkownika usług remarketingowych',
                        'ro' => 'Identificați ID-ul de utilizator pentru serviciile de remarketing',
                        'pt' => 'Identifique o ID do usuário para serviços de remarketing',
                        'br' => 'Identifique o ID do usuário para serviços de remarketing',
                        'sk' => 'Identifikujte ID používateľa pre remarketingové služby',
                        'nl' => 'Identificeer de gebruikers-ID voor remarketingdiensten',
                        'de' => 'Identifizieren Sie die Benutzer-ID für Remarketing-Dienste',
                        'gr' => 'Προσδιορίστε το αναγνωριστικό χρήστη για υπηρεσίες επαναληπτικού μάρκετινγκ',
                        'it' => 'Identifica l\'ID utente per i servizi di remarketing',
                        'si' => 'Določite ID uporabnika za storitve ponovnega trženja',
                        'da' => 'Identificer bruger-ID til remarketingtjenester',
                        'no' => 'Identifiser bruker-ID for remarketingtjenester',
                        'cs' => 'Identifikujte ID uživatele pro remarketingové služby',
                        'hu' => 'Azonosítsa a remarketingszolgáltatások felhasználói azonosítóját',
                        'sv' => 'Identifiera användar-ID för remarketingtjänster',
                    ],
                    'expiry' => [
                        'en' => '30 days',
                        'es' => '30 días',
                        'ag' => '30 días',
                        'cb' => '30 días',
                        'mx' => '30 días',
                        'fr' => '30 jours',
                        'qc' => '30 jours',
                        'pl' => '30 dni',
                        'ro' => '30 de zile',
                        'pt' => '30 dias',
                        'br' => '30 dias',
                        'sk' => '30 dní',
                        'nl' => '30 dagen',
                        'de' => '30 Tage',
                        'gr' => '30 μέρες',
                        'it' => '30 giorni',
                        'si' => '30 dni',
                        'da' => '30 dage',
                        'no' => '30 dager',
                        'cs' => '30 dager',
                        'hu' => '30 nap',
                        'sv' => '30 dagar',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => ['cartsguru'],
                    'name' => 'trkcg_sid',
                    'provider' => 'Carts Guru',
                    'provider_url' => 'https://www.carts.guru/privacy-policy/website-privacy-policy',
                    'purpose' => [
                        'en' => 'Identify the session for remarketing services',
                        'es' => 'Identifica la sesión para servicios de remarketing',
                        'ag' => 'Identifica la sesión para servicios de remarketing',
                        'cb' => 'Identifica la sesión para servicios de remarketing',
                        'mx' => 'Identifica la sesión para servicios de remarketing',
                        'fr' => 'Identifier la session pour les services de remarketing',
                        'qc' => 'Identifier la session pour les services de remarketing',
                        'pl' => 'Zidentyfikuj sesję dla usług remarketingowych',
                        'ro' => 'Identificați sesiunea pentru serviciile de remarketing',
                        'pt' => 'Identifique a sessão para serviços de remarketing',
                        'br' => 'Identifique a sessão para serviços de remarketing',
                        'sk' => 'Identifikujte reláciu pre remarketingové služby',
                        'nl' => 'Identificeer de sessie voor remarketingdiensten',
                        'de' => 'Identifizieren Sie die Sitzung für Remarketing-Dienste',
                        'gr' => 'Προσδιορίστε την περίοδο σύνδεσης για υπηρεσίες επαναληπτικού μάρκετινγκ',
                        'it' => 'Identifica la sessione per i servizi di remarketing',
                        'si' => 'Določite sejo za storitve ponovnega trženja',
                        'da' => 'Identificer sessionen til remarketingtjenester',
                        'no' => 'Identifiser økten for remarketingtjenester',
                        'cs' => 'Určete relaci pro remarketingové služby',
                        'hu' => 'Azonosítsa a munkamenetet a remarketingszolgáltatásokhoz',
                        'sv' => 'Identifiera sessionen för remarketingtjänster',
                    ],
                    'expiry' => [
                        'en' => '30 minutes',
                        'es' => '30 minutos',
                        'ag' => '30 minutos',
                        'cb' => '30 minutos',
                        'mx' => '30 minutos',
                        'fr' => '30 minutes',
                        'qc' => '30 minutes',
                        'pl' => '30 minut',
                        'ro' => '30 minute',
                        'pt' => '30 minutos',
                        'br' => '30 minutos',
                        'sk' => '30 minút',
                        'nl' => '30 minuten',
                        'de' => '30 Minuten',
                        'gr' => '30 λεπτά',
                        'it' => '30 minuti',
                        'si' => '30 minut',
                        'da' => '30 minutter',
                        'no' => '30 minutter',
                        'cs' => '30 minut',
                        'hu' => '30 perc',
                        'sv' => '30 minuter',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => ['sendinblue'],
                    'name' => 'sib_cuid',
                    'provider' => 'Sendinblue',
                    'provider_url' => 'https://sendinblue.com/legal/privacypolicy/',
                    'purpose' => [
                        'en' => 'Collects information on the user\'s website navigation and preferences - This is used to target potential newsletter based upon this information.',
                        'es' => 'Recoge información de la navegación del usuario en la web y sus preferencias. Se usa para dirigir un bolentín potencial a raíz de esta información.',
                        'ag' => 'Recoge información de la navegación del usuario en la web y sus preferencias. Se usa para dirigir un bolentín potencial a raíz de esta información.',
                        'cb' => 'Recoge información de la navegación del usuario en la web y sus preferencias. Se usa para dirigir un bolentín potencial a raíz de esta información.',
                        'mx' => 'Recoge información de la navegación del usuario en la web y sus preferencias. Se usa para dirigir un bolentín potencial a raíz de esta información.',
                        'fr' => 'Collecte les informations sur la navigation et les préférences des utilisateurs sur le site web – Cela est utilisé pour cibler une éventuelle newsletter en fonction de ces informations.',
                        'qc' => 'Collecte les informations sur la navigation et les préférences des utilisateurs sur le site web – Cela est utilisé pour cibler une éventuelle newsletter en fonction de ces informations.',
                        'pl' => 'Zbiera informacje o nawigacji i preferencjach użytkownika w witrynie - służy do kierowania potencjalnego newslettera na podstawie tych informacji.',
                        'ro' => 'Colectează informații despre navigarea și preferințele site-ului utilizatorului - Acesta este utilizat pentru a viza potențialul buletin informativ pe baza acestor informații.',
                        'pt' => 'Coleta informações sobre a navegação e as preferências do usuário no site - Isso é usado para direcionar o boletim informativo potencial com base nessas informações.',
                        'br' => 'Coleta informações sobre a navegação e as preferências do usuário no site - Isso é usado para direcionar o boletim informativo potencial com base nessas informações.',
                        'sk' => 'Zhromažďuje informácie o navigácii a preferenciách na webových stránkach používateľa - používa sa na zacielenie potenciálneho bulletinu na základe týchto informácií.',
                        'nl' => 'Verzamelt informatie over de navigatie en voorkeuren van de websitebezoeker - Dit wordt gebruikt om potentiële nieuwsbrief te targeten op basis van deze informatie.',
                        'de' => 'Sammelt Informationen über die Navigation und die Präferenzen der Benutzer auf der Website - Dies wird verwendet, um potenzielle Newsletter auf der Grundlage dieser Informationen auszurichten.',
                        'gr' => 'Συλλέγει πληροφορίες σχετικά με την πλοήγηση και τις προτιμήσεις του ιστότοπου του χρήστη - Χρησιμοποιείται για τη στόχευση πιθανών ενημερωτικών δελτίων βάσει αυτών των πληροφοριών.',
                        'it' => 'Raccoglie informazioni sulla navigazione dell\'utente sul sito e sulle sue preferenze - Viene utilizzato per indirizzare potenziali newsletter sulla base di queste informazioni.',
                        'si' => 'Zbira informacije o navigaciji in nastavitvah uporabnikovega spletnega mesta - to se uporablja za ciljanje potencialnih novic na podlagi teh informacij.',
                        'da' => 'Indsamler data om brugerens navigation og præferencer på hjemmesiden - Benyttes til at målrette potentielle nyhedsbreve baseret på disse data.',
                        'no' => 'Samler informasjon om brukerens nettstednavigasjon og preferanser - Dette brukes til å målrette potensielt nyhetsbrev basert på denne informasjonen.',
                        'cs' => 'Shromažďuje informace o navigaci a preferencích na webových stránkách uživatele - používá se k zacílení potenciálního zpravodaje na základě těchto informací.',
                        'hu' => 'Információt gyűjt a felhasználó webhelyének navigációjáról és preferenciáiról - Ez arra szolgál, hogy ezen információk alapján megcélozza a lehetséges hírleveleket.',
                        'sv' => 'Samlar in information om användarens webbplatsnavigering och preferenser - Detta används för att rikta dig till potentiellt nyhetsbrev baserat på denna information.',
                    ],
                    'expiry' => [
                        'en' => '1 year',
                        'es' => '1 año',
                        'ag' => '1 año',
                        'cb' => '1 año',
                        'mx' => '1 año',
                        'fr' => '1 année',
                        'qc' => '1 année',
                        'pl' => '1 rok',
                        'ro' => '1 an',
                        'pt' => '1 ano',
                        'br' => '1 ano',
                        'sk' => '1 rok',
                        'nl' => '1 jaar',
                        'de' => '1 Jahr',
                        'gr' => '1 χρόνος',
                        'it' => '1 anno',
                        'si' => '1 leto',
                        'da' => '1 år',
                        'no' => '1 år',
                        'cs' => '1 rok',
                        'hu' => '1 év',
                        'sv' => '1 år',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [],
                    'name' => 'IDE',
                    'provider' => 'doubleclick.net',
                    'provider_url' => 'http://doubleclick.net/',
                    'purpose' => [
                        'en' => 'Used by Google DoubleClick to register and report the website user\'s actions after viewing or clicking one of the advertiser\'s ads with the purpose of measuring the efficacy of an ad and to present targeted ads to the user.',
                        'es' => 'Utilizada por Google DoubleClick para registrar e informar sobre las acciones del usuario en el sitio web tras visualizar o hacer clic en uno de los anuncios del anunciante con el propósito de medir la eficacia de un anuncio y presentar anuncios específicos para el usuario.',
                        'ag' => 'Utilizada por Google DoubleClick para registrar e informar sobre las acciones del usuario en el sitio web tras visualizar o hacer clic en uno de los anuncios del anunciante con el propósito de medir la eficacia de un anuncio y presentar anuncios específicos para el usuario.',
                        'cb' => 'Utilizada por Google DoubleClick para registrar e informar sobre las acciones del usuario en el sitio web tras visualizar o hacer clic en uno de los anuncios del anunciante con el propósito de medir la eficacia de un anuncio y presentar anuncios específicos para el usuario.',
                        'mx' => 'Utilizada por Google DoubleClick para registrar e informar sobre las acciones del usuario en el sitio web tras visualizar o hacer clic en uno de los anuncios del anunciante con el propósito de medir la eficacia de un anuncio y presentar anuncios específicos para el usuario.',
                        'fr' => 'Utilisé par Google DoubleClick pour enregistrer et signaler les actions de l\'utilisateur du site après qu\'il ait vu ou cliqué sur une des pubs de l\'annonceur dans le but de mesurer l\'efficacité et de présenter des annonces publicitaires ciblées à l\'utilisateur.',
                        'qc' => 'Utilisé par Google DoubleClick pour enregistrer et signaler les actions de l\'utilisateur du site après qu\'il ait vu ou cliqué sur une des pubs de l\'annonceur dans le but de mesurer l\'efficacité et de présenter des annonces publicitaires ciblées à l\'utilisateur.',
                        'pl' => 'Używany przez Google DoubleClick do rejestrowania i raportowania działań użytkownika serwisu internetowego po obejrzeniu lub kliknięciu jednej z reklam ogłoszeniodawcy w celu pomiaru skuteczności reklamy i przedstawienia użytkownikowi ukierunkowanych reklam.',
                        'ro' => 'Folosit de Google DoubleClick pentru a înregistra și a raporta acțiunile utilizatorului site-ului web după ce a vizionat sau a făcut clic pe unul dintre anunțurile agentului de publicitate, cu scopul de a măsura eficiența unui anunț și de a prezenta utilizatorului anunțuri direcționate.',
                        'pt' => 'Usado pelo Google DoubleClick para registrar e relatar as ações do usuário do site após visualizar ou clicar em um dos anúncios do anunciante com o objetivo de medir a eficácia de um anúncio e apresentar anúncios direcionados ao usuário.',
                        'br' => 'Usado pelo Google DoubleClick para registrar e relatar as ações do usuário do site após visualizar ou clicar em um dos anúncios do anunciante com o objetivo de medir a eficácia de um anúncio e apresentar anúncios direcionados ao usuário.',
                        'sk' => 'Používa Google DoubleClick na registráciu a hlásenie akcií používateľa webových stránok po zobrazení alebo kliknutí na jednu z reklám inzerenta na účely merania účinnosti reklamy a na predstavenie používateľovi cielených reklám.',
                        'nl' => 'Gebruikt door Google DoubleClick om de acties van de websitegebruiker te registreren en te rapporteren na het bekijken of klikken op een van de advertenties van de adverteerder met het doel de effectiviteit van een advertentie te meten en om gerichte advertenties aan de gebruiker te presenteren.',
                        'de' => 'Verwendet von Google DoubleClick, um die Handlungen des Benutzers auf der Webseite nach der Anzeige oder dem Klicken auf eine der Anzeigen des Anbieters zu registrieren und zu melden, mit dem Zweck der Messung der Wirksamkeit einer Werbung und der Anzeige zielgerichteter Werbung für den Benutzer.',
                        'gr' => 'Χρησιμοποιείται από το Google DoubleClick για να εγγραφεί και να αναφέρει τις ενέργειες του χρήστη του ιστότοπου μετά την προβολή ή κλικ σε μία από τις διαφημίσεις του διαφημιζόμενου με σκοπό τη μέτρηση της αποτελεσματικότητας μιας διαφήμισης και την παρουσίαση στοχευμένων διαφημίσεων στον χρήστη.',
                        'it' => 'Utilizzato da Google DoubleClick per registrare e produrre resoconti sulle azioni dell\'utente sul sito dopo aver visualizzato o cliccato una delle pubblicità dell\'inserzionista al fine di misurare l\'efficacia di una pubblicità e presentare pubblicità mirata all\'utente.',
                        'si' => 'Google DoubleClick ga uporablja za registracijo in poročanje o dejanjih uporabnika spletnega mesta po ogledu ali kliku enega od oglasov oglaševalca z namenom merjenja učinkovitosti oglasa in predstavitve ciljnih oglasov uporabniku.',
                        'da' => 'Anvendes af Google DoubleClick til at registrere og rapportere om hjemmesidebrugerens handlinger efter at have set eller klikket på en af annoncørens annoncer. Formålet er at måle effekten af en annonce samt at målrette annoncer til brugeren.',
                        'no' => 'Brukt av Google DoubleClick til å registrere og rapportere handlingene til brukeren av nettstedet etter å ha sett eller klikket på en av annonsørens annonser med det formål å måle effektiviteten til en annonse og presentere målrettede annonser for brukeren.',
                        'cs' => 'Používá Google DoubleClick k registraci a hlášení akcí uživatele webových stránek po zobrazení nebo kliknutí na jednu z reklam inzerenta za účelem měření účinnosti reklamy a k zobrazení cílených reklam uživateli.',
                        'hu' => 'A Google DoubleClick használja arra, hogy regisztrálja és jelentse a webhely felhasználójának műveleteit, miután megnézte vagy rákattintott a hirdető egyik hirdetésére, a hirdetés hatékonyságának mérése és célzott hirdetések bemutatása céljából.',
                        'sv' => 'Används av Google DoubleClick för att registrera och rapportera webbplatsanvändarens åtgärder efter att ha visat eller klickat på en av annonsörens annonser i syfte att mäta effektiviteten i en annons och att presentera riktade annonser för användaren.',
                    ],
                    'expiry' => [
                        'en' => '1 year',
                        'es' => '1 año',
                        'ag' => '1 año',
                        'cb' => '1 año',
                        'mx' => '1 año',
                        'fr' => '1 année',
                        'qc' => '1 année',
                        'pl' => '1 rok',
                        'ro' => '1 an',
                        'pt' => '1 ano',
                        'br' => '1 ano',
                        'sk' => '1 rok',
                        'nl' => '1 jaar',
                        'de' => '1 Jahr',
                        'gr' => '1 χρόνος',
                        'it' => '1 anno',
                        'si' => '1 leto',
                        'da' => '1 år',
                        'no' => '1 år',
                        'cs' => '1 rok',
                        'hu' => '1 év',
                        'sv' => '1 år',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [],
                    'name' => 'NID',
                    'provider' => 'Google',
                    'provider_url' => 'https://policies.google.com/privacy',
                    'purpose' => [
                        'en' => 'Registers a unique ID that identifies a returning user\'s device. The ID is used for targeted ads.',
                        'es' => 'Registra una identificación única que identifica el dispositivo de un usuario que vuelve. La identificación se utiliza para los anuncios específicos.',
                        'ag' => 'Registra una identificación única que identifica el dispositivo de un usuario que vuelve. La identificación se utiliza para los anuncios específicos.',
                        'cb' => 'Registra una identificación única que identifica el dispositivo de un usuario que vuelve. La identificación se utiliza para los anuncios específicos.',
                        'mx' => 'Registra una identificación única que identifica el dispositivo de un usuario que vuelve. La identificación se utiliza para los anuncios específicos.',
                        'fr' => 'Enregistre un identifiant qui identifie l\'appareil de l\'utilisateur récurrent. Cet identifiant est utilisé pour des annonces ciblées.',
                        'qc' => 'Enregistre un identifiant qui identifie l\'appareil de l\'utilisateur récurrent. Cet identifiant est utilisé pour des annonces ciblées.',
                        'pl' => 'Rejestruje unikalny identyfikator, który identyfikuje urządzenie powracającego użytkownika. Identyfikator służy do kierowanych reklam.',
                        'ro' => 'Înregistrează un ID unic care identifică dispozitivul unui utilizator care se întoarce. ID-ul este utilizat pentru reclame direcționate.',
                        'pt' => 'Registra um ID exclusivo que identifica o dispositivo de um usuário recorrente. O ID é usado para anúncios direcionados.',
                        'br' => 'Registra um ID exclusivo que identifica o dispositivo de um usuário recorrente. O ID é usado para anúncios direcionados.',
                        'sk' => 'Zaregistruje jedinečný identifikátor, ktorý identifikuje zariadenie vracajúceho sa používateľa. ID sa používa pre cielené reklamy.',
                        'nl' => 'Registreert een uniek ID die het apparaat van een terugkerende gebruiker identificeert. Het ID wordt gebruikt voor gerichte advertenties.',
                        'de' => 'Registriert eine eindeutige ID, die das Gerät eines wiederkehrenden Benutzers identifiziert. Die ID wird für gezielte Werbung genutzt.',
                        'gr' => 'Καταγράφει ένα μοναδικό αναγνωριστικό που προσδιορίζει τη συσκευή ενός χρήστη που επιστρέφει. Το αναγνωριστικό χρησιμοποιείται για στοχευμένες διαφημίσεις.',
                        'it' => 'Registra un ID univoco che identifica il dispositivo dell\'utente che ritorna sul sito. L\'ID viene utilizzato per pubblicità mirate.',
                        'si' => 'Registrira enolični ID, ki identificira napravo, ki se vrača. ID se uporablja za ciljne oglase.',
                        'da' => 'Registrerer et unikt ID, der identificerer brugerens enhed ved tilbagevendende besøg. ID\'et anvendes til at målrette annoncer.',
                        'no' => 'Registrerer en unik ID som identifiserer enheten til en bruker som returnerer. ID-en brukes til målrettede annonser.',
                        'cs' => 'Zaregistruje jedinečné ID, které identifikuje zařízení vracejícího se uživatele. ID se používá pro cílené reklamy.',
                        'hu' => 'Egyedi azonosítót regisztrál, amely azonosítja a visszatérő felhasználó eszközét. Az azonosítót célzott hirdetésekhez használják.',
                        'sv' => 'Registrerar ett unikt ID som identifierar en återvändares enhet. ID används för riktade annonser.',
                    ],
                    'expiry' => [
                        'en' => '6 months',
                        'es' => '6 meses',
                        'ag' => '6 meses',
                        'cb' => '6 meses',
                        'mx' => '6 meses',
                        'fr' => '6 mois',
                        'qc' => '6 mois',
                        'pl' => '6 miesiące',
                        'ro' => '6 luni',
                        'pt' => '6 meses',
                        'br' => '6 meses',
                        'sk' => '6 mesiace',
                        'nl' => '6 maanden',
                        'de' => '6 Monate',
                        'gr' => '6 μήνες',
                        'it' => '6 mesi',
                        'si' => '6 meseca',
                        'da' => '6 mdr.',
                        'no' => '6 måneder',
                        'cs' => '6 měsíců',
                        'hu' => '6 hónap',
                        'sv' => '6 månader',
                    ],
                ],
                [
                    /* Smartlook */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'SL_C_#_KEY',
                    'provider' => 'Smartlook',
                    'provider_url' => 'https://help.smartlook.com/en/articles/3244452-privacy-policy',
                    'purpose' => [
                        'en' => 'Collects statistical data related to the user\'s website visits, such as the number of visits, average time spent on the website and what pages have been loaded. The purpose is to segment the website\'s users according to factors such as demographics and geographical location, in order to enable media and marketing agencies to structure and understand their target groups to enable customised online advertising.',
                        'es' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'ag' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'cb' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'mx' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'fr' => 'Recueille des données statistiques liées aux visites du site web de l\'utilisateur, telles que le nombre de visites, le temps moyen passé sur le site et quelles pages ont été chargées. L’objectif étant de segmenter les utilisateurs du site en fonction de facteurs tels que la démographie et la situation géographique, afin de permettre aux médias et aux agences de marketing de structurer et de comprendre leurs groupes cibles et pour être en mesure d\'afficher une publicité en ligne personnalisée.',
                        'qc' => 'Recueille des données statistiques liées aux visites du site web de l\'utilisateur, telles que le nombre de visites, le temps moyen passé sur le site et quelles pages ont été chargées. L’objectif étant de segmenter les utilisateurs du site en fonction de facteurs tels que la démographie et la situation géographique, afin de permettre aux médias et aux agences de marketing de structurer et de comprendre leurs groupes cibles et pour être en mesure d\'afficher une publicité en ligne personnalisée.',
                        'pl' => 'Zbiera dane statystyczne dotyczące odwiedzin serwisu przez użytkownika, takie jak liczba odwiedzin, średni czas spędzony w serwisie oraz jakie strony zostały wczytane. Celem jest segmentacja użytkowników witryny internetowej według czynników, takich jak dane demograficzne i położenie geograficzne, aby umożliwić agencjom medialnym i marketingowym ustrukturyzowanie i zrozumienie ich grup docelowych w celu umożliwienia dostosowanej reklamy online.',
                        'ro' => 'Colectează date statistice legate de vizitele site-ului utilizatorului, cum ar fi numărul de vizite, timpul mediu petrecut pe site și ce pagini au fost încărcate. Scopul este de a segmenta utilizatorii site-ului în funcție de factori precum demografia și locația geografică, pentru a permite agențiilor media și de marketing să își structureze și să înțeleagă grupurile țintă pentru a permite publicitate online personalizată.',
                        'pt' => 'Coleta dados estatísticos relativos às visitas do usuário ao site, como número de visitas, tempo médio de permanência no site e quais páginas foram carregadas. O objetivo é segmentar os usuários do site de acordo com fatores como dados demográficos e localização geográfica, a fim de permitir que as agências de mídia e marketing estruturem e entendam seus públicos-alvo para possibilitar a publicidade online personalizada.',
                        'br' => 'Coleta dados estatísticos relativos às visitas do usuário ao site, como número de visitas, tempo médio de permanência no site e quais páginas foram carregadas. O objetivo é segmentar os usuários do site de acordo com fatores como dados demográficos e localização geográfica, a fim de permitir que as agências de mídia e marketing estruturem e entendam seus públicos-alvo para possibilitar a publicidade online personalizada.',
                        'sk' => 'Zhromažďuje štatistické údaje týkajúce sa návštev webových stránok používateľa, ako napríklad počet návštev, priemerný čas strávený na webových stránkach a načítané stránky. Účelom je segmentovať používateľov webových stránok podľa faktorov, ako sú demografické údaje a geografické umiestnenie, aby mohli mediálne a marketingové agentúry štruktúrovať a porozumieť svojim cieľovým skupinám a umožniť tak prispôsobenú online reklamu.',
                        'nl' => 'Verzamelt statistische gegevens met betrekking tot de websitebezoeken van de gebruiker, zoals het aantal bezoeken, de gemiddelde tijd die op de website is doorgebracht en welke pagina\'s zijn geladen. Het doel is om de gebruikers van de website te segmenteren op basis van factoren zoals demografie en geografische locatie, zodat media- en marketingbureaus hun doelgroepen kunnen structureren en begrijpen om aangepaste online advertenties mogelijk te maken.',
                        'de' => 'Erfasst statistische Daten zu Website-Besuchen des Benutzers, wie z. B. die Anzahl der Besuche, durchschnittliche Verweildauer auf der Website und welche Seiten geladen wurden. Der Zweck ist die Segmentierung der Benutzer der Website nach Faktoren wie Demografie und geografische Lage, damit Medien- und Marketing-Agenturen ihre Zielgruppen strukturieren und verstehen können, um maßgeschneiderte Online-Werbung zu ermöglichen.',
                        'gr' => 'Συλλέγει στατιστικά δεδομένα που σχετίζονται με τις επισκέψεις στον ιστότοπο του χρήστη, όπως ο αριθμός των επισκέψεων, ο μέσος χρόνος που αφιερώνεται στον ιστότοπο και ποιες σελίδες έχουν φορτωθεί. Ο σκοπός είναι η τμηματοποίηση των χρηστών του ιστότοπου σύμφωνα με παράγοντες όπως τα δημογραφικά στοιχεία και η γεωγραφική θέση, προκειμένου να δοθεί η δυνατότητα στα μέσα ενημέρωσης και στα γραφεία μάρκετινγκ να δομήσουν και να κατανοήσουν τις ομάδες-στόχους τους ώστε να επιτρέψουν προσαρμοσμένες διαδικτυακές διαφημίσεις.',
                        'it' => 'Raccoglie dati statistici relativi alle visite del sito internet da parte dell\'utente, come ad esempio il numero di visite, il tempo medio speso sul sito e quali pagine sono state caricate. Lo scopo è di suddividere gli utenti del sito internet a seconda di fattori demografici e geografici, allo scopo di consentire ai media e alle agenzie marketing di strutturare e comprendere i loro gruppi target per effettuare pubblicità online personalizzate.',
                        'si' => 'Zbira statistične podatke, povezane z obiski uporabnikovega spletnega mesta, kot so število obiskov, povprečni čas, ki ga je preživel na spletnem mestu in katere strani so bile naložene. Namen je segmentirati uporabnike spletnega mesta glede na dejavnike, kot so demografski podatki in geografski položaj, da se medijem in marketinškim agencijam omogoči, da strukturirajo in razumejo svoje ciljne skupine, da omogočijo prilagojeno spletno oglaševanje.',
                        'da' => 'Indsamler statistik om brugerens besøg på hjemmesiden såsom antallet af besøg, den gennemsnitlige tid på hjemmesiden og hvilke sider der er læst. Formålet er at segmentere hjemmesidens brugere efter faktorer såsom demografi og geografi for at gøre det muligt for medier og marketingbureauer at strukturere og forstå deres målgrupper med henblik på at tilpasse online annoncering.',
                        'no' => 'Samler inn statistiske data relatert til brukerens nettstedsbesøk, for eksempel antall besøk, gjennomsnittlig tid brukt på nettstedet og hvilke sider som er lastet inn. Hensikten er å segmentere nettstedets brukere i henhold til faktorer som demografi og geografisk beliggenhet, for å gjøre media- og markedsføringsbyråer i stand til å strukturere og forstå sine målgrupper for å muliggjøre tilpasset annonsering på nettet.',
                        'cs' => 'Shromažďuje statistické údaje související s návštěvami webových stránek uživatele, například počet návštěv, průměrný čas strávený na webu a jaké stránky byly načteny. Účelem je segmentovat uživatele webu podle faktorů, jako jsou demografické údaje a zeměpisné umístění, aby mediální a marketingové agentury mohly strukturovat a porozumět jejich cílovým skupinám a umožnit tak přizpůsobenou online reklamu.',
                        'hu' => 'Gyűjti a felhasználó webhelylátogatásaira vonatkozó statisztikai adatokat, például a látogatások számát, a webhelyen töltött átlagos időt és a betöltött oldalakat. A cél a weboldal felhasználói szegmentálása olyan tényezők szerint, mint a demográfia és a földrajzi elhelyezkedés, annak érdekében, hogy a média és a marketing ügynökségek strukturálhassák és megértsék célcsoportjaikat a testreszabott online hirdetések lehetővé tétele érdekében.',
                        'sv' => 'Samlar in statistiska data relaterade till användarens webbplatsbesök, till exempel antalet besök, genomsnittlig tid på webbplatsen och vilka sidor som har laddats. Syftet är att segmentera webbplatsens användare efter faktorer som demografi och geografisk plats för att göra det möjligt för media och marknadsföringsbyråer att strukturera och förstå sina målgrupper för att möjliggöra skräddarsydd onlineannonsering.',
                    ],
                    'expiry' => [
                        'en' => '13 months',
                        'es' => '13 meses',
                        'ag' => '13 meses',
                        'cb' => '13 meses',
                        'mx' => '13 meses',
                        'fr' => '13 mois',
                        'qc' => '13 mois',
                        'pl' => '13 miesiące',
                        'ro' => '13 luni',
                        'pt' => '13 meses',
                        'br' => '13 meses',
                        'sk' => '13 mesiace',
                        'nl' => '13 maanden',
                        'de' => '13 Monate',
                        'gr' => '13 μήνες',
                        'it' => '13 mesi',
                        'si' => '13 meseca',
                        'da' => '13 mdr.',
                        'no' => '13 måneder',
                        'cs' => '13 měsíců',
                        'hu' => '13 hónap',
                        'sv' => '13 månader',
                        'sv' => '13 månader',
                    ],
                ],
                [
                    /* Smartlook */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'SL_C_#_SID',
                    'provider' => 'Smartlook',
                    'provider_url' => 'https://help.smartlook.com/en/articles/3244452-privacy-policy',
                    'purpose' => [
                        'en' => 'Collects statistical data related to the user\'s website visits, such as the number of visits, average time spent on the website and what pages have been loaded. The purpose is to segment the website\'s users according to factors such as demographics and geographical location, in order to enable media and marketing agencies to structure and understand their target groups to enable customised online advertising.',
                        'es' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'ag' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'cb' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'mx' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'fr' => 'Recueille des données statistiques liées aux visites du site web de l\'utilisateur, telles que le nombre de visites, le temps moyen passé sur le site et quelles pages ont été chargées. L’objectif étant de segmenter les utilisateurs du site en fonction de facteurs tels que la démographie et la situation géographique, afin de permettre aux médias et aux agences de marketing de structurer et de comprendre leurs groupes cibles et pour être en mesure d\'afficher une publicité en ligne personnalisée.',
                        'qc' => 'Recueille des données statistiques liées aux visites du site web de l\'utilisateur, telles que le nombre de visites, le temps moyen passé sur le site et quelles pages ont été chargées. L’objectif étant de segmenter les utilisateurs du site en fonction de facteurs tels que la démographie et la situation géographique, afin de permettre aux médias et aux agences de marketing de structurer et de comprendre leurs groupes cibles et pour être en mesure d\'afficher une publicité en ligne personnalisée.',
                        'pl' => 'Zbiera dane statystyczne dotyczące odwiedzin serwisu przez użytkownika, takie jak liczba odwiedzin, średni czas spędzony w serwisie oraz jakie strony zostały wczytane. Celem jest segmentacja użytkowników witryny internetowej według czynników, takich jak dane demograficzne i położenie geograficzne, aby umożliwić agencjom medialnym i marketingowym ustrukturyzowanie i zrozumienie ich grup docelowych w celu umożliwienia dostosowanej reklamy online.',
                        'ro' => 'Colectează date statistice legate de vizitele site-ului utilizatorului, cum ar fi numărul de vizite, timpul mediu petrecut pe site și ce pagini au fost încărcate. Scopul este de a segmenta utilizatorii site-ului în funcție de factori precum demografia și locația geografică, pentru a permite agențiilor media și de marketing să își structureze și să înțeleagă grupurile țintă pentru a permite publicitate online personalizată.',
                        'pt' => 'Coleta dados estatísticos relativos às visitas do usuário ao site, como número de visitas, tempo médio de permanência no site e quais páginas foram carregadas. O objetivo é segmentar os usuários do site de acordo com fatores como dados demográficos e localização geográfica, a fim de permitir que as agências de mídia e marketing estruturem e entendam seus públicos-alvo para possibilitar a publicidade online personalizada.',
                        'br' => 'Coleta dados estatísticos relativos às visitas do usuário ao site, como número de visitas, tempo médio de permanência no site e quais páginas foram carregadas. O objetivo é segmentar os usuários do site de acordo com fatores como dados demográficos e localização geográfica, a fim de permitir que as agências de mídia e marketing estruturem e entendam seus públicos-alvo para possibilitar a publicidade online personalizada.',
                        'sk' => 'Zhromažďuje štatistické údaje týkajúce sa návštev webových stránok používateľa, ako napríklad počet návštev, priemerný čas strávený na webových stránkach a načítané stránky. Účelom je segmentovať používateľov webových stránok podľa faktorov, ako sú demografické údaje a geografické umiestnenie, aby mohli mediálne a marketingové agentúry štruktúrovať a porozumieť svojim cieľovým skupinám a umožniť tak prispôsobenú online reklamu.',
                        'nl' => 'Verzamelt statistische gegevens met betrekking tot de websitebezoeken van de gebruiker, zoals het aantal bezoeken, de gemiddelde tijd die op de website is doorgebracht en welke pagina\'s zijn geladen. Het doel is om de gebruikers van de website te segmenteren op basis van factoren zoals demografie en geografische locatie, zodat media- en marketingbureaus hun doelgroepen kunnen structureren en begrijpen om aangepaste online advertenties mogelijk te maken.',
                        'de' => 'Erfasst statistische Daten zu Website-Besuchen des Benutzers, wie z. B. die Anzahl der Besuche, durchschnittliche Verweildauer auf der Website und welche Seiten geladen wurden. Der Zweck ist die Segmentierung der Benutzer der Website nach Faktoren wie Demografie und geografische Lage, damit Medien- und Marketing-Agenturen ihre Zielgruppen strukturieren und verstehen können, um maßgeschneiderte Online-Werbung zu ermöglichen.',
                        'gr' => 'Συλλέγει στατιστικά δεδομένα που σχετίζονται με τις επισκέψεις στον ιστότοπο του χρήστη, όπως ο αριθμός των επισκέψεων, ο μέσος χρόνος που αφιερώνεται στον ιστότοπο και ποιες σελίδες έχουν φορτωθεί. Ο σκοπός είναι η τμηματοποίηση των χρηστών του ιστότοπου σύμφωνα με παράγοντες όπως τα δημογραφικά στοιχεία και η γεωγραφική θέση, προκειμένου να δοθεί η δυνατότητα στα μέσα ενημέρωσης και στα γραφεία μάρκετινγκ να δομήσουν και να κατανοήσουν τις ομάδες-στόχους τους ώστε να επιτρέψουν προσαρμοσμένες διαδικτυακές διαφημίσεις.',
                        'it' => 'Raccoglie dati statistici relativi alle visite del sito internet da parte dell\'utente, come ad esempio il numero di visite, il tempo medio speso sul sito e quali pagine sono state caricate. Lo scopo è di suddividere gli utenti del sito internet a seconda di fattori demografici e geografici, allo scopo di consentire ai media e alle agenzie marketing di strutturare e comprendere i loro gruppi target per effettuare pubblicità online personalizzate.',
                        'si' => 'Zbira statistične podatke, povezane z obiski uporabnikovega spletnega mesta, kot so število obiskov, povprečni čas, ki ga je preživel na spletnem mestu in katere strani so bile naložene. Namen je segmentirati uporabnike spletnega mesta glede na dejavnike, kot so demografski podatki in geografski položaj, da se medijem in marketinškim agencijam omogoči, da strukturirajo in razumejo svoje ciljne skupine, da omogočijo prilagojeno spletno oglaševanje.',
                        'da' => 'Indsamler statistik om brugerens besøg på hjemmesiden såsom antallet af besøg, den gennemsnitlige tid på hjemmesiden og hvilke sider der er læst. Formålet er at segmentere hjemmesidens brugere efter faktorer såsom demografi og geografi for at gøre det muligt for medier og marketingbureauer at strukturere og forstå deres målgrupper med henblik på at tilpasse online annoncering.',
                        'no' => 'Samler inn statistiske data relatert til brukerens nettstedsbesøk, for eksempel antall besøk, gjennomsnittlig tid brukt på nettstedet og hvilke sider som er lastet inn. Hensikten er å segmentere nettstedets brukere i henhold til faktorer som demografi og geografisk beliggenhet, for å gjøre media- og markedsføringsbyråer i stand til å strukturere og forstå sine målgrupper for å muliggjøre tilpasset annonsering på nettet.',
                        'cs' => 'Shromažďuje statistické údaje související s návštěvami webových stránek uživatele, například počet návštěv, průměrný čas strávený na webu a jaké stránky byly načteny. Účelem je segmentovat uživatele webu podle faktorů, jako jsou demografické údaje a zeměpisné umístění, aby mediální a marketingové agentury mohly strukturovat a porozumět jejich cílovým skupinám a umožnit tak přizpůsobenou online reklamu.',
                        'hu' => 'Gyűjti a felhasználó webhelylátogatásaira vonatkozó statisztikai adatokat, például a látogatások számát, a webhelyen töltött átlagos időt és a betöltött oldalakat. A cél a weboldal felhasználói szegmentálása olyan tényezők szerint, mint a demográfia és a földrajzi elhelyezkedés, annak érdekében, hogy a média és a marketing ügynökségek strukturálhassák és megértsék célcsoportjaikat a testreszabott online hirdetések lehetővé tétele érdekében.',
                        'sv' => 'Samlar in statistiska data relaterade till användarens webbplatsbesök, till exempel antalet besök, genomsnittlig tid på webbplatsen och vilka sidor som har laddats. Syftet är att segmentera webbplatsens användare efter faktorer som demografi och geografisk plats för att göra det möjligt för media och marknadsföringsbyråer att strukturera och förstå sina målgrupper för att möjliggöra skräddarsydd onlineannonsering.',
                    ],
                    'expiry' => [
                        'en' => '13 months',
                        'es' => '13 meses',
                        'ag' => '13 meses',
                        'cb' => '13 meses',
                        'mx' => '13 meses',
                        'fr' => '13 mois',
                        'qc' => '13 mois',
                        'pl' => '13 miesiące',
                        'ro' => '13 luni',
                        'pt' => '13 meses',
                        'br' => '13 meses',
                        'sk' => '13 mesiace',
                        'nl' => '13 maanden',
                        'de' => '13 Monate',
                        'gr' => '13 μήνες',
                        'it' => '13 mesi',
                        'si' => '13 meseca',
                        'da' => '13 mdr.',
                        'no' => '13 måneder',
                        'cs' => '13 měsíců',
                        'hu' => '13 hónap',
                        'sv' => '13 månader',
                    ],
                ],
                [
                    /* Smartlook */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'SL_C_#_VID',
                    'provider' => 'Smartlook',
                    'provider_url' => 'https://help.smartlook.com/en/articles/3244452-privacy-policy',
                    'purpose' => [
                        'en' => 'Collects statistical data related to the user\'s website visits, such as the number of visits, average time spent on the website and what pages have been loaded. The purpose is to segment the website\'s users according to factors such as demographics and geographical location, in order to enable media and marketing agencies to structure and understand their target groups to enable customised online advertising.',
                        'es' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'ag' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'cb' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'mx' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'fr' => 'Recueille des données statistiques liées aux visites du site web de l\'utilisateur, telles que le nombre de visites, le temps moyen passé sur le site et quelles pages ont été chargées. L’objectif étant de segmenter les utilisateurs du site en fonction de facteurs tels que la démographie et la situation géographique, afin de permettre aux médias et aux agences de marketing de structurer et de comprendre leurs groupes cibles et pour être en mesure d\'afficher une publicité en ligne personnalisée.',
                        'qc' => 'Recueille des données statistiques liées aux visites du site web de l\'utilisateur, telles que le nombre de visites, le temps moyen passé sur le site et quelles pages ont été chargées. L’objectif étant de segmenter les utilisateurs du site en fonction de facteurs tels que la démographie et la situation géographique, afin de permettre aux médias et aux agences de marketing de structurer et de comprendre leurs groupes cibles et pour être en mesure d\'afficher une publicité en ligne personnalisée.',
                        'pl' => 'Zbiera dane statystyczne dotyczące odwiedzin serwisu przez użytkownika, takie jak liczba odwiedzin, średni czas spędzony w serwisie oraz jakie strony zostały wczytane. Celem jest segmentacja użytkowników witryny internetowej według czynników, takich jak dane demograficzne i położenie geograficzne, aby umożliwić agencjom medialnym i marketingowym ustrukturyzowanie i zrozumienie ich grup docelowych w celu umożliwienia dostosowanej reklamy online.',
                        'ro' => 'Colectează date statistice legate de vizitele site-ului utilizatorului, cum ar fi numărul de vizite, timpul mediu petrecut pe site și ce pagini au fost încărcate. Scopul este de a segmenta utilizatorii site-ului în funcție de factori precum demografia și locația geografică, pentru a permite agențiilor media și de marketing să își structureze și să înțeleagă grupurile țintă pentru a permite publicitate online personalizată.',
                        'pt' => 'Coleta dados estatísticos relativos às visitas do usuário ao site, como número de visitas, tempo médio de permanência no site e quais páginas foram carregadas. O objetivo é segmentar os usuários do site de acordo com fatores como dados demográficos e localização geográfica, a fim de permitir que as agências de mídia e marketing estruturem e entendam seus públicos-alvo para possibilitar a publicidade online personalizada.',
                        'br' => 'Coleta dados estatísticos relativos às visitas do usuário ao site, como número de visitas, tempo médio de permanência no site e quais páginas foram carregadas. O objetivo é segmentar os usuários do site de acordo com fatores como dados demográficos e localização geográfica, a fim de permitir que as agências de mídia e marketing estruturem e entendam seus públicos-alvo para possibilitar a publicidade online personalizada.',
                        'sk' => 'Zhromažďuje štatistické údaje týkajúce sa návštev webových stránok používateľa, ako napríklad počet návštev, priemerný čas strávený na webových stránkach a načítané stránky. Účelom je segmentovať používateľov webových stránok podľa faktorov, ako sú demografické údaje a geografické umiestnenie, aby mohli mediálne a marketingové agentúry štruktúrovať a porozumieť svojim cieľovým skupinám a umožniť tak prispôsobenú online reklamu.',
                        'nl' => 'Verzamelt statistische gegevens met betrekking tot de websitebezoeken van de gebruiker, zoals het aantal bezoeken, de gemiddelde tijd die op de website is doorgebracht en welke pagina\'s zijn geladen. Het doel is om de gebruikers van de website te segmenteren op basis van factoren zoals demografie en geografische locatie, zodat media- en marketingbureaus hun doelgroepen kunnen structureren en begrijpen om aangepaste online advertenties mogelijk te maken.',
                        'de' => 'Erfasst statistische Daten zu Website-Besuchen des Benutzers, wie z. B. die Anzahl der Besuche, durchschnittliche Verweildauer auf der Website und welche Seiten geladen wurden. Der Zweck ist die Segmentierung der Benutzer der Website nach Faktoren wie Demografie und geografische Lage, damit Medien- und Marketing-Agenturen ihre Zielgruppen strukturieren und verstehen können, um maßgeschneiderte Online-Werbung zu ermöglichen.',
                        'gr' => 'Συλλέγει στατιστικά δεδομένα που σχετίζονται με τις επισκέψεις στον ιστότοπο του χρήστη, όπως ο αριθμός των επισκέψεων, ο μέσος χρόνος που αφιερώνεται στον ιστότοπο και ποιες σελίδες έχουν φορτωθεί. Ο σκοπός είναι η τμηματοποίηση των χρηστών του ιστότοπου σύμφωνα με παράγοντες όπως τα δημογραφικά στοιχεία και η γεωγραφική θέση, προκειμένου να δοθεί η δυνατότητα στα μέσα ενημέρωσης και στα γραφεία μάρκετινγκ να δομήσουν και να κατανοήσουν τις ομάδες-στόχους τους ώστε να επιτρέψουν προσαρμοσμένες διαδικτυακές διαφημίσεις.',
                        'it' => 'Raccoglie dati statistici relativi alle visite del sito internet da parte dell\'utente, come ad esempio il numero di visite, il tempo medio speso sul sito e quali pagine sono state caricate. Lo scopo è di suddividere gli utenti del sito internet a seconda di fattori demografici e geografici, allo scopo di consentire ai media e alle agenzie marketing di strutturare e comprendere i loro gruppi target per effettuare pubblicità online personalizzate.',
                        'si' => 'Zbira statistične podatke, povezane z obiski uporabnikovega spletnega mesta, kot so število obiskov, povprečni čas, ki ga je preživel na spletnem mestu in katere strani so bile naložene. Namen je segmentirati uporabnike spletnega mesta glede na dejavnike, kot so demografski podatki in geografski položaj, da se medijem in marketinškim agencijam omogoči, da strukturirajo in razumejo svoje ciljne skupine, da omogočijo prilagojeno spletno oglaševanje.',
                        'da' => 'Indsamler statistik om brugerens besøg på hjemmesiden såsom antallet af besøg, den gennemsnitlige tid på hjemmesiden og hvilke sider der er læst. Formålet er at segmentere hjemmesidens brugere efter faktorer såsom demografi og geografi for at gøre det muligt for medier og marketingbureauer at strukturere og forstå deres målgrupper med henblik på at tilpasse online annoncering.',
                        'no' => 'Samler inn statistiske data relatert til brukerens nettstedsbesøk, for eksempel antall besøk, gjennomsnittlig tid brukt på nettstedet og hvilke sider som er lastet inn. Hensikten er å segmentere nettstedets brukere i henhold til faktorer som demografi og geografisk beliggenhet, for å gjøre media- og markedsføringsbyråer i stand til å strukturere og forstå sine målgrupper for å muliggjøre tilpasset annonsering på nettet.',
                        'cs' => 'Shromažďuje statistické údaje související s návštěvami webových stránek uživatele, například počet návštěv, průměrný čas strávený na webu a jaké stránky byly načteny. Účelem je segmentovat uživatele webu podle faktorů, jako jsou demografické údaje a zeměpisné umístění, aby mediální a marketingové agentury mohly strukturovat a porozumět jejich cílovým skupinám a umožnit tak přizpůsobenou online reklamu.',
                        'hu' => 'Gyűjti a felhasználó webhelylátogatásaira vonatkozó statisztikai adatokat, például a látogatások számát, a webhelyen töltött átlagos időt és a betöltött oldalakat. A cél a weboldal felhasználói szegmentálása olyan tényezők szerint, mint a demográfia és a földrajzi elhelyezkedés, annak érdekében, hogy a média és a marketing ügynökségek strukturálhassák és megértsék célcsoportjaikat a testreszabott online hirdetések lehetővé tétele érdekében.',
                        'sv' => 'Samlar in statistiska data relaterade till användarens webbplatsbesök, till exempel antalet besök, genomsnittlig tid på webbplatsen och vilka sidor som har laddats. Syftet är att segmentera webbplatsens användare efter faktorer som demografi och geografisk plats för att göra det möjligt för media och marknadsföringsbyråer att strukturera och förstå sina målgrupper för att möjliggöra skräddarsydd onlineannonsering.',
                    ],
                    'expiry' => [
                        'en' => '13 months',
                        'es' => '13 meses',
                        'ag' => '13 meses',
                        'cb' => '13 meses',
                        'mx' => '13 meses',
                        'fr' => '13 mois',
                        'qc' => '13 mois',
                        'pl' => '13 miesiące',
                        'ro' => '13 luni',
                        'pt' => '13 meses',
                        'br' => '13 meses',
                        'sk' => '13 mesiace',
                        'nl' => '13 maanden',
                        'de' => '13 Monate',
                        'gr' => '13 μήνες',
                        'it' => '13 mesi',
                        'si' => '13 meseca',
                        'da' => '13 mdr.',
                        'no' => '13 måneder',
                        'cs' => '13 měsíců',
                        'hu' => '13 hónap',
                        'sv' => '13 månader',
                    ],
                ],
                [
                    /* Smartlook */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'SL_L_#_KEY',
                    'provider' => 'Smartlook',
                    'provider_url' => 'https://help.smartlook.com/en/articles/3244452-privacy-policy',
                    'purpose' => [
                        'en' => 'Collects statistical data related to the user\'s website visits, such as the number of visits, average time spent on the website and what pages have been loaded. The purpose is to segment the website\'s users according to factors such as demographics and geographical location, in order to enable media and marketing agencies to structure and understand their target groups to enable customised online advertising.',
                        'es' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'ag' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'cb' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'mx' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'fr' => 'Recueille des données statistiques liées aux visites du site web de l\'utilisateur, telles que le nombre de visites, le temps moyen passé sur le site et quelles pages ont été chargées. L’objectif étant de segmenter les utilisateurs du site en fonction de facteurs tels que la démographie et la situation géographique, afin de permettre aux médias et aux agences de marketing de structurer et de comprendre leurs groupes cibles et pour être en mesure d\'afficher une publicité en ligne personnalisée.',
                        'qc' => 'Recueille des données statistiques liées aux visites du site web de l\'utilisateur, telles que le nombre de visites, le temps moyen passé sur le site et quelles pages ont été chargées. L’objectif étant de segmenter les utilisateurs du site en fonction de facteurs tels que la démographie et la situation géographique, afin de permettre aux médias et aux agences de marketing de structurer et de comprendre leurs groupes cibles et pour être en mesure d\'afficher une publicité en ligne personnalisée.',
                        'pl' => 'Zbiera dane statystyczne dotyczące odwiedzin serwisu przez użytkownika, takie jak liczba odwiedzin, średni czas spędzony w serwisie oraz jakie strony zostały wczytane. Celem jest segmentacja użytkowników witryny internetowej według czynników, takich jak dane demograficzne i położenie geograficzne, aby umożliwić agencjom medialnym i marketingowym ustrukturyzowanie i zrozumienie ich grup docelowych w celu umożliwienia dostosowanej reklamy online.',
                        'ro' => 'Colectează date statistice legate de vizitele site-ului utilizatorului, cum ar fi numărul de vizite, timpul mediu petrecut pe site și ce pagini au fost încărcate. Scopul este de a segmenta utilizatorii site-ului în funcție de factori precum demografia și locația geografică, pentru a permite agențiilor media și de marketing să își structureze și să înțeleagă grupurile țintă pentru a permite publicitate online personalizată.',
                        'pt' => 'Coleta dados estatísticos relativos às visitas do usuário ao site, como número de visitas, tempo médio de permanência no site e quais páginas foram carregadas. O objetivo é segmentar os usuários do site de acordo com fatores como dados demográficos e localização geográfica, a fim de permitir que as agências de mídia e marketing estruturem e entendam seus públicos-alvo para possibilitar a publicidade online personalizada.',
                        'br' => 'Coleta dados estatísticos relativos às visitas do usuário ao site, como número de visitas, tempo médio de permanência no site e quais páginas foram carregadas. O objetivo é segmentar os usuários do site de acordo com fatores como dados demográficos e localização geográfica, a fim de permitir que as agências de mídia e marketing estruturem e entendam seus públicos-alvo para possibilitar a publicidade online personalizada.',
                        'sk' => 'Zhromažďuje štatistické údaje týkajúce sa návštev webových stránok používateľa, ako napríklad počet návštev, priemerný čas strávený na webových stránkach a načítané stránky. Účelom je segmentovať používateľov webových stránok podľa faktorov, ako sú demografické údaje a geografické umiestnenie, aby mohli mediálne a marketingové agentúry štruktúrovať a porozumieť svojim cieľovým skupinám a umožniť tak prispôsobenú online reklamu.',
                        'nl' => 'Verzamelt statistische gegevens met betrekking tot de websitebezoeken van de gebruiker, zoals het aantal bezoeken, de gemiddelde tijd die op de website is doorgebracht en welke pagina\'s zijn geladen. Het doel is om de gebruikers van de website te segmenteren op basis van factoren zoals demografie en geografische locatie, zodat media- en marketingbureaus hun doelgroepen kunnen structureren en begrijpen om aangepaste online advertenties mogelijk te maken.',
                        'de' => 'Erfasst statistische Daten zu Website-Besuchen des Benutzers, wie z. B. die Anzahl der Besuche, durchschnittliche Verweildauer auf der Website und welche Seiten geladen wurden. Der Zweck ist die Segmentierung der Benutzer der Website nach Faktoren wie Demografie und geografische Lage, damit Medien- und Marketing-Agenturen ihre Zielgruppen strukturieren und verstehen können, um maßgeschneiderte Online-Werbung zu ermöglichen.',
                        'gr' => 'Συλλέγει στατιστικά δεδομένα που σχετίζονται με τις επισκέψεις στον ιστότοπο του χρήστη, όπως ο αριθμός των επισκέψεων, ο μέσος χρόνος που αφιερώνεται στον ιστότοπο και ποιες σελίδες έχουν φορτωθεί. Ο σκοπός είναι η τμηματοποίηση των χρηστών του ιστότοπου σύμφωνα με παράγοντες όπως τα δημογραφικά στοιχεία και η γεωγραφική θέση, προκειμένου να δοθεί η δυνατότητα στα μέσα ενημέρωσης και στα γραφεία μάρκετινγκ να δομήσουν και να κατανοήσουν τις ομάδες-στόχους τους ώστε να επιτρέψουν προσαρμοσμένες διαδικτυακές διαφημίσεις.',
                        'it' => 'Raccoglie dati statistici relativi alle visite del sito internet da parte dell\'utente, come ad esempio il numero di visite, il tempo medio speso sul sito e quali pagine sono state caricate. Lo scopo è di suddividere gli utenti del sito internet a seconda di fattori demografici e geografici, allo scopo di consentire ai media e alle agenzie marketing di strutturare e comprendere i loro gruppi target per effettuare pubblicità online personalizzate.',
                        'si' => 'Zbira statistične podatke, povezane z obiski uporabnikovega spletnega mesta, kot so število obiskov, povprečni čas, ki ga je preživel na spletnem mestu in katere strani so bile naložene. Namen je segmentirati uporabnike spletnega mesta glede na dejavnike, kot so demografski podatki in geografski položaj, da se medijem in marketinškim agencijam omogoči, da strukturirajo in razumejo svoje ciljne skupine, da omogočijo prilagojeno spletno oglaševanje.',
                        'da' => 'Indsamler statistik om brugerens besøg på hjemmesiden såsom antallet af besøg, den gennemsnitlige tid på hjemmesiden og hvilke sider der er læst. Formålet er at segmentere hjemmesidens brugere efter faktorer såsom demografi og geografi for at gøre det muligt for medier og marketingbureauer at strukturere og forstå deres målgrupper med henblik på at tilpasse online annoncering.',
                        'no' => 'Samler inn statistiske data relatert til brukerens nettstedsbesøk, for eksempel antall besøk, gjennomsnittlig tid brukt på nettstedet og hvilke sider som er lastet inn. Hensikten er å segmentere nettstedets brukere i henhold til faktorer som demografi og geografisk beliggenhet, for å gjøre media- og markedsføringsbyråer i stand til å strukturere og forstå sine målgrupper for å muliggjøre tilpasset annonsering på nettet.',
                        'cs' => 'Shromažďuje statistické údaje související s návštěvami webových stránek uživatele, například počet návštěv, průměrný čas strávený na webu a jaké stránky byly načteny. Účelem je segmentovat uživatele webu podle faktorů, jako jsou demografické údaje a zeměpisné umístění, aby mediální a marketingové agentury mohly strukturovat a porozumět jejich cílovým skupinám a umožnit tak přizpůsobenou online reklamu.',
                        'hu' => 'Gyűjti a felhasználó webhelylátogatásaira vonatkozó statisztikai adatokat, például a látogatások számát, a webhelyen töltött átlagos időt és a betöltött oldalakat. A cél a weboldal felhasználói szegmentálása olyan tényezők szerint, mint a demográfia és a földrajzi elhelyezkedés, annak érdekében, hogy a média és a marketing ügynökségek strukturálhassák és megértsék célcsoportjaikat a testreszabott online hirdetések lehetővé tétele érdekében.',
                        'sv' => 'Samlar in statistiska data relaterade till användarens webbplatsbesök, till exempel antalet besök, genomsnittlig tid på webbplatsen och vilka sidor som har laddats. Syftet är att segmentera webbplatsens användare efter faktorer som demografi och geografisk plats för att göra det möjligt för media och marknadsföringsbyråer att strukturera och förstå sina målgrupper för att möjliggöra skräddarsydd onlineannonsering.',
                    ],
                    'expiry' => [
                        'en' => '13 months',
                        'es' => '13 meses',
                        'ag' => '13 meses',
                        'cb' => '13 meses',
                        'mx' => '13 meses',
                        'fr' => '13 mois',
                        'qc' => '13 mois',
                        'pl' => '13 miesiące',
                        'ro' => '13 luni',
                        'pt' => '13 meses',
                        'br' => '13 meses',
                        'sk' => '13 mesiace',
                        'nl' => '13 maanden',
                        'de' => '13 Monate',
                        'gr' => '13 μήνες',
                        'it' => '13 mesi',
                        'si' => '13 meseca',
                        'da' => '13 mdr.',
                        'no' => '13 måneder',
                        'cs' => '13 měsíců',
                        'hu' => '13 hónap',
                        'sv' => '13 månader',
                    ],
                ],
                [
                    /* Smartlook */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'SL_L_#_SID',
                    'provider' => 'Smartlook',
                    'provider_url' => 'https://help.smartlook.com/en/articles/3244452-privacy-policy',
                    'purpose' => [
                        'en' => 'Collects statistical data related to the user\'s website visits, such as the number of visits, average time spent on the website and what pages have been loaded. The purpose is to segment the website\'s users according to factors such as demographics and geographical location, in order to enable media and marketing agencies to structure and understand their target groups to enable customised online advertising.',
                        'es' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'ag' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'cb' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'mx' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'fr' => 'Recueille des données statistiques liées aux visites du site web de l\'utilisateur, telles que le nombre de visites, le temps moyen passé sur le site et quelles pages ont été chargées. L’objectif étant de segmenter les utilisateurs du site en fonction de facteurs tels que la démographie et la situation géographique, afin de permettre aux médias et aux agences de marketing de structurer et de comprendre leurs groupes cibles et pour être en mesure d\'afficher une publicité en ligne personnalisée.',
                        'qc' => 'Recueille des données statistiques liées aux visites du site web de l\'utilisateur, telles que le nombre de visites, le temps moyen passé sur le site et quelles pages ont été chargées. L’objectif étant de segmenter les utilisateurs du site en fonction de facteurs tels que la démographie et la situation géographique, afin de permettre aux médias et aux agences de marketing de structurer et de comprendre leurs groupes cibles et pour être en mesure d\'afficher une publicité en ligne personnalisée.',
                        'pl' => 'Zbiera dane statystyczne dotyczące odwiedzin serwisu przez użytkownika, takie jak liczba odwiedzin, średni czas spędzony w serwisie oraz jakie strony zostały wczytane. Celem jest segmentacja użytkowników witryny internetowej według czynników, takich jak dane demograficzne i położenie geograficzne, aby umożliwić agencjom medialnym i marketingowym ustrukturyzowanie i zrozumienie ich grup docelowych w celu umożliwienia dostosowanej reklamy online.',
                        'ro' => 'Colectează date statistice legate de vizitele site-ului utilizatorului, cum ar fi numărul de vizite, timpul mediu petrecut pe site și ce pagini au fost încărcate. Scopul este de a segmenta utilizatorii site-ului în funcție de factori precum demografia și locația geografică, pentru a permite agențiilor media și de marketing să își structureze și să înțeleagă grupurile țintă pentru a permite publicitate online personalizată.',
                        'pt' => 'Coleta dados estatísticos relativos às visitas do usuário ao site, como número de visitas, tempo médio de permanência no site e quais páginas foram carregadas. O objetivo é segmentar os usuários do site de acordo com fatores como dados demográficos e localização geográfica, a fim de permitir que as agências de mídia e marketing estruturem e entendam seus públicos-alvo para possibilitar a publicidade online personalizada.',
                        'br' => 'Coleta dados estatísticos relativos às visitas do usuário ao site, como número de visitas, tempo médio de permanência no site e quais páginas foram carregadas. O objetivo é segmentar os usuários do site de acordo com fatores como dados demográficos e localização geográfica, a fim de permitir que as agências de mídia e marketing estruturem e entendam seus públicos-alvo para possibilitar a publicidade online personalizada.',
                        'sk' => 'Zhromažďuje štatistické údaje týkajúce sa návštev webových stránok používateľa, ako napríklad počet návštev, priemerný čas strávený na webových stránkach a načítané stránky. Účelom je segmentovať používateľov webových stránok podľa faktorov, ako sú demografické údaje a geografické umiestnenie, aby mohli mediálne a marketingové agentúry štruktúrovať a porozumieť svojim cieľovým skupinám a umožniť tak prispôsobenú online reklamu.',
                        'nl' => 'Verzamelt statistische gegevens met betrekking tot de websitebezoeken van de gebruiker, zoals het aantal bezoeken, de gemiddelde tijd die op de website is doorgebracht en welke pagina\'s zijn geladen. Het doel is om de gebruikers van de website te segmenteren op basis van factoren zoals demografie en geografische locatie, zodat media- en marketingbureaus hun doelgroepen kunnen structureren en begrijpen om aangepaste online advertenties mogelijk te maken.',
                        'de' => 'Erfasst statistische Daten zu Website-Besuchen des Benutzers, wie z. B. die Anzahl der Besuche, durchschnittliche Verweildauer auf der Website und welche Seiten geladen wurden. Der Zweck ist die Segmentierung der Benutzer der Website nach Faktoren wie Demografie und geografische Lage, damit Medien- und Marketing-Agenturen ihre Zielgruppen strukturieren und verstehen können, um maßgeschneiderte Online-Werbung zu ermöglichen.',
                        'gr' => 'Συλλέγει στατιστικά δεδομένα που σχετίζονται με τις επισκέψεις στον ιστότοπο του χρήστη, όπως ο αριθμός των επισκέψεων, ο μέσος χρόνος που αφιερώνεται στον ιστότοπο και ποιες σελίδες έχουν φορτωθεί. Ο σκοπός είναι η τμηματοποίηση των χρηστών του ιστότοπου σύμφωνα με παράγοντες όπως τα δημογραφικά στοιχεία και η γεωγραφική θέση, προκειμένου να δοθεί η δυνατότητα στα μέσα ενημέρωσης και στα γραφεία μάρκετινγκ να δομήσουν και να κατανοήσουν τις ομάδες-στόχους τους ώστε να επιτρέψουν προσαρμοσμένες διαδικτυακές διαφημίσεις.',
                        'it' => 'Raccoglie dati statistici relativi alle visite del sito internet da parte dell\'utente, come ad esempio il numero di visite, il tempo medio speso sul sito e quali pagine sono state caricate. Lo scopo è di suddividere gli utenti del sito internet a seconda di fattori demografici e geografici, allo scopo di consentire ai media e alle agenzie marketing di strutturare e comprendere i loro gruppi target per effettuare pubblicità online personalizzate.',
                        'si' => 'Zbira statistične podatke, povezane z obiski uporabnikovega spletnega mesta, kot so število obiskov, povprečni čas, ki ga je preživel na spletnem mestu in katere strani so bile naložene. Namen je segmentirati uporabnike spletnega mesta glede na dejavnike, kot so demografski podatki in geografski položaj, da se medijem in marketinškim agencijam omogoči, da strukturirajo in razumejo svoje ciljne skupine, da omogočijo prilagojeno spletno oglaševanje.',
                        'da' => 'Indsamler statistik om brugerens besøg på hjemmesiden såsom antallet af besøg, den gennemsnitlige tid på hjemmesiden og hvilke sider der er læst. Formålet er at segmentere hjemmesidens brugere efter faktorer såsom demografi og geografi for at gøre det muligt for medier og marketingbureauer at strukturere og forstå deres målgrupper med henblik på at tilpasse online annoncering.',
                        'no' => 'Samler inn statistiske data relatert til brukerens nettstedsbesøk, for eksempel antall besøk, gjennomsnittlig tid brukt på nettstedet og hvilke sider som er lastet inn. Hensikten er å segmentere nettstedets brukere i henhold til faktorer som demografi og geografisk beliggenhet, for å gjøre media- og markedsføringsbyråer i stand til å strukturere og forstå sine målgrupper for å muliggjøre tilpasset annonsering på nettet.',
                        'cs' => 'Shromažďuje statistické údaje související s návštěvami webových stránek uživatele, například počet návštěv, průměrný čas strávený na webu a jaké stránky byly načteny. Účelem je segmentovat uživatele webu podle faktorů, jako jsou demografické údaje a zeměpisné umístění, aby mediální a marketingové agentury mohly strukturovat a porozumět jejich cílovým skupinám a umožnit tak přizpůsobenou online reklamu.',
                        'hu' => 'Gyűjti a felhasználó webhelylátogatásaira vonatkozó statisztikai adatokat, például a látogatások számát, a webhelyen töltött átlagos időt és a betöltött oldalakat. A cél a weboldal felhasználói szegmentálása olyan tényezők szerint, mint a demográfia és a földrajzi elhelyezkedés, annak érdekében, hogy a média és a marketing ügynökségek strukturálhassák és megértsék célcsoportjaikat a testreszabott online hirdetések lehetővé tétele érdekében.',
                        'sv' => 'Samlar in statistiska data relaterade till användarens webbplatsbesök, till exempel antalet besök, genomsnittlig tid på webbplatsen och vilka sidor som har laddats. Syftet är att segmentera webbplatsens användare efter faktorer som demografi och geografisk plats för att göra det möjligt för media och marknadsföringsbyråer att strukturera och förstå sina målgrupper för att möjliggöra skräddarsydd onlineannonsering.',
                    ],
                    'expiry' => [
                        'en' => '13 months',
                        'es' => '13 meses',
                        'ag' => '13 meses',
                        'cb' => '13 meses',
                        'mx' => '13 meses',
                        'fr' => '13 mois',
                        'qc' => '13 mois',
                        'pl' => '13 miesiące',
                        'ro' => '13 luni',
                        'pt' => '13 meses',
                        'br' => '13 meses',
                        'sk' => '13 mesiace',
                        'nl' => '13 maanden',
                        'de' => '13 Monate',
                        'gr' => '13 μήνες',
                        'it' => '13 mesi',
                        'si' => '13 meseca',
                        'da' => '13 mdr.',
                        'no' => '13 måneder',
                        'cs' => '13 měsíců',
                        'hu' => '13 hónap',
                        'sv' => '13 månader',
                    ],
                ],
                [
                    /* Smartlook */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'SL_L_#_VID',
                    'provider' => 'Smartlook',
                    'provider_url' => 'https://help.smartlook.com/en/articles/3244452-privacy-policy',
                    'purpose' => [
                        'en' => 'Collects statistical data related to the user\'s website visits, such as the number of visits, average time spent on the website and what pages have been loaded. The purpose is to segment the website\'s users according to factors such as demographics and geographical location, in order to enable media and marketing agencies to structure and understand their target groups to enable customised online advertising.',
                        'es' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'ag' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'cb' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'mx' => 'Recopila datos estadísticos relacionados con las visitas del usuario al sitio web, como el número de visitas, el tiempo medio pasado en el sitio web y qué páginas se han cargado. El propósito es segmentar los usuarios del sitio web según factores como factores demográficos y ubicación geográfica para permitir a las agencias multimedia y de marketing estructurar y comprender sus grupos objetos para habilitar la publicidad online personalizada.',
                        'fr' => 'Recueille des données statistiques liées aux visites du site web de l\'utilisateur, telles que le nombre de visites, le temps moyen passé sur le site et quelles pages ont été chargées. L’objectif étant de segmenter les utilisateurs du site en fonction de facteurs tels que la démographie et la situation géographique, afin de permettre aux médias et aux agences de marketing de structurer et de comprendre leurs groupes cibles et pour être en mesure d\'afficher une publicité en ligne personnalisée.',
                        'qc' => 'Recueille des données statistiques liées aux visites du site web de l\'utilisateur, telles que le nombre de visites, le temps moyen passé sur le site et quelles pages ont été chargées. L’objectif étant de segmenter les utilisateurs du site en fonction de facteurs tels que la démographie et la situation géographique, afin de permettre aux médias et aux agences de marketing de structurer et de comprendre leurs groupes cibles et pour être en mesure d\'afficher une publicité en ligne personnalisée.',
                        'pl' => 'Zbiera dane statystyczne dotyczące odwiedzin serwisu przez użytkownika, takie jak liczba odwiedzin, średni czas spędzony w serwisie oraz jakie strony zostały wczytane. Celem jest segmentacja użytkowników witryny internetowej według czynników, takich jak dane demograficzne i położenie geograficzne, aby umożliwić agencjom medialnym i marketingowym ustrukturyzowanie i zrozumienie ich grup docelowych w celu umożliwienia dostosowanej reklamy online.',
                        'ro' => 'Colectează date statistice legate de vizitele site-ului utilizatorului, cum ar fi numărul de vizite, timpul mediu petrecut pe site și ce pagini au fost încărcate. Scopul este de a segmenta utilizatorii site-ului în funcție de factori precum demografia și locația geografică, pentru a permite agențiilor media și de marketing să își structureze și să înțeleagă grupurile țintă pentru a permite publicitate online personalizată.',
                        'pt' => 'Coleta dados estatísticos relativos às visitas do usuário ao site, como número de visitas, tempo médio de permanência no site e quais páginas foram carregadas. O objetivo é segmentar os usuários do site de acordo com fatores como dados demográficos e localização geográfica, a fim de permitir que as agências de mídia e marketing estruturem e entendam seus públicos-alvo para possibilitar a publicidade online personalizada.',
                        'br' => 'Coleta dados estatísticos relativos às visitas do usuário ao site, como número de visitas, tempo médio de permanência no site e quais páginas foram carregadas. O objetivo é segmentar os usuários do site de acordo com fatores como dados demográficos e localização geográfica, a fim de permitir que as agências de mídia e marketing estruturem e entendam seus públicos-alvo para possibilitar a publicidade online personalizada.',
                        'sk' => 'Zhromažďuje štatistické údaje týkajúce sa návštev webových stránok používateľa, ako napríklad počet návštev, priemerný čas strávený na webových stránkach a načítané stránky. Účelom je segmentovať používateľov webových stránok podľa faktorov, ako sú demografické údaje a geografické umiestnenie, aby mohli mediálne a marketingové agentúry štruktúrovať a porozumieť svojim cieľovým skupinám a umožniť tak prispôsobenú online reklamu.',
                        'nl' => 'Verzamelt statistische gegevens met betrekking tot de websitebezoeken van de gebruiker, zoals het aantal bezoeken, de gemiddelde tijd die op de website is doorgebracht en welke pagina\'s zijn geladen. Het doel is om de gebruikers van de website te segmenteren op basis van factoren zoals demografie en geografische locatie, zodat media- en marketingbureaus hun doelgroepen kunnen structureren en begrijpen om aangepaste online advertenties mogelijk te maken.',
                        'de' => 'Erfasst statistische Daten zu Website-Besuchen des Benutzers, wie z. B. die Anzahl der Besuche, durchschnittliche Verweildauer auf der Website und welche Seiten geladen wurden. Der Zweck ist die Segmentierung der Benutzer der Website nach Faktoren wie Demografie und geografische Lage, damit Medien- und Marketing-Agenturen ihre Zielgruppen strukturieren und verstehen können, um maßgeschneiderte Online-Werbung zu ermöglichen.',
                        'gr' => 'Συλλέγει στατιστικά δεδομένα που σχετίζονται με τις επισκέψεις στον ιστότοπο του χρήστη, όπως ο αριθμός των επισκέψεων, ο μέσος χρόνος που αφιερώνεται στον ιστότοπο και ποιες σελίδες έχουν φορτωθεί. Ο σκοπός είναι η τμηματοποίηση των χρηστών του ιστότοπου σύμφωνα με παράγοντες όπως τα δημογραφικά στοιχεία και η γεωγραφική θέση, προκειμένου να δοθεί η δυνατότητα στα μέσα ενημέρωσης και στα γραφεία μάρκετινγκ να δομήσουν και να κατανοήσουν τις ομάδες-στόχους τους ώστε να επιτρέψουν προσαρμοσμένες διαδικτυακές διαφημίσεις.',
                        'it' => 'Raccoglie dati statistici relativi alle visite del sito internet da parte dell\'utente, come ad esempio il numero di visite, il tempo medio speso sul sito e quali pagine sono state caricate. Lo scopo è di suddividere gli utenti del sito internet a seconda di fattori demografici e geografici, allo scopo di consentire ai media e alle agenzie marketing di strutturare e comprendere i loro gruppi target per effettuare pubblicità online personalizzate.',
                        'si' => 'Zbira statistične podatke, povezane z obiski uporabnikovega spletnega mesta, kot so število obiskov, povprečni čas, ki ga je preživel na spletnem mestu in katere strani so bile naložene. Namen je segmentirati uporabnike spletnega mesta glede na dejavnike, kot so demografski podatki in geografski položaj, da se medijem in marketinškim agencijam omogoči, da strukturirajo in razumejo svoje ciljne skupine, da omogočijo prilagojeno spletno oglaševanje.',
                        'da' => 'Indsamler statistik om brugerens besøg på hjemmesiden såsom antallet af besøg, den gennemsnitlige tid på hjemmesiden og hvilke sider der er læst. Formålet er at segmentere hjemmesidens brugere efter faktorer såsom demografi og geografi for at gøre det muligt for medier og marketingbureauer at strukturere og forstå deres målgrupper med henblik på at tilpasse online annoncering.',
                        'no' => 'Samler inn statistiske data relatert til brukerens nettstedsbesøk, for eksempel antall besøk, gjennomsnittlig tid brukt på nettstedet og hvilke sider som er lastet inn. Hensikten er å segmentere nettstedets brukere i henhold til faktorer som demografi og geografisk beliggenhet, for å gjøre media- og markedsføringsbyråer i stand til å strukturere og forstå sine målgrupper for å muliggjøre tilpasset annonsering på nettet.',
                        'cs' => 'Shromažďuje statistické údaje související s návštěvami webových stránek uživatele, například počet návštěv, průměrný čas strávený na webu a jaké stránky byly načteny. Účelem je segmentovat uživatele webu podle faktorů, jako jsou demografické údaje a zeměpisné umístění, aby mediální a marketingové agentury mohly strukturovat a porozumět jejich cílovým skupinám a umožnit tak přizpůsobenou online reklamu.',
                        'hu' => 'Gyűjti a felhasználó webhelylátogatásaira vonatkozó statisztikai adatokat, például a látogatások számát, a webhelyen töltött átlagos időt és a betöltött oldalakat. A cél a weboldal felhasználói szegmentálása olyan tényezők szerint, mint a demográfia és a földrajzi elhelyezkedés, annak érdekében, hogy a média és a marketing ügynökségek strukturálhassák és megértsék célcsoportjaikat a testreszabott online hirdetések lehetővé tétele érdekében.',
                        'sv' => 'Samlar in statistiska data relaterade till användarens webbplatsbesök, till exempel antalet besök, genomsnittlig tid på webbplatsen och vilka sidor som har laddats. Syftet är att segmentera webbplatsens användare efter faktorer som demografi och geografisk plats för att göra det möjligt för media och marknadsföringsbyråer att strukturera och förstå sina målgrupper för att möjliggöra skräddarsydd onlineannonsering.',
                    ],
                    'expiry' => [
                        'en' => '13 months',
                        'es' => '13 meses',
                        'ag' => '13 meses',
                        'cb' => '13 meses',
                        'mx' => '13 meses',
                        'fr' => '13 mois',
                        'qc' => '13 mois',
                        'pl' => '13 miesiące',
                        'ro' => '13 luni',
                        'pt' => '13 meses',
                        'br' => '13 meses',
                        'sk' => '13 mesiace',
                        'nl' => '13 maanden',
                        'de' => '13 Monate',
                        'gr' => '13 μήνες',
                        'it' => '13 mesi',
                        'si' => '13 meseca',
                        'da' => '13 mdr.',
                        'no' => '13 måneder',
                        'cs' => '13 měsíců',
                        'hu' => '13 hónap',
                        'sv' => '13 månader',
                    ],
                ],
                [
                    /* Youtube */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'VISITOR_INFO1_LIVE',
                    'provider' => '	youtube.com',
                    'provider_url' => 'https://policies.google.com/technologies/cookies',
                    'purpose' => [
                        'en' => 'Tries to estimate the users\' bandwidth on pages with integrated YouTube videos.',
                        'es' => 'Intenta calcular el ancho de banda del usuario en páginas con vídeos de YouTube integrados.',
                        'ag' => 'Intenta calcular el ancho de banda del usuario en páginas con vídeos de YouTube integrados.',
                        'cb' => 'Intenta calcular el ancho de banda del usuario en páginas con vídeos de YouTube integrados.',
                        'mx' => 'Intenta calcular el ancho de banda del usuario en páginas con vídeos de YouTube integrados.',
                        'fr' => 'Tente d\'estimer la bande passante des utilisateurs sur des pages avec des vidéos YouTube intégrées.',
                        'qc' => 'Tente d\'estimer la bande passante des utilisateurs sur des pages avec des vidéos YouTube intégrées.',
                        'pl' => 'Próbuje oszacować przepustowość użytkowników na stronach ze zintegrowanymi filmami z YouTube.',
                        'ro' => 'Încearcă să estimeze lățimea de bandă a utilizatorilor pe paginile cu videoclipuri YouTube integrate.',
                        'pt' => 'Tenta estimar a largura de banda dos usuários em páginas com vídeos integrados do YouTube.',
                        'br' => 'Tenta estimar a largura de banda dos usuários em páginas com vídeos integrados do YouTube.',
                        'sk' => 'Pokúša sa odhadnúť šírku pásma používateľov na stránkach s integrovanými videami YouTube.',
                        'nl' => 'Probeert de bandbreedte van gebruikers te schatten op pagina\'s met geïntegreerde YouTube-video\'s.',
                        'de' => 'Versucht, die Benutzerbandbreite auf Seiten mit integrierten YouTube-Videos zu schätzen.',
                        'gr' => 'Προσπαθεί να εκτιμήσει το εύρος ζώνης των χρηστών σε σελίδες με ενσωματωμένα βίντεο YouTube.',
                        'it' => 'Prova a stimare la velocità della connessione dell\'utente su pagine con video YouTube integrati.',
                        'si' => 'Poskuša oceniti pasovno širino uporabnikov na straneh z integriranimi videoposnetki v YouTubu.',
                        'da' => 'Forsøger at estimere brugernes båndbredde på sider med integreret YouTube-video.',
                        'no' => 'Prøver å estimere brukernes båndbredde på sider med integrerte YouTube-videoer.',
                        'cs' => 'Pokouší se odhadnout šířku pásma uživatelů na stránkách s integrovanými videi YouTube.',
                        'hu' => 'Megpróbálja megbecsülni a felhasználók sávszélességét az integrált YouTube videókat tartalmazó oldalakon.',
                        'sv' => 'Försöker uppskatta användarnas bandbredd på sidor med integrerade YouTube-videor.',
                    ],
                    'expiry' => [
                        'en' => '179 days',
                        'es' => '179 días',
                        'ag' => '179 días',
                        'cb' => '179 días',
                        'mx' => '179 días',
                        'fr' => '179 jours',
                        'qc' => '179 jours',
                        'pl' => '179 dni',
                        'ro' => '179 de zile',
                        'pt' => '179 dias',
                        'br' => '179 dias',
                        'sk' => '179 dní',
                        'nl' => '179 dagen',
                        'de' => '179 Tage',
                        'gr' => '179 μέρες',
                        'it' => '179 giorni',
                        'si' => '179 dni',
                        'da' => '179 dage',
                        'no' => '179 dager',
                        'cs' => '179 dager',
                        'hu' => '179 nap',
                        'sv' => '179 dagar',
                    ],
                ],
                [
                    /* Youtube */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'YSC',
                    'provider' => '	youtube.com',
                    'provider_url' => 'https://policies.google.com/technologies/cookies',
                    'purpose' => [
                        'en' => 'Registers a unique ID to keep statistics of what videos from YouTube the user has seen.',
                        'es' => 'Registra una identificación única para mantener estadísticas de qué vídeos de YouTube ha visto el usuario.',
                        'ag' => 'Registra una identificación única para mantener estadísticas de qué vídeos de YouTube ha visto el usuario.',
                        'cb' => 'Registra una identificación única para mantener estadísticas de qué vídeos de YouTube ha visto el usuario.',
                        'mx' => 'Registra una identificación única para mantener estadísticas de qué vídeos de YouTube ha visto el usuario.',
                        'fr' => 'Enregistre un identifiant unique pour conserver des statistiques sur les vidéos de YouTube vues par l\'utilisateur.',
                        'qc' => 'Enregistre un identifiant unique pour conserver des statistiques sur les vidéos de YouTube vues par l\'utilisateur.',
                        'pl' => 'Rejestruje unikalny identyfikator, aby prowadzić statystyki dotyczące filmów wideo z YouTube, które widział użytkownik.',
                        'ro' => 'Înregistrează un ID unic pentru a păstra statistici despre videoclipurile de pe YouTube pe care le-a văzut utilizatorul.',
                        'pt' => 'Registra um ID único para manter estatísticas de quais vídeos do YouTube o usuário viu.',
                        'br' => 'Registra um ID único para manter estatísticas de quais vídeos do YouTube o usuário viu.',
                        'sk' => 'Zaregistruje jedinečný identifikátor, ktorý umožňuje štatistiku videí, ktoré používateľ YouTube videl.',
                        'nl' => 'Registreert een unieke ID om statistieken bij te houden van welke video\'s van YouTube de gebruiker heeft gezien.',
                        'de' => 'Registriert eine eindeutige ID, um Statistiken der Videos von YouTube, die der Benutzer gesehen hat, zu behalten.',
                        'gr' => 'Καταχωρεί ένα μοναδικό αναγνωριστικό για να διατηρεί στατιστικά στοιχεία για τα βίντεο από το YouTube που έχει δει ο χρήστης.',
                        'it' => 'Registra un ID univoco per statistiche legate a quali video YouTube sono stati visualizzati dall\'utente.',
                        'si' => 'Registrira enolični ID, da vodi statistiko o tem, katere videoposnetke iz YouTuba je uporabnik videl.',
                        'da' => 'Registrerer et unikt ID for at føre statistik over hvilke videoer fra YouTube brugeren har set.',
                        'no' => 'Registrerer en unik ID for å føre statistikk over hvilke videoer fra YouTube brukeren har sett.',
                        'cs' => 'Zaregistruje jedinečné ID, které udržuje statistiky o tom, jaká videa z YouTube uživatel viděl.',
                        'hu' => 'Egyedi azonosítót regisztrál, hogy statisztikákat készítsen arról, hogy a YouTube milyen videókat látott a felhasználó.',
                        'sv' => 'Registrerar ett unikt ID för att hålla statistik över vilka videor från YouTube användaren har sett.',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    /* Youtube */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'yt-remote-cast-installed',
                    'provider' => '	youtube.com',
                    'provider_url' => 'https://policies.google.com/technologies/cookies',
                    'purpose' => [
                        'en' => 'Stores the user\'s video player preferences using embedded YouTube video',
                        'es' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'ag' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'cb' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'mx' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'fr' => 'Stocke les préférences de lecture vidéo de l\'utilisateur pour les vidéos YouTube incorporées.',
                        'qc' => 'Stocke les préférences de lecture vidéo de l\'utilisateur pour les vidéos YouTube incorporées.',
                        'pl' => 'Przechowuje preferencje odtwarzacza wideo użytkownika za pomocą osadzonego wideo YouTube.',
                        'ro' => 'Stochează preferințele playerului video ale utilizatorului utilizând videoclipuri YouTube încorporate.',
                        'pt' => 'Armazena as preferências do player de vídeo do usuário usando o vídeo do YouTube incorporado.',
                        'br' => 'Armazena as preferências do player de vídeo do usuário usando o vídeo do YouTube incorporado.',
                        'sk' => 'Ukladá predvoľby prehrávača videa používateľa pomocou vloženého videa YouTube.',
                        'nl' => 'Bewaart de voorkeuren van de videospeler van de gebruiker met ingesloten YouTube-video',
                        'de' => 'Registriert eine eindeutige ID, um Statistiken der Videos von YouTube, die der Benutzer gesehen hat, zu behalten.',
                        'gr' => 'Αποθηκεύει τις προτιμήσεις του προγράμματος αναπαραγωγής βίντεο του χρήστη χρησιμοποιώντας ενσωματωμένο βίντεο YouTube.',
                        'it' => 'Memorizza le preferenze del lettore video dell\'utente usando il video YouTube incorporato',
                        'si' => 'Shrani nastavitve uporabnikovega video predvajalnika z uporabo vdelanega videoposnetka YouTube.',
                        'da' => 'Gemmer brugerens video-afspiller-præferencer ved afspilning af en indlejret YouTube video.',
                        'no' => 'Lagrer brukerens videospillerinnstillinger ved hjelp av innebygd YouTube-video.',
                        'cs' => 'Ukládá předvolby přehrávače videa uživatele pomocí vloženého videa YouTube.',
                        'hu' => 'A felhasználó videólejátszójának beállításait tárolja a beágyazott YouTube-videók segítségével.',
                        'sv' => 'Lagrar användarens videospelarinställningar med inbäddad YouTube-video',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    /* Youtube */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'yt-remote-connected-devices',
                    'provider' => '	youtube.com',
                    'provider_url' => 'https://policies.google.com/technologies/cookies',
                    'purpose' => [
                        'en' => 'Stores the user\'s video player preferences using embedded YouTube video',
                        'es' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'ag' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'cb' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'mx' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'fr' => 'Stocke les préférences de lecture vidéo de l\'utilisateur pour les vidéos YouTube incorporées.',
                        'qc' => 'Stocke les préférences de lecture vidéo de l\'utilisateur pour les vidéos YouTube incorporées.',
                        'pl' => 'Przechowuje preferencje odtwarzacza wideo użytkownika za pomocą osadzonego wideo YouTube.',
                        'ro' => 'Stochează preferințele playerului video ale utilizatorului utilizând videoclipuri YouTube încorporate.',
                        'pt' => 'Armazena as preferências do player de vídeo do usuário usando o vídeo do YouTube incorporado.',
                        'br' => 'Armazena as preferências do player de vídeo do usuário usando o vídeo do YouTube incorporado.',
                        'sk' => 'Ukladá predvoľby prehrávača videa používateľa pomocou vloženého videa YouTube.',
                        'nl' => 'Bewaart de voorkeuren van de videospeler van de gebruiker met ingesloten YouTube-video',
                        'de' => 'Registriert eine eindeutige ID, um Statistiken der Videos von YouTube, die der Benutzer gesehen hat, zu behalten.',
                        'gr' => 'Αποθηκεύει τις προτιμήσεις του προγράμματος αναπαραγωγής βίντεο του χρήστη χρησιμοποιώντας ενσωματωμένο βίντεο YouTube.',
                        'it' => 'Memorizza le preferenze del lettore video dell\'utente usando il video YouTube incorporato',
                        'si' => 'Shrani nastavitve uporabnikovega video predvajalnika z uporabo vdelanega videoposnetka YouTube.',
                        'da' => 'Gemmer brugerens video-afspiller-præferencer ved afspilning af en indlejret YouTube video.',
                        'no' => 'Lagrer brukerens videospillerinnstillinger ved hjelp av innebygd YouTube-video.',
                        'cs' => 'Ukládá předvolby přehrávače videa uživatele pomocí vloženého videa YouTube.',
                        'hu' => 'A felhasználó videólejátszójának beállításait tárolja a beágyazott YouTube-videók segítségével.',
                        'sv' => 'Lagrar användarens videospelarinställningar med inbäddad YouTube-video',
                    ],
                    'expiry' => [
                        'en' => 'Persistent',
                        'es' => 'Persistente',
                        'ag' => 'Persistente',
                        'cb' => 'Persistente',
                        'mx' => 'Persistente',
                        'fr' => 'Persistant',
                        'qc' => 'Persistant',
                        'pl' => 'Trwały',
                        'ro' => 'Persistent',
                        'pt' => 'Persistente',
                        'br' => 'Persistente',
                        'sk' => 'Vytrvalý',
                        'nl' => 'Aanhoudend',
                        'de' => 'Hartnäckig',
                        'gr' => 'Επίμονος',
                        'it' => 'Persistente',
                        'si' => 'Vztrajno',
                        'da' => 'Vedholdende',
                        'no' => 'Vedvarende',
                        'cs' => 'Trvalý',
                        'hu' => 'Kitartó',
                        'sv' => 'Beständig',
                    ],
                ],
                [
                    /* Youtube */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'yt-remote-device-id',
                    'provider' => '	youtube.com',
                    'provider_url' => 'https://policies.google.com/technologies/cookies',
                    'purpose' => [
                        'en' => 'Stores the user\'s video player preferences using embedded YouTube video',
                        'es' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'ag' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'cb' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'mx' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'fr' => 'Stocke les préférences de lecture vidéo de l\'utilisateur pour les vidéos YouTube incorporées.',
                        'qc' => 'Stocke les préférences de lecture vidéo de l\'utilisateur pour les vidéos YouTube incorporées.',
                        'pl' => 'Przechowuje preferencje odtwarzacza wideo użytkownika za pomocą osadzonego wideo YouTube.',
                        'ro' => 'Stochează preferințele playerului video ale utilizatorului utilizând videoclipuri YouTube încorporate.',
                        'pt' => 'Armazena as preferências do player de vídeo do usuário usando o vídeo do YouTube incorporado.',
                        'br' => 'Armazena as preferências do player de vídeo do usuário usando o vídeo do YouTube incorporado.',
                        'sk' => 'Ukladá predvoľby prehrávača videa používateľa pomocou vloženého videa YouTube.',
                        'nl' => 'Bewaart de voorkeuren van de videospeler van de gebruiker met ingesloten YouTube-video',
                        'de' => 'Registriert eine eindeutige ID, um Statistiken der Videos von YouTube, die der Benutzer gesehen hat, zu behalten.',
                        'gr' => 'Αποθηκεύει τις προτιμήσεις του προγράμματος αναπαραγωγής βίντεο του χρήστη χρησιμοποιώντας ενσωματωμένο βίντεο YouTube.',
                        'it' => 'Memorizza le preferenze del lettore video dell\'utente usando il video YouTube incorporato',
                        'si' => 'Shrani nastavitve uporabnikovega video predvajalnika z uporabo vdelanega videoposnetka YouTube.',
                        'da' => 'Gemmer brugerens video-afspiller-præferencer ved afspilning af en indlejret YouTube video.',
                        'no' => 'Lagrer brukerens videospillerinnstillinger ved hjelp av innebygd YouTube-video.',
                        'cs' => 'Ukládá předvolby přehrávače videa uživatele pomocí vloženého videa YouTube.',
                        'hu' => 'A felhasználó videólejátszójának beállításait tárolja a beágyazott YouTube-videók segítségével.',
                        'sv' => 'Lagrar användarens videospelarinställningar med inbäddad YouTube-video',
                    ],
                    'expiry' => [
                        'en' => 'Persistent',
                        'es' => 'Persistente',
                        'ag' => 'Persistente',
                        'cb' => 'Persistente',
                        'mx' => 'Persistente',
                        'fr' => 'Persistant',
                        'qc' => 'Persistant',
                        'pl' => 'Trwały',
                        'ro' => 'Persistent',
                        'pt' => 'Persistente',
                        'br' => 'Persistente',
                        'sk' => 'Vytrvalý',
                        'nl' => 'Aanhoudend',
                        'de' => 'Hartnäckig',
                        'gr' => 'Επίμονος',
                        'it' => 'Persistente',
                        'si' => 'Vztrajno',
                        'da' => 'Vedholdende',
                        'no' => 'Vedvarende',
                        'cs' => 'Trvalý',
                        'hu' => 'Kitartó',
                        'sv' => 'Beständig',
                    ],
                ],
                [
                    /* Youtube */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'yt-remote-fast-check-period',
                    'provider' => '	youtube.com',
                    'provider_url' => 'https://policies.google.com/technologies/cookies',
                    'purpose' => [
                        'en' => 'Stores the user\'s video player preferences using embedded YouTube video',
                        'es' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'ag' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'cb' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'mx' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'fr' => 'Stocke les préférences de lecture vidéo de l\'utilisateur pour les vidéos YouTube incorporées.',
                        'qc' => 'Stocke les préférences de lecture vidéo de l\'utilisateur pour les vidéos YouTube incorporées.',
                        'pl' => 'Przechowuje preferencje odtwarzacza wideo użytkownika za pomocą osadzonego wideo YouTube.',
                        'ro' => 'Stochează preferințele playerului video ale utilizatorului utilizând videoclipuri YouTube încorporate.',
                        'pt' => 'Armazena as preferências do player de vídeo do usuário usando o vídeo do YouTube incorporado.',
                        'br' => 'Armazena as preferências do player de vídeo do usuário usando o vídeo do YouTube incorporado.',
                        'sk' => 'Ukladá predvoľby prehrávača videa používateľa pomocou vloženého videa YouTube.',
                        'nl' => 'Bewaart de voorkeuren van de videospeler van de gebruiker met ingesloten YouTube-video',
                        'de' => 'Registriert eine eindeutige ID, um Statistiken der Videos von YouTube, die der Benutzer gesehen hat, zu behalten.',
                        'gr' => 'Αποθηκεύει τις προτιμήσεις του προγράμματος αναπαραγωγής βίντεο του χρήστη χρησιμοποιώντας ενσωματωμένο βίντεο YouTube.',
                        'it' => 'Memorizza le preferenze del lettore video dell\'utente usando il video YouTube incorporato',
                        'si' => 'Shrani nastavitve uporabnikovega video predvajalnika z uporabo vdelanega videoposnetka YouTube.',
                        'da' => 'Gemmer brugerens video-afspiller-præferencer ved afspilning af en indlejret YouTube video.',
                        'no' => 'Lagrer brukerens videospillerinnstillinger ved hjelp av innebygd YouTube-video.',
                        'cs' => 'Ukládá předvolby přehrávače videa uživatele pomocí vloženého videa YouTube.',
                        'hu' => 'A felhasználó videólejátszójának beállításait tárolja a beágyazott YouTube-videók segítségével.',
                        'sv' => 'Lagrar användarens videospelarinställningar med inbäddad YouTube-video.',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    /* Youtube */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'yt-remote-session-app',
                    'provider' => '	youtube.com',
                    'provider_url' => 'https://policies.google.com/technologies/cookies',
                    'purpose' => [
                        'en' => 'Stores the user\'s video player preferences using embedded YouTube video',
                        'es' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'ag' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'cb' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'mx' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'fr' => 'Stocke les préférences de lecture vidéo de l\'utilisateur pour les vidéos YouTube incorporées.',
                        'qc' => 'Stocke les préférences de lecture vidéo de l\'utilisateur pour les vidéos YouTube incorporées.',
                        'pl' => 'Przechowuje preferencje odtwarzacza wideo użytkownika za pomocą osadzonego wideo YouTube.',
                        'ro' => 'Stochează preferințele playerului video ale utilizatorului utilizând videoclipuri YouTube încorporate.',
                        'pt' => 'Armazena as preferências do player de vídeo do usuário usando o vídeo do YouTube incorporado.',
                        'br' => 'Armazena as preferências do player de vídeo do usuário usando o vídeo do YouTube incorporado.',
                        'sk' => 'Ukladá predvoľby prehrávača videa používateľa pomocou vloženého videa YouTube.',
                        'nl' => 'Bewaart de voorkeuren van de videospeler van de gebruiker met ingesloten YouTube-video',
                        'de' => 'Registriert eine eindeutige ID, um Statistiken der Videos von YouTube, die der Benutzer gesehen hat, zu behalten.',
                        'gr' => 'Αποθηκεύει τις προτιμήσεις του προγράμματος αναπαραγωγής βίντεο του χρήστη χρησιμοποιώντας ενσωματωμένο βίντεο YouTube.',
                        'it' => 'Memorizza le preferenze del lettore video dell\'utente usando il video YouTube incorporato',
                        'si' => 'Shrani nastavitve uporabnikovega video predvajalnika z uporabo vdelanega videoposnetka YouTube.',
                        'da' => 'Gemmer brugerens video-afspiller-præferencer ved afspilning af en indlejret YouTube video.',
                        'no' => 'Lagrer brukerens videospillerinnstillinger ved hjelp av innebygd YouTube-video.',
                        'cs' => 'Ukládá předvolby přehrávače videa uživatele pomocí vloženého videa YouTube.',
                        'hu' => 'A felhasználó videólejátszójának beállításait tárolja a beágyazott YouTube-videók segítségével.',
                        'sv' => 'Lagrar användarens videospelarinställningar med inbäddad YouTube-video',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    /* Youtube */
                    'active' => 0,
                    'modules' => [],
                    'name' => 'yt-remote-session-name',
                    'provider' => '	youtube.com',
                    'provider_url' => 'https://policies.google.com/technologies/cookies',
                    'purpose' => [
                        'en' => 'Stores the user\'s video player preferences using embedded YouTube video',
                        'es' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'ag' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'cb' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'mx' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'fr' => 'Stocke les préférences de lecture vidéo de l\'utilisateur pour les vidéos YouTube incorporées.',
                        'qc' => 'Stocke les préférences de lecture vidéo de l\'utilisateur pour les vidéos YouTube incorporées.',
                        'pl' => 'Przechowuje preferencje odtwarzacza wideo użytkownika za pomocą osadzonego wideo YouTube.',
                        'ro' => 'Stochează preferințele playerului video ale utilizatorului utilizând videoclipuri YouTube încorporate.',
                        'pt' => 'Armazena as preferências do player de vídeo do usuário usando o vídeo do YouTube incorporado.',
                        'br' => 'Armazena as preferências do player de vídeo do usuário usando o vídeo do YouTube incorporado.',
                        'sk' => 'Ukladá predvoľby prehrávača videa používateľa pomocou vloženého videa YouTube.',
                        'nl' => 'Bewaart de voorkeuren van de videospeler van de gebruiker met ingesloten YouTube-video',
                        'de' => 'Registriert eine eindeutige ID, um Statistiken der Videos von YouTube, die der Benutzer gesehen hat, zu behalten.',
                        'gr' => 'Αποθηκεύει τις προτιμήσεις του προγράμματος αναπαραγωγής βίντεο του χρήστη χρησιμοποιώντας ενσωματωμένο βίντεο YouTube.',
                        'it' => 'Memorizza le preferenze del lettore video dell\'utente usando il video YouTube incorporato',
                        'si' => 'Shrani nastavitve uporabnikovega video predvajalnika z uporabo vdelanega videoposnetka YouTube.',
                        'da' => 'Gemmer brugerens video-afspiller-præferencer ved afspilning af en indlejret YouTube video.',
                        'no' => 'Lagrer brukerens videospillerinnstillinger ved hjelp av innebygd YouTube-video.',
                        'cs' => 'Ukládá předvolby přehrávače videa uživatele pomocí vloženého videa YouTube.',
                        'hu' => 'A felhasználó videólejátszójának beállításait tárolja a beágyazott YouTube-videók segítségével.',
                        'sv' => 'Lagrar användarens videospelarinställningar med inbäddad YouTube-video',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [],
                    'name' => 'ads/ga-audiences',
                    'provider' => 'Google',
                    'provider_url' => 'https://policies.google.com/privacy',
                    'purpose' => [
                        'en' => 'These cookies are used by Google AdWords to re-engage visitors that are likely to convert to customers based on the visitor’s online behaviour across websites.',
                        'es' => 'Google AdWords utiliza estas cookies para volver a atraer a los visitantes que probablemente se conviertan en clientes en función del comportamiento en línea del visitante en los sitios web.',
                        'ag' => 'Google AdWords utiliza estas cookies para volver a atraer a los visitantes que probablemente se conviertan en clientes en función del comportamiento en línea del visitante en los sitios web.',
                        'cb' => 'Google AdWords utiliza estas cookies para volver a atraer a los visitantes que probablemente se conviertan en clientes en función del comportamiento en línea del visitante en los sitios web.',
                        'mx' => 'Google AdWords utiliza estas cookies para volver a atraer a los visitantes que probablemente se conviertan en clientes en función del comportamiento en línea del visitante en los sitios web.',
                        'fr' => 'Ces cookies sont utilisés par Google AdWords pour réengager les visiteurs susceptibles de se convertir en clients en fonction du comportement en ligne du visiteur sur les sites Web.',
                        'qc' => 'Ces cookies sont utilisés par Google AdWords pour réengager les visiteurs susceptibles de se convertir en clients en fonction du comportement en ligne du visiteur sur les sites Web.',
                        'pl' => 'Te pliki cookie są używane przez Google AdWords do ponownego angażowania użytkowników, którzy mogą przekształcić się w klientów na podstawie zachowania użytkownika online w różnych witrynach.',
                        'ro' => 'Aceste cookie-uri sunt utilizate de Google AdWords pentru a re-atrage vizitatori care ar putea converti la clienți pe baza comportamentului online al vizitatorului pe site-uri web.',
                        'pt' => 'Esses cookies são usados pelo Google AdWords para reconquistar visitantes que provavelmente se converterão em clientes com base no comportamento online do visitante nos sites.',
                        'br' => 'Esses cookies são usados pelo Google AdWords para reconquistar visitantes que provavelmente se converterão em clientes com base no comportamento online do visitante nos sites.',
                        'sk' => 'Tieto cookies používa Google AdWords na opätovné zapojenie návštevníkov, u ktorých je pravdepodobné, že sa prevedú na zákazníkov na základe online správania návštevníka na rôznych webových stránkach.',
                        'nl' => 'Deze cookies worden door Google AdWords gebruikt om bezoekers opnieuw aan te spreken die waarschijnlijk in klanten zullen worden omgezet op basis van het online gedrag van de bezoeker op verschillende websites.',
                        'de' => 'Diese Cookies werden von Google AdWords verwendet, um Besucher wieder einzubeziehen, die aufgrund des Online-Verhaltens des Besuchers auf verschiedenen Websites wahrscheinlich zu Kunden werden.',
                        'gr' => 'Αυτά τα cookie χρησιμοποιούνται από το Google AdWords για να προσελκύσουν εκ νέου επισκέπτες που είναι πιθανό να μετατρέψουν σε πελάτες βάσει της διαδικτυακής συμπεριφοράς του επισκέπτη σε ιστότοπους.',
                        'it' => 'Questi cookie vengono utilizzati da Google AdWords per coinvolgere nuovamente i visitatori che potrebbero convertirsi in clienti in base al comportamento online del visitatore sui siti web.',
                        'si' => 'Tieto cookies používa Google AdWords na opätovné zapojenie návštevníkov, u ktorých je pravdepodobné, že sa prevedú na zákazníkov na základe online správania návštevníka na rôznych webových stránkach.',
                        'da' => 'Disse cookies bruges af Google AdWords til at engagere besøgende igen, som sandsynligvis konverterer til kunder baseret på den besøgendes online adfærd på tværs af websteder.',
                        'no' => 'Disse informasjonskapslene brukes av Google AdWords for å engasjere besøkende som sannsynligvis kan konvertere til kunder, basert på den besøkendes online atferd på tvers av nettsteder.',
                        'cs' => 'Tyto soubory cookie používá Google AdWords k opětovnému zapojení návštěvníků, u nichž je pravděpodobné, že se převedou na zákazníky na základě online chování návštěvníka na různých webech.',
                        'hu' => 'Ezeket a cookie-kat a Google AdWords arra használja, hogy újból bevonja azokat a látogatókat, akik valószínűleg a látogatók webhelyeken keresztüli online viselkedése alapján vásárlóvá válnak.',
                        'sv' => 'Dessa cookies används av Google AdWords för att återanvända besökare som sannolikt kommer att konvertera till kunder baserat på besökarens onlinebeteende över webbplatser.',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [],
                    'name' => 'yt-player-headers-readable',
                    'provider' => 'youtube.com',
                    'provider_url' => '',
                    'purpose' => [
                        'en' => 'Used to determine the optimal video quality based on the visitor\'s device and network settings.',
                        'es' => 'Utilizada para determinar la calidad óptima del video basada en el dispositivo del visitante y en los ajustes de la red.',
                        'ag' => '',
                        'cb' => '',
                        'mx' => '',
                        'fr' => 'Utilisé pour déterminer la qualité vidéo optimale en fonction du périphériqu e du visiteur et des paramètres réseau.',
                        'qc' => '',
                        'pl' => '',
                        'ro' => '',
                        'pt' => '',
                        'br' => '',
                        'sk' => '',
                        'nl' => '',
                        'de' => 'Wird verwendet, um basierend auf den Geräte-und Netzwerkeinstellungen des Besuchers die optimale Videoqualität zuermitteln.',
                        'gr' => '',
                        'it' => 'Utilizzato per determinare la qualità video ottimale in base al dispositivo del visitatore e alle impostazioni di rete.',
                        'si' => '',
                        'da' => 'Bestemmer den optimale video kvalitet ud fraden besøgendes platform og netværkshastighed.',
                        'no' => '',
                        'cs' => '',
                        'hu' => '',
                        'sv' => '',
                    ],
                    'expiry' => [
                        'en' => 'Persistent',
                        'es' => 'Persistente',
                        'ag' => 'Persistente',
                        'cb' => 'Persistente',
                        'mx' => 'Persistente',
                        'fr' => 'Persistant',
                        'qc' => 'Persistant',
                        'pl' => 'Trwały',
                        'ro' => 'Persistent',
                        'pt' => 'Persistente',
                        'br' => 'Persistente',
                        'sk' => 'Vytrvalý',
                        'nl' => 'Aanhoudend',
                        'de' => 'Hartnäckig',
                        'gr' => 'Επίμονος',
                        'it' => 'Persistente',
                        'si' => 'Vztrajno',
                        'da' => 'Vedholdende',
                        'no' => 'Vedvarende',
                        'cs' => 'Trvalý',
                        'hu' => 'Kitartó',
                        'sv' => 'Beständig',
                    ],
                ],
            ],
            CookiesPlusFinality::PREFERENCE_COOKIE => [
                [
                    'active' => 0,
                    'modules' => ['smartsupp'],
                    'name' => 'ssupp.opened',
                    'provider' => 'Smartsupp',
                    'provider_url' => 'https://www.smartsupp.com/help/privacy/',
                    'purpose' => [
                        'en' => 'Check if chat box is opened from live chat service provided by Smartsupp.',
                        'es' => 'Comprueba si el cuadro de chat está abierto desde el servicio de chat en vivo proporcionado por Smartsupp.',
                        'ag' => 'Comprueba si el cuadro de chat está abierto desde el servicio de chat en vivo proporcionado por Smartsupp.',
                        'cb' => 'Comprueba si el cuadro de chat está abierto desde el servicio de chat en vivo proporcionado por Smartsupp.',
                        'mx' => 'Comprueba si el cuadro de chat está abierto desde el servicio de chat en vivo proporcionado por Smartsupp.',
                        'fr' => 'Vérifie si la boîte de discussion est ouverte à partir du service de chat en direct fourni par Smartsupp.',
                        'qc' => 'Vérifie si la boîte de discussion est ouverte à partir du service de chat en direct fourni par Smartsupp.',
                        'pl' => 'Sprawdza, czy okno czatu jest otwierane z usługi czatu na żywo dostarczanej przez Smartsupp.',
                        'ro' => 'Verifică dacă caseta de chat este deschisă din serviciul de chat live furnizat de Smartsupp.',
                        'pt' => 'Verifica se a caixa de bate-papo é aberta a partir do serviço de bate-papo ao vivo fornecido pela Smartsupp.',
                        'br' => 'Verifica se a caixa de bate-papo é aberta a partir do serviço de bate-papo ao vivo fornecido pela Smartsupp.',
                        'sk' => 'Skontroluje, či je okno chatu otvorené zo služby živého chatu poskytovanej spoločnosťou Smartsupp.',
                        'nl' => 'Controleert of de chatbox wordt geopend vanuit de live chatservice van Smartsupp.',
                        'de' => 'Überprüft, ob das Chatfeld über den von Smartsupp bereitgestellten Live-Chat-Dienst geöffnet ist.',
                        'gr' => 'Ελέγχει εάν έχει ανοίξει το πλαίσιο συνομιλίας από την υπηρεσία ζωντανής συνομιλίας που παρέχεται από το Smartsupp.',
                        'it' => 'Controlla se la casella di chat è aperta dal servizio di chat dal vivo fornito da Smartsupp.',
                        'si' => 'Preveri, ali se polje za klepet odpre s storitvijo klepeta v živo, ki jo nudi Smartsupp.',
                        'da' => 'Kontrollerer, om chatfeltet åbnes fra live chat-tjenesten leveret af Smartsupp.',
                        'no' => 'Sjekker om chat-boksen åpnes fra live chat-tjenesten levert av Smartsupp.',
                        'cs' => 'Zkontroluje, zda je okno chatu otevřeno ze služby živého chatu poskytované společností Smartsupp.',
                        'hu' => 'Ellenőrzi, hogy a csevegőmező nyitva van-e a Smartsupp által biztosított élő csevegési szolgáltatásból.',
                        'sv' => 'Kontrollera om chattrutan öppnas från livechatttjänsten från Smartsupp.',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => ['smartsupp'],
                    'name' => 'ssupp.barclicked',
                    'provider' => 'Smartsupp',
                    'provider_url' => 'https://www.smartsupp.com/help/privacy/',
                    'purpose' => [
                        'en' => 'Needed for automatic messages from live chat service provided by Smartsupp.',
                        'es' => 'Necesario para mensajes automáticos del servicio de chat en vivo proporcionado por Smartsupp.',
                        'ag' => 'Necesario para mensajes automáticos del servicio de chat en vivo proporcionado por Smartsupp.',
                        'cb' => 'Necesario para mensajes automáticos del servicio de chat en vivo proporcionado por Smartsupp.',
                        'mx' => 'Necesario para mensajes automáticos del servicio de chat en vivo proporcionado por Smartsupp.',
                        'fr' => 'Nécessaire pour les messages automatiques du service de chat en direct fourni par Smartsupp.',
                        'qc' => 'Nécessaire pour les messages automatiques du service de chat en direct fourni par Smartsupp.',
                        'pl' => 'Potrzebne do automatycznych wiadomości z usługi czatu na żywo dostarczanej przez Smartsupp.',
                        'ro' => 'Este necesar pentru mesaje automate din serviciul de chat live furnizat de Smartsupp.',
                        'pt' => 'Necessário para mensagens automáticas do serviço de chat ao vivo fornecido pela Smartsupp.',
                        'br' => 'Necessário para mensagens automáticas do serviço de chat ao vivo fornecido pela Smartsupp.',
                        'sk' => 'Potrebné pre automatické správy zo služby živého chatu poskytované spoločnosťou Smartsupp.',
                        'nl' => 'Nodig voor automatische berichten van de live chatservice van Smartsupp.',
                        'de' => 'Wird für automatische Nachrichten vom Live-Chat-Dienst von Smartsupp benötigt.',
                        'gr' => 'Απαιτείται για αυτόματα μηνύματα από την υπηρεσία ζωντανής συνομιλίας που παρέχεται από το Smartsupp.',
                        'it' => 'Necessario per i messaggi automatici dal servizio di live chat fornito da Smartsupp.',
                        'si' => 'Potrebno za samodejna sporočila iz storitve klepeta v živo, ki jo ponuja Smartsupp.',
                        'da' => 'Nødvendigt til automatiske meddelelser fra live chat-tjeneste leveret af Smartsupp.',
                        'no' => 'Nødvendig for automatiske meldinger fra live chat-tjenesten levert av Smartsupp.',
                        'cs' => 'Potřebné pro automatické zprávy ze služby živého chatu poskytované společností Smartsupp.',
                        'hu' => 'Szükséges a Smartsupp által biztosított élő chat szolgáltatás automatikus üzeneteihez.',
                        'sv' => 'Behövs för automatiska meddelanden från chattjänsten som tillhandahålls av Smartsupp.',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => ['smartsupp'],
                    'name' => 'ssupp.message',
                    'provider' => 'Smartsupp',
                    'provider_url' => 'https://www.smartsupp.com/help/privacy/',
                    'purpose' => [
                        'en' => 'Stores content in text area from live chat service provided by Smartsupp.',
                        'es' => 'Almacena contenido en el área de texto del servicio de chat en vivo proporcionado por Smartsupp.',
                        'ag' => 'Almacena contenido en el área de texto del servicio de chat en vivo proporcionado por Smartsupp.',
                        'cb' => 'Almacena contenido en el área de texto del servicio de chat en vivo proporcionado por Smartsupp.',
                        'mx' => 'Almacena contenido en el área de texto del servicio de chat en vivo proporcionado por Smartsupp.',
                        'fr' => 'Stocke le contenu dans la zone de texte du service de chat en direct fourni par Smartsupp.',
                        'qc' => 'Stocke le contenu dans la zone de texte du service de chat en direct fourni par Smartsupp.',
                        'pl' => 'Przechowuje zawartość w obszarze tekstowym z usługi czatu na żywo dostarczanej przez Smartsupp.',
                        'ro' => 'Stochează conținut în zona de text din serviciul de chat live furnizat de Smartsupp.',
                        'pt' => 'Armazena conteúdo na área de texto do serviço de chat ao vivo fornecido pela Smartsupp.',
                        'br' => 'Armazena conteúdo na área de texto do serviço de chat ao vivo fornecido pela Smartsupp.',
                        'sk' => 'Ukladá obsah do textovej oblasti zo služby živého chatu poskytovanej spoločnosťou Smartsupp.',
                        'nl' => 'Slaat inhoud op in het tekstgedeelte van de live chatservice van Smartsupp.',
                        'de' => 'Speichert Inhalte im Textbereich des von Smartsupp bereitgestellten Live-Chat-Dienstes.',
                        'gr' => 'Αποθηκεύει περιεχόμενο στην περιοχή κειμένου από την υπηρεσία ζωντανής συνομιλίας που παρέχεται από το Smartsupp.',
                        'it' => 'Memorizza il contenuto nell\'area di testo dal servizio di chat dal vivo fornito da Smartsupp.',
                        'si' => 'Shrani vsebino v besedilno območje iz storitve klepeta v živo, ki jo ponuja Smartsupp.',
                        'da' => 'Gemmer indhold i tekstområdet fra live chat-tjeneste leveret af Smartsupp.',
                        'no' => 'Lagrer innhold i tekstområdet fra live chat-tjenesten levert av Smartsupp.',
                        'cs' => 'Ukládá obsah do textové oblasti ze služby živého chatu poskytované společností Smartsupp.',
                        'hu' => 'A szöveges területen tárolja a Smartsupp által biztosított élő csevegőszolgáltatás tartalmát.',
                        'sv' => 'Lagrar innehåll i textområdet från livechatttjänsten från Smartsupp.',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => ['smartsupp'],
                    'name' => 'ssupp.unreaded',
                    'provider' => 'Smartsupp',
                    'provider_url' => 'https://www.smartsupp.com/help/privacy/',
                    'purpose' => [
                        'en' => 'Number of unread messages from live chat service provided by Smartsupp.',
                        'es' => 'Número de mensajes no leídos del servicio de chat en vivo proporcionado por Smartsupp.',
                        'ag' => 'Número de mensajes no leídos del servicio de chat en vivo proporcionado por Smartsupp.',
                        'cb' => 'Número de mensajes no leídos del servicio de chat en vivo proporcionado por Smartsupp.',
                        'mx' => 'Número de mensajes no leídos del servicio de chat en vivo proporcionado por Smartsupp.',
                        'fr' => 'Nombre de messages non lus du service de chat en direct fourni par Smartsupp.',
                        'qc' => 'Nombre de messages non lus du service de chat en direct fourni par Smartsupp.',
                        'pl' => 'Liczba nieprzeczytanych wiadomości z usługi czatu na żywo świadczonej przez Smartsupp.',
                        'ro' => 'Numărul de mesaje necitite din serviciul de chat live furnizat de Smartsupp.',
                        'pt' => 'Número de mensagens não lidas do serviço de chat ao vivo fornecido pela Smartsupp.',
                        'br' => 'Número de mensagens não lidas do serviço de chat ao vivo fornecido pela Smartsupp.',
                        'sk' => 'Počet neprečítaných správ zo služby živého chatu poskytovaných spoločnosťou Smartsupp.',
                        'nl' => 'Aantal ongelezen berichten van live chatservice geleverd door Smartsupp.',
                        'de' => 'Anzahl der ungelesenen Nachrichten vom Live-Chat-Dienst von Smartsupp.',
                        'gr' => 'Αριθμός μη αναγνωσμένων μηνυμάτων από την υπηρεσία ζωντανής συνομιλίας που παρέχεται από το Smartsupp.',
                        'it' => 'Numero di messaggi non letti dal servizio di chat dal vivo fornito da Smartsupp.',
                        'si' => 'Število neprebranih sporočil iz storitve klepeta v živo, ki jo nudi Smartsupp.',
                        'da' => 'Antal ulæste beskeder fra live chat-tjeneste leveret af Smartsupp.',
                        'no' => 'Antall uleste meldinger fra live chat-tjenesten levert av Smartsupp.',
                        'cs' => 'Počet nepřečtených zpráv ze služby živého chatu poskytovaných Smartsupp.',
                        'hu' => 'A Smartsupp által biztosított élő csevegőszolgáltatás olvasatlan üzeneteinek száma.',
                        'sv' => 'Antal olästa meddelanden från chattjänsten från Smartsupp.',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [],
                    'name' => 'DEVICE_INFO',
                    'provider' => 'youtube.com',
                    'provider_url' => '',
                    'purpose' => [
                        'en' => 'Used to track user\'s interaction with embedded content.',
                        'es' => 'Se usa para rastrear la interacción del usuario con el contenido integrado.',
                        'ag' => 'Se usa para rastrear la interacción del usuario con el contenido integrado.',
                        'cb' => 'Se usa para rastrear la interacción del usuario con el contenido integrado.',
                        'mx' => 'Se usa para rastrear la interacción del usuario con el contenido integrado.',
                        'fr' => 'Utilisé pour suivre l\'interaction de l\'utilisateur avec le contenu intégré.',
                        'qc' => 'Utilisé pour suivre l\'interaction de l\'utilisateur avec le contenu intégré.',
                        'pl' => '',
                        'ro' => '',
                        'pt' => '',
                        'br' => '',
                        'sk' => '',
                        'nl' => '',
                        'de' => 'Wird verwendet, um die Interaktion der Nutzer mit eingebetteten Inhalten zu verfolgen.',
                        'gr' => '',
                        'it' => 'Utilizzato per tracciare l\'interazione dell\'utente con i contenuti incorporati',
                        'si' => '',
                        'da' => 'Benyttes til indsamling data omhandlen debrugerens interaktion med indlejret indhold.',
                        'no' => '',
                        'cs' => '',
                        'hu' => '',
                        'sv' => '',
                    ],
                    'expiry' => [
                        'en' => '180 days',
                        'es' => '180 días',
                        'ag' => '180 días',
                        'cb' => '180 días',
                        'mx' => '180 días',
                        'fr' => '180 jours',
                        'qc' => '180 jours',
                        'pl' => '180 dni',
                        'ro' => '180 de zile',
                        'pt' => '180 dias',
                        'br' => '180 dias',
                        'sk' => '180 dní',
                        'nl' => '180 dagen',
                        'de' => '180 Tage',
                        'gr' => '180 μέρες',
                        'it' => '180 giorni',
                        'si' => '180 dni',
                        'da' => '180 dage',
                        'no' => '180 dager',
                        'cs' => '180 dager',
                        'hu' => '180 nap',
                        'sv' => '180 dagar',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [],
                    'name' => 'LAST_RESULT_ENTRY_KEY',
                    'provider' => 'youtube.com',
                    'provider_url' => '',
                    'purpose' => [
                        'en' => '',
                        'es' => '',
                        'ag' => '',
                        'cb' => '',
                        'mx' => '',
                        'fr' => '',
                        'qc' => '',
                        'pl' => '',
                        'ro' => '',
                        'pt' => '',
                        'br' => '',
                        'sk' => '',
                        'nl' => '',
                        'de' => '',
                        'gr' => '',
                        'it' => '',
                        'si' => '',
                        'da' => '',
                        'no' => '',
                        'cs' => '',
                        'hu' => '',
                        'sv' => '',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [],
                    'name' => 'TESTCOOKIESENABLED',
                    'provider' => 'youtube.com',
                    'provider_url' => '',
                    'purpose' => [
                        'en' => '',
                        'es' => '',
                        'ag' => '',
                        'cb' => '',
                        'mx' => '',
                        'fr' => '',
                        'qc' => '',
                        'pl' => '',
                        'ro' => '',
                        'pt' => '',
                        'br' => '',
                        'sk' => '',
                        'nl' => '',
                        'de' => '',
                        'gr' => '',
                        'it' => '',
                        'si' => '',
                        'da' => '',
                        'no' => '',
                        'cs' => '',
                        'hu' => '',
                        'sv' => '',
                    ],
                    'expiry' => [
                        'en' => '1 day',
                        'es' => '1 día',
                        'ag' => '1 día',
                        'cb' => '1 día',
                        'mx' => '1 día',
                        'fr' => '1 jour',
                        'qc' => '1 jour',
                        'pl' => '1 dzień',
                        'ro' => '1 zi',
                        'pt' => '1 dia',
                        'br' => '1 dia',
                        'sk' => '1 deň',
                        'nl' => '1 dag',
                        'de' => '1 Tag',
                        'gr' => '1 μέρα',
                        'it' => '1 giorno',
                        'si' => '1 dan',
                        'da' => '1 dag',
                        'no' => '1 dag',
                        'cs' => '1 den',
                        'hu' => '1 nap',
                        'sv' => '1 dag',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [],
                    'name' => 'VISITOR_INFO1_LIVE',
                    'provider' => 'youtube.com',
                    'provider_url' => '',
                    'purpose' => [
                        'en' => 'Tries to estimate the users\' bandwidth on pages with integrated YouTube videos.',
                        'es' => 'Intenta calcular el ancho de banda del usuario en páginas con vídeos de YouTube integrados.',
                        'ag' => 'Intenta calcular el ancho de banda del usuario en páginas con vídeos de YouTube integrados.',
                        'cb' => 'Intenta calcular el ancho de banda del usuario en páginas con vídeos de YouTube integrados.',
                        'mx' => 'Intenta calcular el ancho de banda del usuario en páginas con vídeos de YouTube integrados.',
                        'fr' => 'Tente d\'estimer la bande passante des utilisateurs sur des pages avec des vidéos YouTube intégrées',
                        'qc' => 'Tente d\'estimer la bande passante des utilisateurs sur des pages avec des vidéos YouTube intégrées',
                        'pl' => '',
                        'ro' => '',
                        'pt' => '',
                        'br' => '',
                        'sk' => '',
                        'nl' => '',
                        'de' => 'Versucht, die Benutzerbandbreite auf Seiten mit integrierten YouTube-Videos zu schätzen.',
                        'gr' => '',
                        'it' => 'Prova a stimare la velocità della connession e dell\'u ente su pagine con video YouTube integrati.',
                        'si' => '',
                        'da' => 'Forsøger at estimere brugernes båndbredde på sider med integreret YouTube-video.',
                        'no' => '',
                        'cs' => '',
                        'hu' => '',
                        'sv' => '',
                    ],
                    'expiry' => [
                        'en' => '180 days',
                        'es' => '180 días',
                        'ag' => '180 días',
                        'cb' => '180 días',
                        'mx' => '180 días',
                        'fr' => '180 jours',
                        'qc' => '180 jours',
                        'pl' => '180 dni',
                        'ro' => '180 de zile',
                        'pt' => '180 dias',
                        'br' => '180 dias',
                        'sk' => '180 dní',
                        'nl' => '180 dagen',
                        'de' => '180 Tage',
                        'gr' => '180 μέρες',
                        'it' => '180 giorni',
                        'si' => '180 dni',
                        'da' => '180 dage',
                        'no' => '180 dager',
                        'cs' => '180 dager',
                        'hu' => '180 nap',
                        'sv' => '180 dagar',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [],
                    'name' => 'YSC',
                    'provider' => 'youtube.com',
                    'provider_url' => '',
                    'purpose' => [
                        'en' => 'Registers a unique ID to keep statistics of what videos from YouTube the user has seen.',
                        'es' => 'Registra una identificación única para mantener estadísticas de qué vídeos de YouTube ha visto el usuario.',
                        'ag' => 'Registra una identificación única para mantener estadísticas de qué vídeos de YouTube ha visto el usuario.',
                        'cb' => 'Registra una identificación única para mantener estadísticas de qué vídeos de YouTube ha visto el usuario.',
                        'mx' => 'Registra una identificación única para mantener estadísticas de qué vídeos de YouTube ha visto el usuario.',
                        'fr' => 'Tente d\'estimer la bande passante des utilisateurs sur des pages avec des vidéos YouTube intégrées.',
                        'qc' => 'Tente d\'estimer la bande passante des utilisateurs sur des pages avec des vidéos YouTube intégrées.',
                        'pl' => '',
                        'ro' => '',
                        'pt' => '',
                        'br' => '',
                        'sk' => '',
                        'nl' => '',
                        'de' => 'Registriert eine eindeutige ID, um Statistiken der Videos von YouTube, die der Benutzer gesehen hat, zu behalten.',
                        'gr' => '',
                        'it' => 'Registra un ID univoco per statistiche legate a quali video YouTube sono stati visualizzati dall\'utente.',
                        'si' => '',
                        'da' => 'Registrerer et unikt ID for at føre statistik over hvilke videoer fra YouTube brugeren har set.',
                        'no' => '',
                        'cs' => '',
                        'hu' => '',
                        'sv' => '',
                    ],
                    'expiry' => [
                        'en' => 'Session',
                        'es' => 'Sesión',
                        'ag' => 'Sesión',
                        'cb' => 'Sesión',
                        'mx' => 'Sesión',
                        'fr' => 'Session',
                        'qc' => 'Session',
                        'pl' => 'Sesja',
                        'ro' => 'Sesiune',
                        'pt' => 'Sessão',
                        'br' => 'Sessão',
                        'sk' => 'Session',
                        'nl' => 'Sessie',
                        'de' => 'Session',
                        'gr' => 'Συνεδρία',
                        'it' => 'Sessione',
                        'si' => 'Seja',
                        'da' => 'Session',
                        'no' => 'Økt',
                        'cs' => 'Zasedání',
                        'hu' => 'Ülés',
                        'sv' => 'Session',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [],
                    'name' => 'ytidb::LAST_RESULT_ENTRY_KEY',
                    'provider' => 'youtube.com',
                    'provider_url' => '',
                    'purpose' => [
                        'en' => 'It records the user\'s video player preferences when viewing embedded YouTube videos.',
                        'es' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'ag' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'cb' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'mx' => 'Registra las preferencias del reproductor de vídeo del usuario al ver vídeos incrustados de YouTube.',
                        'fr' => 'Stocke les préférences de lecture vidéo de l\'utilisateur pour les vidéos YouTube incorporées.',
                        'qc' => 'Stocke les préférences de lecture vidéo de l\'utilisateur pour les vidéos YouTube incorporées.',
                        'pl' => '',
                        'ro' => '',
                        'pt' => '',
                        'br' => '',
                        'sk' => '',
                        'nl' => '',
                        'de' => 'Speichert die Benutzereinstellungen beim Abruf eines auf anderen Webseiten integrierten Youtube-Videos.',
                        'gr' => '',
                        'it' => 'Memorizza le preferenze del lettore video dell\'u tente usando il video YouTube incorporato.',
                        'si' => '',
                        'da' => 'Gemmer brugerens video-afspiller-præferencer ved afspilning af en indlejret YouTube video.',
                        'no' => '',
                        'cs' => '',
                        'hu' => '',
                        'sv' => '',
                    ],
                    'expiry' => [
                        'en' => 'Persistent',
                        'es' => 'Persistente',
                        'ag' => 'Persistente',
                        'cb' => 'Persistente',
                        'mx' => 'Persistente',
                        'fr' => 'Persistant',
                        'qc' => 'Persistant',
                        'pl' => 'Trwały',
                        'ro' => 'Persistent',
                        'pt' => 'Persistente',
                        'br' => 'Persistente',
                        'sk' => 'Vytrvalý',
                        'nl' => 'Aanhoudend',
                        'de' => 'Hartnäckig',
                        'gr' => 'Επίμονος',
                        'it' => 'Persistente',
                        'si' => 'Vztrajno',
                        'da' => 'Vedholdende',
                        'no' => 'Vedvarende',
                        'cs' => 'Trvalý',
                        'hu' => 'Kitartó',
                        'sv' => 'Beständig',
                    ],
                ],
                [
                    'active' => 0,
                    'modules' => [],
                    'name' => 'YtIdbMeta#databases',
                    'provider' => 'youtube.com',
                    'provider_url' => '',
                    'purpose' => [
                        'en' => '',
                        'es' => '',
                        'ag' => '',
                        'cb' => '',
                        'mx' => '',
                        'fr' => '',
                        'qc' => '',
                        'pl' => '',
                        'ro' => '',
                        'pt' => '',
                        'br' => '',
                        'sk' => '',
                        'nl' => '',
                        'de' => '',
                        'gr' => '',
                        'it' => '',
                        'si' => '',
                        'da' => '',
                        'no' => '',
                        'cs' => '',
                        'hu' => '',
                        'sv' => '',
                    ],
                    'expiry' => [
                        'en' => 'Persistent',
                        'es' => 'Persistente',
                        'ag' => 'Persistente',
                        'cb' => 'Persistente',
                        'mx' => 'Persistente',
                        'fr' => 'Persistant',
                        'qc' => 'Persistant',
                        'pl' => 'Trwały',
                        'ro' => 'Persistent',
                        'pt' => 'Persistente',
                        'br' => 'Persistente',
                        'sk' => 'Vytrvalý',
                        'nl' => 'Aanhoudend',
                        'de' => 'Hartnäckig',
                        'gr' => 'Επίμονος',
                        'it' => 'Persistente',
                        'si' => 'Vztrajno',
                        'da' => 'Vedholdende',
                        'no' => 'Vedvarende',
                        'cs' => 'Trvalý',
                        'hu' => 'Kitartó',
                        'sv' => 'Beständig',
                    ],
                ],
                [
                    /* Tawk.to */
                    'active' => 0,
                    'modules' => ['tawkto', 'tawktoconfig'],
                    'name' => 'twk_uuid_#',
                    'provider' => Tools::getShopDomainSsl(true),
                    'provider_url' => '',
                    'purpose' => [
                        'en' => '',
                        'es' => '',
                        'ag' => '',
                        'cb' => '',
                        'mx' => '',
                        'fr' => '',
                        'qc' => '',
                        'pl' => '',
                        'ro' => '',
                        'pt' => '',
                        'br' => '',
                        'sk' => '',
                        'nl' => '',
                        'de' => '',
                        'gr' => '',
                        'it' => '',
                        'si' => '',
                        'da' => '',
                        'no' => '',
                        'cs' => '',
                        'hu' => '',
                        'sv' => '',
                    ],
                    'expiry' => [
                        'en' => '180 days',
                        'es' => '180 días',
                        'ag' => '180 días',
                        'cb' => '180 días',
                        'mx' => '180 días',
                        'fr' => '180 jours',
                        'qc' => '180 jours',
                        'pl' => '180 dni',
                        'ro' => '180 de zile',
                        'pt' => '180 dias',
                        'br' => '180 dias',
                        'sk' => '180 dní',
                        'nl' => '180 dagen',
                        'de' => '180 Tage',
                        'gr' => '180 μέρες',
                        'it' => '180 giorni',
                        'si' => '180 dni',
                        'da' => '180 dage',
                        'no' => '180 dager',
                        'cs' => '180 dager',
                        'hu' => '180 nap',
                        'sv' => '180 dagar',
                    ],
                ],
            ],
            CookiesPlusFinality::PERFORMANCE_COOKIE => [
                [
                    'active' => 0,
                    'name' => 'AWSALB',
                    'modules' => ['smartsupp'],
                    'provider' => 'Smartsupp',
                    'provider_url' => 'https://www.smartsupp.com/help/privacy/',
                    'purpose' => [
                        'en' => 'Generated by AWS (Amazon Web Services), needed for sending the requests to the server correctly',
                        'es' => 'Generado por AWS (Amazon Web Services), necesario para enviar correctamente las solicitudes al servidor.',
                        'ag' => 'Generado por AWS (Amazon Web Services), necesario para enviar correctamente las solicitudes al servidor.',
                        'cb' => 'Generado por AWS (Amazon Web Services), necesario para enviar correctamente las solicitudes al servidor.',
                        'mx' => 'Generado por AWS (Amazon Web Services), necesario para enviar correctamente las solicitudes al servidor.',
                        'fr' => 'Généré par AWS (Amazon Web Services), nécessaire pour envoyer correctement les requêtes au serveur.',
                        'qc' => 'Généré par AWS (Amazon Web Services), nécessaire pour envoyer correctement les requêtes au serveur.',
                        'pl' => 'Wygenerowane przez AWS (Amazon Web Services), potrzebne do prawidłowego wysyłania żądań do serwera.',
                        'ro' => 'Generat de AWS (Amazon Web Services), necesar pentru trimiterea corectă a cererilor către server.',
                        'pt' => 'Gerado pela AWS (Amazon Web Services), necessário para o envio correto das solicitações ao servidor.',
                        'br' => 'Gerado pela AWS (Amazon Web Services), necessário para o envio correto das solicitações ao servidor.',
                        'sk' => 'Generované AWS (Amazon Web Services), potrebné na správne odoslanie požiadaviek na server.',
                        'nl' => 'Gegenereerd door AWS (Amazon Web Services), nodig om de verzoeken correct naar de server te sturen.',
                        'de' => 'Wird von AWS (Amazon Web Services) generiert und wird zum korrekten Senden der Anforderungen an den Server benötigt.',
                        'gr' => 'Δημιουργήθηκε από AWS (Amazon Web Services), που απαιτείται για την σωστή αποστολή των αιτημάτων στον διακομιστή.',
                        'it' => 'Generato da AWS (Amazon Web Services), necessario per inviare correttamente le richieste al server.',
                        'si' => 'Ustvari AWS (Amazon Web Services), potreben za pravilno pošiljanje zahtev na strežnik.',
                        'da' => 'Genereret af AWS (Amazon Web Services), der er nødvendigt for at sende anmodningerne korrekt til serveren.',
                        'no' => 'Generert av AWS (Amazon Web Services), nødvendig for å sende forespørslene til serveren riktig.',
                        'cs' => 'Generováno AWS (Amazon Web Services), potřebné pro správné odesílání požadavků na server.',
                        'hu' => 'Az AWS (Amazon Web Services) által generált, a kérések szerverre történő megfelelő elküldéséhez szükséges.',
                        'sv' => 'Genereras av AWS (Amazon Web Services), behövs för att skicka förfrågningarna till servern korrekt',
                    ],
                    'expiry' => [
                        'en' => '7 days',
                        'es' => '7 días',
                        'ag' => '7 días',
                        'cb' => '7 días',
                        'mx' => '7 días',
                        'fr' => '7 jours',
                        'qc' => '7 jours',
                        'pl' => '7 dni',
                        'ro' => '7 de zile',
                        'pt' => '7 dias',
                        'br' => '7 dias',
                        'sk' => '7 dní',
                        'nl' => '7 dagen',
                        'de' => '7 Tage',
                        'gr' => '7 μέρες',
                        'it' => '7 giorni',
                        'si' => '7 dni',
                        'da' => '7 dage',
                        'no' => '7 dager',
                        'cs' => '7 dager',
                        'hu' => '7 nap',
                        'sv' => '7 dagar',
                    ],
                ],
                [
                    'active' => 0,
                    'name' => 'AWSALBCORS',
                    'modules' => ['smartsupp'],
                    'provider' => 'Smartsupp',
                    'provider_url' => 'https://www.smartsupp.com/help/privacy/',
                    'purpose' => [
                        'en' => 'Generated by AWS (Amazon Web Services), needed for sending the requests to the server correctly',
                        'es' => 'Generado por AWS (Amazon Web Services), necesario para enviar correctamente las solicitudes al servidor.',
                        'ag' => 'Generado por AWS (Amazon Web Services), necesario para enviar correctamente las solicitudes al servidor.',
                        'cb' => 'Generado por AWS (Amazon Web Services), necesario para enviar correctamente las solicitudes al servidor.',
                        'mx' => 'Generado por AWS (Amazon Web Services), necesario para enviar correctamente las solicitudes al servidor.',
                        'fr' => 'Généré par AWS (Amazon Web Services), nécessaire pour envoyer correctement les requêtes au serveur.',
                        'qc' => 'Généré par AWS (Amazon Web Services), nécessaire pour envoyer correctement les requêtes au serveur.',
                        'pl' => 'Wygenerowane przez AWS (Amazon Web Services), potrzebne do prawidłowego wysyłania żądań do serwera.',
                        'ro' => 'Generat de AWS (Amazon Web Services), necesar pentru trimiterea corectă a cererilor către server.',
                        'pt' => 'Gerado pela AWS (Amazon Web Services), necessário para o envio correto das solicitações ao servidor.',
                        'br' => 'Gerado pela AWS (Amazon Web Services), necessário para o envio correto das solicitações ao servidor.',
                        'sk' => 'Generované AWS (Amazon Web Services), potrebné na správne odoslanie požiadaviek na server.',
                        'nl' => 'Gegenereerd door AWS (Amazon Web Services), nodig om de verzoeken correct naar de server te sturen.',
                        'de' => 'Wird von AWS (Amazon Web Services) generiert und wird zum korrekten Senden der Anforderungen an den Server benötigt.',
                        'gr' => 'Δημιουργήθηκε από AWS (Amazon Web Services), που απαιτείται για την σωστή αποστολή των αιτημάτων στον διακομιστή.',
                        'it' => 'Generato da AWS (Amazon Web Services), necessario per inviare correttamente le richieste al server.',
                        'si' => 'Ustvari AWS (Amazon Web Services), potreben za pravilno pošiljanje zahtev na strežnik.',
                        'da' => 'Genereret af AWS (Amazon Web Services), der er nødvendigt for at sende anmodningerne korrekt til serveren.',
                        'no' => 'Generert av AWS (Amazon Web Services), nødvendig for å sende forespørslene til serveren riktig.',
                        'cs' => 'Generováno AWS (Amazon Web Services), potřebné pro správné odesílání požadavků na server.',
                        'hu' => 'Az AWS (Amazon Web Services) által generált, a kérések szerverre történő megfelelő elküldéséhez szükséges.',
                        'sv' => 'Genereras av AWS (Amazon Web Services), behövs för att skicka förfrågningarna till servern korrekt',
                    ],
                    'expiry' => [
                        'en' => '7 days',
                        'es' => '7 días',
                        'ag' => '7 días',
                        'cb' => '7 días',
                        'mx' => '7 días',
                        'fr' => '7 jours',
                        'qc' => '7 jours',
                        'pl' => '7 dni',
                        'ro' => '7 de zile',
                        'pt' => '7 dias',
                        'br' => '7 dias',
                        'sk' => '7 dní',
                        'nl' => '7 dagen',
                        'de' => '7 Tage',
                        'gr' => '7 μέρες',
                        'it' => '7 giorni',
                        'si' => '7 dni',
                        'da' => '7 dage',
                        'no' => '7 dager',
                        'cs' => '7 dager',
                        'hu' => '7 nap',
                        'sv' => '7 dagar',
                    ],
                ],
            ],
        ];

        if (isset($cookiesPlusCookieDefaultValues[$cookiesPlusFinality])) {
            return $cookiesPlusCookieDefaultValues[$cookiesPlusFinality];
        }

        return [];
    }

    public static function getCookiePurposeCallback($value)
    {
        return Tools::strlen($value) > 100 ? Tools::substr($value, 0, 100) . '...' : $value;
    }
}
